/*
	File:		AEHshTbl.cpp

	Contains:	Implementation of XMPAEHashTable class

	Written by:	Nick Pilch

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <2>	 3/15/94	MB		Changes to support SCpp/ASLM builds.
		<13>	  2/7/94	JA		Capitalized THROW.
		<12>	  2/7/94	JA		Tiger Team Makeover!
		<11>	 12/3/93	T		Stop including XMPError.h, it is included
									as ErrorDef.h inside Except.h
		<10>	11/30/93	JA		Rearranged #includes for new "frame"
									constant fix.
		 <9>	 11/2/93	NP		Cast kXMPNULL to proper type.
		 <8>	10/22/93	NP		Changed for new protocol-only throw for out
									of memory.
		 <7>	10/18/93	NP		Changed code name.
		 <6>	 7/21/93	VL		Added Exists method.
		 <5>	  7/2/93	NP		Mods to prevent shared memory problems.
		 <4>	 6/22/93	NP		Added iterator.
		 <3>	  6/2/93	NP		Changed throw for Find to return an OpenDoc
									code and not a system code.
		 <2>	 4/28/93	NP		File name changes.
		 <1>	 4/23/93	NP		first checked in

	To Do:
*/

#ifndef _AEHSHTBL_
#include "AEHshTbl.h"
#endif

#ifndef _EXCEPT_
#include "Except.h"
#endif

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef __AEOBJECTS__
#include <AEObjects.h>
#endif

#pragma segment XMPAEHashTable

//==============================================================================
// Implementation comments
//==============================================================================

/*
The error codes returned from Hash Manager functions are incompletely
documented.
Therefore, I make some pessimistic assumptions. If creating a new hash table or
any other routine that may need to allocate memory fails, I assume that it's
out of memory and throw kXMPErrOutOfMemory. Otherwise, I don't check error codes
explicitly.
*/

//==============================================================================
// XMPAEHashTable
//==============================================================================

ASLM_SCDECLARATION(XMPAEHashTable)

//------------------------------------------------------------------------------
// XMPAEHashTable::XMPAEHashTable
//------------------------------------------------------------------------------

XMPAEHashTable::XMPAEHashTable()
{
	fAEHashTable = kXMPNULL;
}

//------------------------------------------------------------------------------
// XMPAEHashTable::Initialize
//------------------------------------------------------------------------------

void XMPAEHashTable::Initialize(XMPULong numEntries, XMPUShort keySize,
								XMPUShort valueSize, XMPBoolean inSysHeap)
{
	OSErr error = NewHashTable(numEntries, keySize, valueSize,
								NULL, inSysHeap, &fAEHashTable);
	if (error || !fAEHashTable)
		THROW(kXMPErrOutOfMemory);
}

//------------------------------------------------------------------------------
// XMPAEHashTable::~XMPAEHashTable
//------------------------------------------------------------------------------

XMPAEHashTable::~XMPAEHashTable()
{
	if (fAEHashTable)
		DisposeHashTable(&fAEHashTable, NULL);
}

//------------------------------------------------------------------------------
// XMPAEHashTable::ReplaceEntry
//------------------------------------------------------------------------------

void XMPAEHashTable::ReplaceEntry(XMPKeyPtr key, XMPEntryPtr value)
{
	OSErr error = ::ReplaceEntry(fAEHashTable, NULL, key, value);
	if (error)
		THROW(kXMPErrOutOfMemory);
}

//------------------------------------------------------------------------------
// XMPAEHashTable::RemoveEntry
//------------------------------------------------------------------------------

void XMPAEHashTable::RemoveEntry(XMPKeyPtr key)
{
	// As far as I could tell, RemoveKeyEntry never returns an error.

	RemoveKeyEntry(fAEHashTable, NULL, key);
}

//------------------------------------------------------------------------------
// XMPAEHashTable::GetValue
//------------------------------------------------------------------------------

XMPBoolean XMPAEHashTable::GetValue(XMPKeyPtr key, XMPEntryPtr value)
{
	if (GetKeyValue(fAEHashTable, NULL, key, value) != noErr)
		return kXMPFalse;
	else
		return kXMPTrue;
}

//------------------------------------------------------------------------------
// XMPAEHashTable::Exists
//------------------------------------------------------------------------------

XMPBoolean XMPAEHashTable::Exists(XMPKeyPtr key)
{
	return CheckKey(fAEHashTable, (void*)kXMPNULL, key);
}

//------------------------------------------------------------------------------
// XMPAEHashTable::GetAEHashTable
//------------------------------------------------------------------------------

HashTable XMPAEHashTable::GetAEHashTable()
{
	return fAEHashTable;
}

//==============================================================================
// XMPAEHashTableIterator
//==============================================================================

//------------------------------------------------------------------------------
// XMPAEHashTableIterator::XMPAEHashTableIterator
//------------------------------------------------------------------------------

XMPAEHashTableIterator::XMPAEHashTableIterator(XMPAEHashTable* table)
{
	fTable = table;
	fIndex = 0;
	fDone = kXMPFalse;
}

//------------------------------------------------------------------------------
// XMPAEHashTableIterator::~XMPAEHashTableIterator
//------------------------------------------------------------------------------

XMPAEHashTableIterator::~XMPAEHashTableIterator()
{
}

//------------------------------------------------------------------------------
// XMPAEHashTableIterator::First
//------------------------------------------------------------------------------

void XMPAEHashTableIterator::First(XMPKeyPtr key, XMPEntryPtr value)
{
	if (!this->GetNext(key, value))
		fDone = kXMPTrue;
}

//------------------------------------------------------------------------------
// XMPAEHashTableIterator::Next
//------------------------------------------------------------------------------

void XMPAEHashTableIterator::Next(XMPKeyPtr key, XMPEntryPtr value)
{
	if (!this->GetNext(key, value))
		fDone = kXMPTrue;
}

//------------------------------------------------------------------------------
// XMPAEHashTableIterator::IsNotComplete
//------------------------------------------------------------------------------

XMPBoolean XMPAEHashTableIterator::IsNotComplete()
{
	return !fDone;
}

//------------------------------------------------------------------------------
// XMPAEHashTableIterator::GetNext
//
//	Returns true if another entry can be found, false otherwise.
//------------------------------------------------------------------------------

XMPBoolean XMPAEHashTableIterator::GetNext(XMPKeyPtr key, XMPEntryPtr value)
{
	OSErr	result = noErr;
	
	do
	{
		result = GetIndexedEntry(fTable->GetAEHashTable(),
									(void*)kXMPNULL, fIndex, key, value);
		++fIndex;
		if (result == noErr)
			return kXMPTrue;
	}
	while (result != kAEErrEndOfTable);

	return kXMPFalse;
}

