/*
	File:		AEHshTbl.h

	Contains:	Interface to AppleEvent hash table code and XMPAEHashTable
				class.

	Written by:	Nick Pilch

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<13>	  4/1/94	T		#1154941: Get OpenDoc source to actually
									compile for PowerPC
		<12>	 3/15/94	MB		Changes to support SCpp/ASLM builds.
		<11>	  2/7/94	JA		Tiger Team Makeover!
		<10>	10/22/93	NP		Changed for new protocol-only throw for out
									of memory.
		 <9>	 9/24/93	JA		Minor syntactic tweaks for THINK C++.
		 <8>	  8/9/93	NP		Added class id string.
		 <7>	 7/21/93	NP		Added a virtual keyword (temporary for
									ASLM).
		 <6>	 7/21/93	VL		Added Exists method.
		 <5>	  7/2/93	NP		Mods to prevent shared memory problems.
		 <4>	 6/22/93	NP		Added iterator.
		 <3>	  6/4/93	NP		Gave XMPAEHashTable a friend. (Aw)
		 <2>	 4/28/93	NP		File name changes.
		 <1>	 4/23/93	NP		first checked in

	To Do:
*/

#ifndef _AEHSHTBL_
#define _AEHSHTBL_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

//==============================================================================
// Theory of Operation
//==============================================================================

/*
	C Interface to some of the routines in HashTabl.A and class wrapper.

	The class simply provides an object-oriented wrapper to the AE hash table
	functions found in HashTable.A. It removes access to the MemHooks parameter. (MemHooks is set to NULL and SysHeap is set to false.)
	It hides the Table parameter since that is a member variable of the class.
*/

//==============================================================================
// Scalar Types
//==============================================================================

typedef	Handle HashTable;

//==============================================================================
// Error Codes
//==============================================================================

const XMPSShort kAEErrAlreadyExists	= -1722;
const XMPSShort kAEErrNotFound		= -1723;
const XMPSShort kAEErrEndOfTable	= -1724;

//==============================================================================
// Functions
//==============================================================================

pascal OSErr NewHashTable(XMPSLong NumEntries, XMPSShort KeySize, XMPSShort ValueSize,
							void* MemHooks, Boolean SysHeap, HashTable* Table);
pascal OSErr DisposeHashTable(HashTable* Table, void* MemHooks);
pascal OSErr ReplaceEntry(HashTable Table, void* MemHooks, KeyPtr Key,
							HEntryPtr Value);
pascal OSErr RemoveKeyEntry(HashTable Table, void* MemHooks, KeyPtr Key);
pascal OSErr GetKeyValue(HashTable Table, void* MemHooks, KeyPtr Key,
							HEntryPtr Value);
pascal OSErr GetKeyValue(HashTable Table, void* MemHooks, KeyPtr Key,
							HEntryPtr Value);
pascal OSErr GetIndexedEntry(HashTable Table, void* MemHooks, XMPSLong index,
								KeyPtr Key, HEntryPtr Value);
pascal Boolean CheckKey(HashTable Table, void* MemHooks, KeyPtr Key);

//==============================================================================
// Classes defined in this interface
//==============================================================================

class XMPAEHashTable;
class XMPAEHashTableIterator;

//==============================================================================
// XMPAEHashTable
//==============================================================================

#define kXMPAEHashTableID "appl:xmpaehashtable$class,1.0.0"

class XMPAEHashTable
{
	ASLM_COMPATIBLE
	
	friend class XMPAEHashTableIterator;

  public:

	XMPAEHashTable();
	XMPVMethod void Initialize(XMPULong numEntries, XMPUShort keySize,
								XMPUShort valueSize, XMPBoolean inSysHeap);
		// kXMPErrOutOfMemory is thrown if the table cannot be created.

	virtual ~ CDECL XMPAEHashTable();

	XMPVMethod void			ReplaceEntry(XMPKeyPtr key, XMPEntryPtr value);
		// Replace and/or add an entry. Pass a POINTER to the key as well as a
		//	a POINTER to the value to be added. kXMPErrOutOfMemory is thrown if
		//	the entry cannot be added.

	XMPVMethod void			RemoveEntry(XMPKeyPtr key);
		// Pass a POINTER to the key.

	XMPVMethod XMPBoolean	GetValue(XMPKeyPtr key, XMPEntryPtr value);
		// Pass a POINTER to the key as well as a POINTER to the value to be
		//	retrieved. kXMPTrue is returned if the key was found, kXMPFalse
		//	otherwise.
		
	XMPVMethod XMPBoolean	Exists(XMPKeyPtr key);
		// Check to see if a key exists. This function is not much faster than
		//	GetValue and should probably only be used when you do not intend to
		//	fetch the value immediately. kXMPTrue is returned if the key exists,
		//	kXMPFalse, otherwise.
	
  protected:

	HashTable	fAEHashTable;
	
	HashTable	GetAEHashTable();
};

//==============================================================================
// XMPAEHashTableIterator
//
//	This iterator is only meant to be used in the the context of a for loop,
//	e.g.:
//
//	XMPAEHashTableIterator iter;
//	for (iter.First(key, value); iter.IsNotComplete(); iter.Next(key, value))
//	{
//		...
//	}
//
//==============================================================================

#define kXMPAEHashTableIteratorID "appl:xmpaehashtableiterator$class,1.0.0"

class XMPAEHashTableIterator
{
  public:
	XMPAEHashTableIterator(XMPAEHashTable* table);
	~ CDECL XMPAEHashTableIterator();
	
	void			First(XMPKeyPtr key, XMPEntryPtr value);
	void			Next(XMPKeyPtr key, XMPEntryPtr value);
	XMPBoolean		IsNotComplete();
  private:
	XMPAEHashTable* fTable;
	XMPSLong		fIndex;
	XMPBoolean		fDone;
	XMPBoolean		GetNext(XMPKeyPtr key, XMPEntryPtr value);
};

#endif // _AEHSHTBL_