/*
	File:		DArray.cpp

	Contains:	Dynamic array class.

	Written by:	The Gang

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <3>	  3/1/94	JA		Use only XMP mem calls (1146233) and added
									include file (1147588)
		 <2>	 2/23/94	JA		Changed Toolbox memory allocation to
									official XMP allocators.
		 <4>	  2/7/94	JA		Capitalized THROW.
		 <3>	  2/7/94	JA		Tony the Tiger sez: Renamed "GetItem" to
									"Get" to avoid Menu Mgr conflicts.
		 <2>	 12/3/93	T		Stop including XMPError.h, it is included
									as ErrorDef.h inside Except.h
		 <1>	 4/29/93	VL		first checked in

	To Do:
*/


#ifndef _DARRAY_
#include "DArray.h"
#endif

#ifndef _EXCEPT_
#include "Except.h"
#endif

#ifndef _XMPMEM_
#include "XMPMem.h"
#endif

#ifndef __STDIO__
#include "StdIO.h"
#endif


// $$$$$ These errors should be renumbered and moved to ErrorDef.h!! --jpa
const XMPError	kXMPErrIndexOutOfRange = 100;
const XMPError	kXMPErrEntryNotFound = 101;
const XMPError	kXMPErrIncompatibleArrays = 102;
//const XMPError kXMPErrDuplicateKey = ??     $$$$$ ADD THIS TOO

#pragma segment DArray

//==============================================================================
// class DynamicArray
//==============================================================================

//------------------------------------------------------------------------------
// DynamicArray::DynamicArray
//------------------------------------------------------------------------------
DynamicArray::DynamicArray(XMPSLong maxEntryCount, XMPULong itemSize)
{
	fMaxEntries = maxEntryCount;
	fKeySize = sizeof(XMPULong);
	fItemSize = itemSize;
	fNumEntriesUsed = 0;
	fNumEntriesAllocated = 0;
	fArray = kXMPNULL;
}

//------------------------------------------------------------------------------
// DynamicArray::~DynamicArray
//------------------------------------------------------------------------------
DynamicArray::~DynamicArray()
{
	XMPDisposePtr(fArray);
}

//------------------------------------------------------------------------------
// DynamicArray::Initialize
//------------------------------------------------------------------------------
void DynamicArray::Initialize()
{
	if (fArray == kXMPNULL) {
		fArray = (XMPSByte*) XMPNewPtr(fMaxEntries * (fKeySize + fItemSize));
		fNumEntriesAllocated = fMaxEntries;
	}
}

//------------------------------------------------------------------------------
// DynamicArray::Add
//------------------------------------------------------------------------------
void DynamicArray::Add(XMPULong key, XMPSByte* item)
{
	XMPSLong	index;
	XMPULong	numMatched;
	
	if ( this->Find(key, &index, &numMatched) == kXMPTrue ) {
		THROW(kXMPErrEntryNotFound);	// $$$$$ Should be kXMPErrDuplicateKey
	}
	this->Allocate(index, 1);
	this->SetEntry(index, key, item);
}

//------------------------------------------------------------------------------
// DynamicArray::Remove
//------------------------------------------------------------------------------
void DynamicArray::Remove(XMPULong key)
{
	XMPSLong	index;
	XMPULong	numMatched;
	
	if (this->Find(key, &index, &numMatched) == kXMPFalse) {
		THROW(kXMPErrEntryNotFound);
	}
	this->Allocate(index, -1);
}

//------------------------------------------------------------------------------
// DynamicArray::Get
//------------------------------------------------------------------------------
XMPBoolean DynamicArray::Get(XMPULong key, XMPSByte* item)
{
	XMPSLong	index;
	XMPULong	numMatched;
	
	if (this->Find(key, &index, &numMatched) == kXMPTrue) {
		XMPBlockMove(this->EntryItemAddress(index), item, fItemSize);
		return kXMPTrue;
	}
	return kXMPFalse;
}

//------------------------------------------------------------------------------
// DynamicArray::SetEntry
//------------------------------------------------------------------------------
void DynamicArray::SetEntry(XMPSLong index, XMPULong key, XMPSByte* item)
{	
	XMPBlockMove(item, this->EntryItemAddress(index), fItemSize);
	XMPBlockMove(&key, this->EntryKeyAddress(index), fKeySize);
}

//------------------------------------------------------------------------------
// DynamicArray::EntryKeyAddress
//------------------------------------------------------------------------------
XMPSByte* DynamicArray::EntryKeyAddress(XMPSLong index)
{
	return (fArray + index * (fKeySize + fItemSize));
}

//------------------------------------------------------------------------------
// DynamicArray::EntryItemAddress
//------------------------------------------------------------------------------
XMPSByte* DynamicArray::EntryItemAddress(XMPSLong index)
{
	return ((fArray + index * (fKeySize + fItemSize)) + fKeySize);
}

//------------------------------------------------------------------------------
// DynamicArray::EntryKey
//------------------------------------------------------------------------------
XMPULong DynamicArray::EntryKey(XMPSLong index)
{
	return (*((XMPULong*) this->EntryKeyAddress(index)));
}

//------------------------------------------------------------------------------
// DynamicArray::EntryItem
//------------------------------------------------------------------------------
XMPSByte* DynamicArray::EntryItem(XMPSLong index)
{
	return this->EntryItemAddress(index);
}

//------------------------------------------------------------------------------
// DynamicArray::GetEntryCount
//------------------------------------------------------------------------------
XMPSLong DynamicArray::GetEntryCount()
{
	return fNumEntriesUsed;
}

//------------------------------------------------------------------------------
// DynamicArray::GetMaxEntryCount
//------------------------------------------------------------------------------
XMPSLong DynamicArray::GetMaxEntryCount()
{
	return fMaxEntries;
}

//------------------------------------------------------------------------------
// DynamicArray::Find
//------------------------------------------------------------------------------
XMPBoolean DynamicArray::Find(XMPULong key, XMPSLong* index, XMPULong* numMatched)
{
	XMPSLong 		r = fNumEntriesUsed - 1;
	XMPSLong 		l = 0;
	XMPSLong		tmp;
	XMPBoolean		found = false;
	XMPULong		tmpKey;
	
	*numMatched = 0;
	if (fNumEntriesUsed == 0) {
		*index = 0;
		return kXMPFalse;
	}
	do {
		tmp = (l + r) / 2;
		tmpKey = this->EntryKey(tmp);
		if (key < tmpKey)
			r = tmp - 1;
		else if (key > tmpKey)
			l = tmp + 1;
	} while ((key != tmpKey) && (l <= r));

#ifdef debug
	printf("tmp %d tmpKey %x l %d r %d\n", tmp, tmpKey, l, r);
#endif

	if (key == tmpKey) {
		*numMatched = 1;
		while ((tmp + 1 < fNumEntriesUsed) && (key == this->EntryKey(tmp + 1)))
			tmp++;
		while ((tmp > 0) && (key == this->EntryKey(tmp - 1))) {
			(*numMatched)++;
			--tmp;
		}
		found = true;
	}
	else {
		while ((tmp > 0) && (key < this->EntryKey(tmp)))
			--tmp;
		while ((tmp < fNumEntriesUsed) && (key > this->EntryKey(tmp)))
			tmp++;
	}
	*index = tmp;
#ifdef debug
	printf("index %d numMatched %d\n", *index, *numMatched);
#endif
	return found;
}

//------------------------------------------------------------------------------
// DynamicArray::Allocate
//
//
// 	Example: Begin with values  0  1  2  3  4
//			Allocate(0, -1) -> 1 2 3 4
//			Allocate(0, 1) -> ? 0 1 2 3 4
//			Allocate(1, 3) -> 0 ? ? ? 1 2 3 4
//			Allocate(1, -1) -> 0 2 3 4
//			Allocate(4, -1) -> 0 1 2 3
//------------------------------------------------------------------------------
void DynamicArray::Allocate(XMPSLong fromIndexInclusive, XMPSLong deltaIndex)
{
	if ((deltaIndex > 0) && (fNumEntriesUsed + deltaIndex > fNumEntriesAllocated))
		THROW(kXMPErrIndexOutOfRange);
	if (deltaIndex < 0) {
		fromIndexInclusive -= deltaIndex;
	}
	XMPBlockMove(this->EntryKeyAddress(fromIndexInclusive),
			this->EntryKeyAddress(fromIndexInclusive + deltaIndex),
			(fNumEntriesUsed - fromIndexInclusive) * (fKeySize + fItemSize));
	fNumEntriesUsed += deltaIndex;
}

//------------------------------------------------------------------------------
// DynamicArray::MoveEntries
//------------------------------------------------------------------------------
void DynamicArray::MoveEntriesTo(DynamicArray* array, XMPULong numEntries)
{
	XMPULong	i;
	
	if ((fKeySize != array->fKeySize) || (fItemSize != array->fItemSize))
		THROW(kXMPErrIncompatibleArrays);
	if (numEntries > fNumEntriesUsed)
		numEntries = fNumEntriesUsed;
			
	// $$$$$ Let's do it the slow way for the time being.
	// Obviously, the fast way is to do ONE XMPBlockMove.	
	for (i = 0; i < numEntries; i++) {
		array->Add(this->EntryKey(i), this->EntryItem(i));
	}
	this->Allocate(0, -numEntries);
}

//==============================================================================
// class DynamicArrayIterator
//==============================================================================

//------------------------------------------------------------------------------
// DynamicArrayIterator::DynamicArrayIterator
//------------------------------------------------------------------------------
DynamicArrayIterator::DynamicArrayIterator(DynamicArray* array)
{
	fDynamicArray = array;
	fIndex = 0;
	fEntry = kXMPNULL;
}

//------------------------------------------------------------------------------
// DynamicArrayIterator::~DynamicArrayIterator
//------------------------------------------------------------------------------
DynamicArrayIterator::~DynamicArrayIterator()
{
	delete fEntry;
}

//------------------------------------------------------------------------------
// DynamicArrayIterator::Initialize
//------------------------------------------------------------------------------
void DynamicArrayIterator::Initialize()
{
	fEntry = new DynamicArrayEntry(fDynamicArray->fItemSize);
}

//------------------------------------------------------------------------------
// DynamicArrayIterator::First
//------------------------------------------------------------------------------
DynamicArrayEntry* DynamicArrayIterator::First()
{
	fIndex = 0;
	
	if (fIndex < fDynamicArray->fNumEntriesUsed) {
		fEntry->Set(fDynamicArray->EntryKey(fIndex),
					fDynamicArray->EntryItemAddress(fIndex));
	}
	return fEntry;
}

//------------------------------------------------------------------------------
// DynamicArrayIterator::Next
//------------------------------------------------------------------------------
DynamicArrayEntry* DynamicArrayIterator::Next()
{
	fIndex++;
	
	if (fIndex < fDynamicArray->fNumEntriesUsed) {
		fEntry->Set(fDynamicArray->EntryKey(fIndex),
					fDynamicArray->EntryItemAddress(fIndex));
	}
	return fEntry;
}

//------------------------------------------------------------------------------
// DynamicArrayIterator::IsNotComplete
//------------------------------------------------------------------------------
XMPBoolean DynamicArrayIterator::IsNotComplete()
{
	return (fIndex < fDynamicArray->fNumEntriesUsed);
}

//==============================================================================
// class DynamicArrayEntry
//==============================================================================

//------------------------------------------------------------------------------
// DynamicArrayEntry::DynamicArrayEntry
//------------------------------------------------------------------------------
DynamicArrayEntry::DynamicArrayEntry(XMPULong size)
{
	fItemSize = size;
}

//------------------------------------------------------------------------------
// DynamicArrayEntry::~DynamicArrayEntry
//------------------------------------------------------------------------------
DynamicArrayEntry::~DynamicArrayEntry()
{
}

//------------------------------------------------------------------------------
// DynamicArrayEntry::GetKey
//------------------------------------------------------------------------------
XMPULong DynamicArrayEntry::GetKey()
{
	return fKey;
}

//------------------------------------------------------------------------------
// DynamicArrayEntry::Get
//------------------------------------------------------------------------------
void DynamicArrayEntry::Get(XMPPtr item)
{
	XMPBlockMove(fItemAddress, (XMPSByte*) item, fItemSize);
}

//------------------------------------------------------------------------------
// DynamicArrayEntry::Set
//------------------------------------------------------------------------------
void DynamicArrayEntry::Set(XMPULong key, XMPPtr item)
{
	fKey = key;
	fItemAddress = (XMPSByte*) item;
}

