/*
	File:		DictList.cpp

	Contains:	Implementation of DictionaryList

	Written by:	Richard Rodseth

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <2>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <3>	 11/4/93	RR		Moved class Association from .h. Fixed
									memory leak caused by DeleteValues removing
									links so that they weren't there anymore
									for RemoveAll to delete.
		 <2>	10/28/93	RR		Removed ambiguous constructor
		 <1>	 8/16/93	RCR		first checked in

	To Do:
*/

#ifndef _DICTLIST_
#include "DictList.h"
#endif

#pragma segment XMPCollections

//=====================================================================================
// Class Association
//=====================================================================================

class Association : public Link {
	public:
		Association(KeyType key, ValueType value);		
		virtual ~ CDECL Association();
		KeyType	GetKey()	{ return fKey;}
		void		SetKey(KeyType key)	{ fKey = key;}
		ValueType	GetValue()	{ return fValue;}
		void		SetValue(ValueType value)	{ fValue = value;}
	
	private:
		KeyType 		fKey;
		ValueType 		fValue;
};
	
Association::Association(KeyType key, ValueType value)
{
	fKey = key;
	fValue = value;
}

Association::~Association()
{
}

//======================================================================================
// Class DictionaryList
//======================================================================================

ASLM_SCDECLARATION(DictionaryList)

//------------------------------------------------------------------------------
// DictionaryList::DictionaryList
//
// Description
//------------------------------------------------------------------------------

DictionaryList::DictionaryList()
{
}

//------------------------------------------------------------------------------
// DictionaryList::~DictionaryList
//
// Description
//------------------------------------------------------------------------------

DictionaryList::~DictionaryList()
{
	this->RemoveAll();
}

//------------------------------------------------------------------------------
// DictionaryList::AddPair
//
// Description
//------------------------------------------------------------------------------

void DictionaryList::AddPair(KeyType key, ValueType value)
{
	XMPBoolean found = kXMPFalse;
	LinkedListIterator iter(&fImplementation);
	Association* association = (Association*) iter.First();
	while (association != kXMPNULL)
	{
		KeyType k = association->GetKey();

		if (this->KeysMatch(k,key))
		{
			found = kXMPTrue;
			association->SetValue(value);
			break;
		}
		else
			association = (Association*) iter.Next();
	}	
	if (!found)
	{
		association = this->CreateNewAssociation(key,value);
		fImplementation.AddLast(association);
	}
}

//------------------------------------------------------------------------------
// DictionaryList::Count
//
// Description
//------------------------------------------------------------------------------

XMPULong DictionaryList::Count()
{
	return fImplementation.Count();
}


//------------------------------------------------------------------------------
// DictionaryList::ContainsKey
//
// Description
//------------------------------------------------------------------------------

XMPBoolean DictionaryList::ContainsKey(KeyType key)
{
	LinkedListIterator iter(&fImplementation);
	Association* association = (Association*) iter.First();
	while (association != kXMPNULL)
	{
		KeyType k = association->GetKey();

		if (this->KeysMatch(k,key))
		{
			return kXMPTrue;	
		}
		else
			association = (Association*) iter.Next();
	}	
	return kXMPFalse;
}

//------------------------------------------------------------------------------
// DictionaryList::ValueAtKey
//
// Description
//------------------------------------------------------------------------------

ValueType DictionaryList::ValueAtKey(KeyType key)
{
	LinkedListIterator iter(&fImplementation);
	Association* association = (Association*) iter.First();
	while (association != kXMPNULL)
	{
		KeyType k = association->GetKey();

		if (this->KeysMatch(k,key))
		{
			return association->GetValue();	
		}
		else
			association = (Association*) iter.Next();
	}	
	return kXMPNULL;
}

//------------------------------------------------------------------------------
// DictionaryList::RemoveKey
//
// Description
//------------------------------------------------------------------------------

void DictionaryList::RemoveKey(KeyType key)
{
	LinkedListIterator iter(&fImplementation);
	Association* association = (Association*) iter.First();
	while (association != kXMPNULL)
	{
		KeyType k = association->GetKey();

		if (this->KeysMatch(k,key))
		{
			fImplementation.Remove(*association);
			break;
		}
		else
			association = (Association*) iter.Next();
	}	
}

//------------------------------------------------------------------------------
// DictionaryList::RemoveAll
//
// Description
//------------------------------------------------------------------------------

void DictionaryList::RemoveAll()
{
	Link* link = fImplementation.RemoveFirst();
	while (link != kXMPNULL)
	{
		delete link;
		link = fImplementation.RemoveFirst();
	}
}

//------------------------------------------------------------------------------
// DictionaryList::DeleteKeys
//
// Description
//------------------------------------------------------------------------------

void DictionaryList::DeleteKeys()
{
	LinkedListIterator iter(&fImplementation);
	
	for (Association* association = (Association*) iter.First();
		iter.IsNotComplete();
		association = (Association*) iter.Next())
	{
		KeyType key = association->GetKey();
		delete key; 
	}
}

//------------------------------------------------------------------------------
// DictionaryList::DeleteValues
//
// Description
//------------------------------------------------------------------------------

void DictionaryList::DeleteValues()
{
	LinkedListIterator iter(&fImplementation);
	
	for (Association* association = (Association*) iter.First();
		iter.IsNotComplete();
		association = (Association*) iter.Next())
	{
		ValueType value = association->GetValue();
		delete value; 
	}
}

//------------------------------------------------------------------------------
// DictionaryList::CreateNewAssociation
//
// Description
//------------------------------------------------------------------------------

Association*	DictionaryList::CreateNewAssociation(KeyType key, ValueType value) const
{
	return new Association(key, value);
}

//------------------------------------------------------------------------------
// DictionaryList::KeysMatch
//
// Description
//------------------------------------------------------------------------------

XMPBoolean	DictionaryList::KeysMatch(KeyType key1,KeyType key2) const
{
	return (key1 == key2);
}

