/*
	File:		List.cpp

	Contains:	Primitive linked list class

	Written by:	Richard Rodseth, Jens Alfke

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <2>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<11>	  2/7/94	JA		Capitalized THROW.
		<10>	  2/7/94	JA		Tiger Team Makeover. Added some inline
									helpers to eliminate cfront warnings in
									const methods.
		 <9>	 1/31/94	JA		Added more methods to Link and LinkedList.
									Inlined some getters/setters.
		 <7>	10/14/93	RR		Added fSeed and throw exception when
									iterator is out of sync
		 <6>	 8/13/93	RR		Fix up fSentinel in DeleteAllLinks
		 <5>	 8/13/93	RR		Added DeleteLinks method
		 <3>	 4/27/93	T		do some casting
		 <2>	  4/27/93	RCR		Include XMPTypes and use XMPBoolean and kXMPNULL
		 <1>	  4/27/93	RCR		first checked in
	To Do:
*/

#ifndef _LIST_
#include "List.h"
#endif

#ifndef _EXCEPT_
#include "Except.h"
#endif

#include <Errors.h>


#pragma segment XMPLinkedList

//==============================================================================
// Constants
//==============================================================================

//==============================================================================
// Scalar Types
//==============================================================================

//==============================================================================
// Local Classes
//==============================================================================

//==============================================================================
// Global Variables
//==============================================================================

//==============================================================================
// Function Prototype
//==============================================================================


//==============================================================================
// Link
//==============================================================================

ASLM_SCDECLARATION(Link)


// Many of the simple link methods are inlines; see List.h for implementations.


//------------------------------------------------------------------------------
// Link::Link
//
// Constructor for Link
//------------------------------------------------------------------------------

Link::Link()							
{ 
	fNext = kXMPNULL; 
	fPrevious = kXMPNULL;
}

//------------------------------------------------------------------------------
// Link::Link
//
// Constructor for Link
//------------------------------------------------------------------------------

Link::Link(Link* next, Link* previous)							
{ 
	fNext = next; 
	fPrevious = previous;
}

//------------------------------------------------------------------------------
// Link::Link
//
// Destructor for Link
//------------------------------------------------------------------------------

Link::~Link()											
{
}

//------------------------------------------------------------------------------
// Link::Remove
//
// Remove a link from its list (if any). DO NOT call this directly if there are
// any iterators active on the list; use LinkedList::Remove instead.
//------------------------------------------------------------------------------

void	Link::Remove( )
{
	if( fPrevious )
		fPrevious->SetNext(fNext);
	if( fNext )
		fNext->SetPrevious(fPrevious);
	fNext = kXMPNULL;
	fPrevious = kXMPNULL;
}

//------------------------------------------------------------------------------
// Link::AddBefore
//
// Add a link to a list before another link. It must not already be on any list.
// DO NOT call this directly if there are any iterators active on the list;
// use LinkedList::Remove instead.
//------------------------------------------------------------------------------

void	Link::AddBefore( Link *link )
{
	ASSERT(link!=kXMPNULL,paramErr);
	WASSERT(fNext==kXMPNULL);
	WASSERT(fPrevious==kXMPNULL);
	fNext = link;
	fPrevious = link->GetPrevious();
	fPrevious->SetNext(this);
	fNext->SetPrevious(this);
}

//------------------------------------------------------------------------------
// Link::AddAfter
//
// Add a link to a list after another link. It must not already be on any list.
// DO NOT call this directly if there are any iterators active on the list;
// use LinkedList::Remove instead.
//------------------------------------------------------------------------------

void	Link::AddAfter( Link *link )
{
	ASSERT(link!=kXMPNULL,paramErr);
	WASSERT(fNext==kXMPNULL);
	WASSERT(fPrevious==kXMPNULL);
	fPrevious = link;
	fNext = link->GetNext();
	fPrevious->SetNext(this);
	fNext->SetPrevious(this);
}

//======================================================================================
// Class LinkedList
//======================================================================================

//------------------------------------------------------------------------------
// LinkedList::LinkedList
//
// Constructor for LinkedList
//------------------------------------------------------------------------------

LinkedList::LinkedList()
	:fSentinel(&fSentinel,&fSentinel),
	 fSeed(0)
{
}

//------------------------------------------------------------------------------
// LinkedList::~LinkedList
//
// Destructor for LinkedList
//------------------------------------------------------------------------------

LinkedList::~LinkedList()
{
	// The list does NOT delete all its links!
}

//------------------------------------------------------------------------------
// LinkedList::IsEmpty
//
// Description
//------------------------------------------------------------------------------

XMPBoolean LinkedList::IsEmpty() const
{
	return this->IsSentinel( fSentinel.GetNext() );
}

//------------------------------------------------------------------------------
// LinkedList::Count
//
// Description
//------------------------------------------------------------------------------

XMPULong LinkedList::Count() const
{
	Link* l = fSentinel.GetNext();
	XMPULong count = 0;
	while (this->NotSentinel(l))
	{
		count++;
		l = l->GetNext();
	}
	return count;
}

//------------------------------------------------------------------------------
// LinkedList::Includes
//
// Does the list include a particular link?
//------------------------------------------------------------------------------

XMPBoolean LinkedList::Includes( const Link *link ) const
{
	Link* l = fSentinel.GetNext();
	while (this->NotSentinel(l))
	{
		if( l == link )
			return kXMPTrue;
		else
			l = l->GetNext();
	}
	return kXMPFalse;
}

//------------------------------------------------------------------------------
// LinkedList::DeleteAllLinks
//
// Description
//------------------------------------------------------------------------------

void	LinkedList::DeleteAllLinks()
{

	Link* l = fSentinel.GetNext();
	Link* n = l;
	while (this->NotSentinel(n))
	{
		n = l->GetNext();
		delete l;	
		l = n;
	}
	fSentinel.SetNext(&fSentinel);
	fSentinel.SetPrevious(&fSentinel);
	fSeed++;
}

//------------------------------------------------------------------------------
// LinkedList::RemoveAll
//
// Description
//------------------------------------------------------------------------------

void	LinkedList::RemoveAll()
{

	Link* l = fSentinel.GetNext();
	Link* n = l;
	while (this->NotSentinel(l))
	{
		n = l->GetNext();
		l->SetNext(kXMPNULL);
		l->SetPrevious(kXMPNULL);
		l = n;
	}
	fSentinel.SetNext(&fSentinel);
	fSentinel.SetPrevious(&fSentinel);
}

//------------------------------------------------------------------------------
// LinkedList::Remove
//
// Description
//------------------------------------------------------------------------------

void	LinkedList::Remove(Link& aLink)
{
	fSeed++;
	aLink.Remove();
}

//------------------------------------------------------------------------------
// LinkedList::RemoveFirst
//
// Description
//------------------------------------------------------------------------------

Link* LinkedList::RemoveFirst()
{
	Link* old = fSentinel.GetNext();
	if (this->NotSentinel(old))
	{
		fSeed++;
		old->Remove();
		return old;
	}
	else
	{
		return kXMPNULL;
	}
}

//------------------------------------------------------------------------------
// LinkedList::RemoveLast
//
// Description
//------------------------------------------------------------------------------

Link* LinkedList::RemoveLast()
{
	Link* old = fSentinel.GetPrevious();
	if (this->NotSentinel(old))
	{
		fSeed++;
		old->Remove();
		return old;
	}
	else
	{
		return kXMPNULL;
	}
}

//------------------------------------------------------------------------------
// LinkedList::AddFirst
//
// Description
//------------------------------------------------------------------------------

void	LinkedList::AddFirst(Link* link)
{
	link->AddAfter(this->GetSentinel());
	fSeed++;
}

//------------------------------------------------------------------------------
// LinkedList::AddLast( Link )
//
// Description
//------------------------------------------------------------------------------

void	LinkedList::AddLast(Link* link)
{
	link->AddBefore(this->GetSentinel());
	fSeed++;
}

//------------------------------------------------------------------------------
// LinkedList::AddLast( LinkedList )
//
// Append contents of another list to my end. (Other list becomes empty.)
//------------------------------------------------------------------------------

void	LinkedList::AddLast( LinkedList &list )
{
	if( !list.IsEmpty() ) {
		Link *myLast = fSentinel.GetPrevious();
		Link *itsFirst = list.First();
		myLast->SetNext(itsFirst);
		itsFirst->SetPrevious(myLast);
		
		Link *itsLast = list.Last();
		itsLast->SetNext(this->GetSentinel());
		fSentinel.SetPrevious(itsLast);
		
		list.fSentinel.SetNext(this->GetSentinel());
		list.fSentinel.SetPrevious(this->GetSentinel());

		fSeed++;
		list.fSeed++;
	}
}

//------------------------------------------------------------------------------
// LinkedList::AddLastUnique( LinkedList )
//
// Append contents of another list (w/o duplication) to my end.
//------------------------------------------------------------------------------

void	LinkedList::AddLastUnique( LinkedList &list )
{
	for( Link *link = fSentinel.GetNext(); this->NotSentinel(link); link=link->GetNext() )
		if( list.Includes(link) )
			list.Remove(*link);
	this->AddLast(list);
}

//------------------------------------------------------------------------------
// LinkedList::AddBefore
//
// Description
//------------------------------------------------------------------------------

void LinkedList::AddBefore(Link& existing, Link* link)
{
	link->AddBefore(&existing);
	fSeed++;
}

//------------------------------------------------------------------------------
// LinkedList::AddAfter
//
// Description
//------------------------------------------------------------------------------

void LinkedList::AddAfter(Link& existing, Link* link)
{
	link->AddAfter(&existing);
	fSeed++;
}

//------------------------------------------------------------------------------
// LinkedList::After
//
// Description
//------------------------------------------------------------------------------

Link* LinkedList::After(const Link& link) const
{
	Link *next = link.GetNext();
	if( this->IsSentinel(next) )
		return kXMPNULL;
	else
		return next;
}

//------------------------------------------------------------------------------
// LinkedList::Before
//
// Description
//------------------------------------------------------------------------------

Link* LinkedList::Before(const Link& link) const
{
	Link *prev = link.GetPrevious();
	if( this->IsSentinel(prev) )
		return kXMPNULL;
	else
		return prev;
}

//------------------------------------------------------------------------------
// LinkedList::First
//
// Description
//------------------------------------------------------------------------------

Link*	LinkedList::First()  const
{
	return this->NotSentinel(fSentinel.GetNext()) ? fSentinel.GetNext() : (Link*) kXMPNULL;
}

//------------------------------------------------------------------------------
// LinkedList::Last
//
// Description
//------------------------------------------------------------------------------

Link* LinkedList::Last() const
{
	return this->NotSentinel(fSentinel.GetPrevious()) ? fSentinel.GetPrevious() : (Link*) kXMPNULL;
}


//======================================================================================
// Class LinkedListIterator
//======================================================================================

//------------------------------------------------------------------------------
// LinkedListIterator::LinkedListIterator
//
// Constructor for LinkedListIterator
//------------------------------------------------------------------------------

LinkedListIterator::LinkedListIterator(LinkedList* list)
{
	fList = list;
	fCurrent = kXMPNULL;
	fSentinel = &list->fSentinel;
	fSeed = fList->fSeed;	
}

//------------------------------------------------------------------------------
// LinkedListIterator::~LinkedListIterator
//
// Destructor for LinkedListIterator
//------------------------------------------------------------------------------

LinkedListIterator::~LinkedListIterator()
{
}

//------------------------------------------------------------------------------
// LinkedListIterator::First
//
// Description
//------------------------------------------------------------------------------

Link* LinkedListIterator::First()
{
	if (fSeed != fList->fSeed)
		THROW(kXMPErrIteratorOutOfSync);
		
	fCurrent = fList->First();
	if (fCurrent == fSentinel)
		fCurrent = kXMPNULL;
	return fCurrent;
}

//------------------------------------------------------------------------------
// LinkedListIterator::Next
//
// Description
//------------------------------------------------------------------------------

Link* LinkedListIterator::Next()
{
	if (fSeed != fList->fSeed)
		THROW(kXMPErrIteratorOutOfSync);

	if (fCurrent)
		fCurrent = fCurrent->GetNext();
	if (fCurrent == fSentinel)
		fCurrent = kXMPNULL;
	return fCurrent;
}

//------------------------------------------------------------------------------
// LinkedListIterator::Last
//
// Description
//------------------------------------------------------------------------------

Link* LinkedListIterator::Last()
{
	if (fSeed != fList->fSeed)
		THROW(kXMPErrIteratorOutOfSync);

	fCurrent = fList->Last();
	if (fCurrent == fSentinel)
		fCurrent = kXMPNULL;
	return fCurrent;
}

//------------------------------------------------------------------------------
// LinkedListIterator::Previous
//
// Description
//------------------------------------------------------------------------------

Link* LinkedListIterator::Previous()
{
	if (fSeed != fList->fSeed)
		THROW(kXMPErrIteratorOutOfSync);

	if (fCurrent)
		fCurrent = fCurrent->GetPrevious();
	if (fCurrent == fSentinel)
		fCurrent = kXMPNULL;
	return fCurrent;
}

//------------------------------------------------------------------------------
// LinkedListIterator::Current
//
// Description
//------------------------------------------------------------------------------

Link* LinkedListIterator::Current()
{
	return fCurrent;
}

//------------------------------------------------------------------------------
// LinkedListIterator::IsNotComplete
//
// Description
//------------------------------------------------------------------------------

XMPBoolean LinkedListIterator::IsNotComplete()
{
	return (fCurrent != kXMPNULL);
}
