/*
	File:		XMPLinkedList.h

	Contains:	Primitive linked list class

	Written by:	Richard Rodseth

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<10>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <9>	  2/7/94	JA		Tiger Team Makeover. Added some inline
									helpers to eliminate cfront warnings in
									const methods.
		 <8>	 1/31/94	JA		Added more methods to Link and LinkedList.
									Inlined some getters/setters.
		 <7>	10/14/93	RR		Added fSeed to detect when iterator is out of sync
		 <6>	 10/6/93	PH		Make destructor virtual
		 <5>	 8/13/93	RR		Added DeleteLinks method
		 <5>	 8/13/93	RR		Added DeleteLinks method
		 <4>	  8/9/93	NP		Added class id string.
		 <3>	  4/27/93	RCR		Rename Includes
		 <2>	  4/27/93	RCR		Include XMPTypes and use XMPBoolean and kXMPNULL
		 <1>	  4/27/93	RCR		first checked in
	To Do:
*/

#ifndef _LIST_
#define _LIST_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif


//=====================================================================================
// Theory of Operation
//=====================================================================================

/*

 Note: These classes are private to the implementation. They are not available
 to part handlers.

 Note: These are primitive classes for implementing higher-level collections
 For example, to create a FrameList class, subclass Link to add a field to
 store the frame. The FrameList class would use a LinkedList in its implementation
 and would manufacture the Link objects internally.
 
 Note: SimpleLinkedList is a list without a "seed" for consistency checking.
 It cannot be iterated over.

*/

//==============================================================================
// Constants
//==============================================================================

//==============================================================================
// Scalar Types
//==============================================================================

//==============================================================================
// Classes defined in this interface
//==============================================================================

class Link;
class LinkedList;
class LinkedListIterator;

//==============================================================================
// Classes used by this interface
//==============================================================================


//==============================================================================
// Link
//==============================================================================

#define kLinkID "appl:link$class,1.0.0"

class  Link {
	ASLM_COMPATIBLE

	public:
	
		Link();
		
		Link(Link* next, Link* previous);
		
		virtual ~ CDECL Link();
		
		Link* GetNext() const					{return fNext;}
		
		Link* GetPrevious() const				{return fPrevious;}
	
	// The following operations are provided for efficiency, but DO NOT USE THEM
	// if there are any iterators active on a list. These operations don't bump
	// the list's fSeed and the iterators will not be able to detect that they
	// are out of sync!
		
		void  Remove( );
		
		void  AddBefore( Link *aLink );
		
		void  AddAfter( Link *aLink );
	
  //private-by-convention:
  		
		void  SetNext(Link* aLink)				{fNext = aLink;}
		
		void  SetPrevious(Link* aLink)			{fPrevious = aLink;}

	private:
	
		Link*		fNext;
		Link*		fPrevious;
};


//==============================================================================
// LinkedList
//==============================================================================

#define kLinkedListID "appl:linkedlist$class,1.0.0"

class LinkedList {

	public:
	
	  	LinkedList();
	  	
	  	~ CDECL LinkedList();
	  	
	  	XMPBoolean		IsEmpty( )										const;
	  	
	  	XMPULong		Count()											const;
	  	
	  	XMPBoolean		Includes( const Link* )							const;
	  	
	  	void			Remove(Link&);
	  	
	  	void			RemoveAll();

	  	void			DeleteAllLinks();
	  	
	  	Link*			RemoveFirst();
	  	
	  	Link*			RemoveLast();
	  	
	  	void			AddBefore(Link& existing, Link* link);
	  	
	  	void			AddAfter(Link& existing, Link* link);
	  	
	  	void			AddFirst(Link* link);
	  	
	  	void			AddLast(Link* link);
	  	
	  	void			AddLast( LinkedList &list );
	  	
	  	void			AddLastUnique( LinkedList &list );
	  	
	  	Link*			After(const Link& link)							const;
	  	
	  	Link* 			Before(const Link& link)						const;
	  	
	  	Link*			First()											const;
	  	
	  	Link* 			Last()											const;

protected:

		Link			fSentinel;	// Marks the head & tail
		XMPULong		fSeed;		// Used to detect out-of-sync iterators
		
		Link*			GetSentinel( )
												{return &fSentinel;}
		const Link*		GetSentinel( )									const
												{return &fSentinel;}
		XMPBoolean		IsSentinel( const Link* link )					const
												{return link==this->GetSentinel();}
		XMPBoolean		NotSentinel( const Link* link )					const
												{return link!=this->GetSentinel();}

private:			  	
		friend class LinkedListIterator;
};


//=====================================================================================
// LinkedListIterator
//=====================================================================================

#define kLinkedListIteratorID "appl:linkedlistiterator$class,1.0.0"

class LinkedListIterator {

	public:
	
		LinkedListIterator(LinkedList*	list);
		
		~ CDECL LinkedListIterator();
		
		Link*			First();
		
		Link*			Next();
		
		Link*			Last();
		
		Link*			Previous();

		Link*			Current();

		XMPBoolean 		IsNotComplete();
		
	private:
	
		LinkedList*	fList;
		Link*					fCurrent;
		Link*					fSentinel;
		XMPULong		fSeed;		// Used to detect out-of-sync iterators
		
};


#endif // _LIST_