/*
	File:		Node.cpp

	Contains:	Implementation of Node class

	Written by:	Richard Rodseth

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <2>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <6>	  2/7/94	JA		Tiger Team Makeover!
		 <5>	 1/31/94	JA		Syntactic tweaks to deal with changes in
									Link and LinkedList API.
		 <4>	11/12/93	JBS		fix SkipChildren()
		 <3>	  9/3/93	JBS		added SkipChildren()
		 <2>	 7/21/93	NP		Added outline destructor to Node for ASLM.
		 <1>	 1/26/93	RCR		first checked in

	To Do:
		Optimization: Many methods could be inlined for speed & code size improvements.
*/

#ifndef _NODE_
#include "Node.h"
#endif

#ifndef _EXCEPT_
#include "Except.h"
#endif

#pragma segment XMPNode

//======================================================================================
// Class Node
//======================================================================================

//------------------------------------------------------------------------------
// Node::Node
//
// Description
//------------------------------------------------------------------------------

Node::Node() 
{
	fParent = kXMPNULL; 
}

//------------------------------------------------------------------------------
// Node::~Node
//
// Description
//------------------------------------------------------------------------------

Node::~Node() 
{
}

//------------------------------------------------------------------------------
// Node::Node
//
// Description
//------------------------------------------------------------------------------

XMPULong Node::Size() 
{
	XMPULong size = 0;
	
	Node* node = this->FirstTopDown();
	while (node)
	{
		size++;
		node = node->NextTopDown(kXMPFrontToBack);
	}
	return size;
}

//------------------------------------------------------------------------------
// Node::Node
//
// Description
//------------------------------------------------------------------------------

Node* Node::GetParent()
{
	return fParent;
}

//------------------------------------------------------------------------------
// Node::Node
//
// Description
//------------------------------------------------------------------------------

Node* Node::GetFirstChild()
{
	return (Node*) this->First();
}

//------------------------------------------------------------------------------
// Node::Node
//
// Description
//------------------------------------------------------------------------------

Node* Node::GetLastChild()
{
	return (Node*) this->Last();
}

//------------------------------------------------------------------------------
// Node::Node
//
// Description
//------------------------------------------------------------------------------

Node* Node::GetNextSibling()
{
	Node* parent = this->GetParent();
	return parent ? parent->GetChildAfter(this) : kXMPNULL;
}

//------------------------------------------------------------------------------
// Node::Node
//
// Description
//------------------------------------------------------------------------------

Node* Node::GetChildAfter(Node* node)
{
	return (Node*) this->After(*node);
}

//------------------------------------------------------------------------------
// Node::Node
//
// Description
//------------------------------------------------------------------------------

Node* Node::GetChildBefore(Node* node)
{
	return (Node*) this->Before(*node);
}

//------------------------------------------------------------------------------
// Node::Node
//
// Description
//------------------------------------------------------------------------------

Node* Node::GetPreviousSibling()
{
	Node* parent = this->GetParent();
	return parent ? parent->GetChildBefore(this) : kXMPNULL;
}

//------------------------------------------------------------------------------
// Node::Node
//
// Description
//------------------------------------------------------------------------------

void Node::SetParent(Node* parent)
{
	fParent = parent;
}

//------------------------------------------------------------------------------
// Node::Node
//
// Description
//------------------------------------------------------------------------------

void Node::AddChildFirst(Node* node)
{
	node->SetParent(this);
	this->AddFirst(node);
}

//------------------------------------------------------------------------------
// Node::Node
//
// Description
//------------------------------------------------------------------------------

void Node::AddChildLast(Node* node)
{
	node->SetParent(this);
	this->AddLast(node);
}

//------------------------------------------------------------------------------
// Node::Node
//
// Description
//------------------------------------------------------------------------------

void Node::AddChildBefore(Node& existing, Node* node)
{
	node->SetParent(this);
	this->LinkedList::AddBefore(existing, node);
}

//------------------------------------------------------------------------------
// Node::Node
//
// Description
//------------------------------------------------------------------------------

void Node::AddChildAfter(Node& existing, Node* node)
{
	node->SetParent(this);
	this->LinkedList::AddAfter(existing, node);
}

//------------------------------------------------------------------------------
// Node::Node
//
// Description
//------------------------------------------------------------------------------

void Node::RemoveChild(Node& child)
{
	this->LinkedList::Remove(child);
}

//------------------------------------------------------------------------------
// Node::Node
//
// Description
//------------------------------------------------------------------------------

Node* Node::FirstTopDown()
{
	return this;
}

//------------------------------------------------------------------------------
// Node::Node
//
// Description
//------------------------------------------------------------------------------

Node* Node::NextTopDown(XMPSiblingOrder siblingOrder)
{
	Node* child;
	if (siblingOrder == kXMPFrontToBack)
		child = this->GetFirstChild();
	else
		child = this->GetLastChild();
	
	if (child)
		return child;
	else 
	{
		Node* sibling;
		if (siblingOrder == kXMPFrontToBack)
			sibling = this->GetNextSibling();
		else 
			sibling = this->GetPreviousSibling();

		if (sibling) 
			return  sibling;
		else 
			return this->GetNextAunt(siblingOrder);
	}
	return kXMPNULL;
}

//------------------------------------------------------------------------------
// Node::Node
//
// Description
//------------------------------------------------------------------------------

Node* Node::GetNextAunt(XMPSiblingOrder siblingOrder)
{
	Node* parent = this->GetParent();
	
	if (parent)
	{
		Node* sibling;
		if (siblingOrder == kXMPFrontToBack)
			sibling = parent->GetNextSibling();
		else
			sibling = parent->GetPreviousSibling();
		
		if (sibling) 
			return sibling;
		else 
			return parent->GetNextAunt(siblingOrder);
	}
	else
		return kXMPNULL;
}

//------------------------------------------------------------------------------
// Node::Node
//
// Description
//------------------------------------------------------------------------------

Node* Node::FirstBottomUp(XMPSiblingOrder siblingOrder)
{
	Node* child;
	if (siblingOrder == kXMPFrontToBack)
		child = this->GetFirstChild();
	else
		child = this->GetLastChild();

	if (child)
		return child->FirstBottomUp(siblingOrder);
	else
		return this;
}

//------------------------------------------------------------------------------
// Node::Node
//
// Description
//------------------------------------------------------------------------------

Node* Node::NextBottomUp(XMPSiblingOrder siblingOrder)
{
	Node* parent = this->GetParent();

	if (parent)
	{
		Node* sibling;
		if (siblingOrder == kXMPFrontToBack)
			sibling = parent->GetChildAfter(this);
		else
			sibling = parent->GetChildBefore(this);
		
		if (sibling)
			return sibling->FirstBottomUp(siblingOrder);
		else
			return parent;
	}
	else
		return kXMPNULL;	

	return kXMPNULL;
}

//======================================================================================
// NodeTraverser
//======================================================================================

//------------------------------------------------------------------------------
// NodeTraverser::NodeTraverser
//
// Description
//------------------------------------------------------------------------------

NodeTraverser::NodeTraverser(Node* root, 
					XMPTraversalType traversalType, 
					XMPSiblingOrder siblingOrder)
{
	fRoot = root;
	fCurrent = kXMPNULL;
	fTraversalType = traversalType;
	fSiblingOrder = siblingOrder;
}

//------------------------------------------------------------------------------
// NodeTraverser::NodeTraverser
//
// Description
//------------------------------------------------------------------------------

NodeTraverser::~NodeTraverser()
{
}

//------------------------------------------------------------------------------
// NodeTraverser::NodeTraverser
//
// Description
//------------------------------------------------------------------------------

Node*	NodeTraverser::First()
{
	if (fRoot)
	{
		if (fTraversalType == kXMPTopDown)
			fCurrent = this->FirstTopDown(fRoot);
		else if (fTraversalType == kXMPBottomUp)
			fCurrent = this->FirstBottomUp(fRoot,fSiblingOrder);
		else if (fTraversalType == kXMPChildrenOnly)
		{
			if (fSiblingOrder == kXMPFrontToBack)
				fCurrent = fRoot->GetFirstChild();
			else
				fCurrent = fRoot->GetLastChild();
		}
		return fCurrent;
	}
	else
		return kXMPNULL;
}

//------------------------------------------------------------------------------
// NodeTraverser::Next
//
// Description
//------------------------------------------------------------------------------

Node*	NodeTraverser::Next()
{
	if (fCurrent)
	{
		if (fTraversalType == kXMPTopDown)
			fCurrent = this->NextTopDown(fCurrent, fSiblingOrder);
		else if (fTraversalType == kXMPBottomUp)
			fCurrent = this->NextBottomUp(fCurrent, fSiblingOrder);
		else if (fTraversalType == kXMPChildrenOnly)
		{
			if (fSiblingOrder == kXMPFrontToBack)
				fCurrent = fCurrent->GetNextSibling();
			else
				fCurrent = fCurrent->GetPreviousSibling();
		}
		return fCurrent;
	}
	else
		return kXMPNULL;
}

//------------------------------------------------------------------------------
// NodeTraverser::SkipChildren
//
// Description
//------------------------------------------------------------------------------

void NodeTraverser::SkipChildren()
{
	Node* next = kXMPNULL;
	Node* skipTo = kXMPNULL;
	Node* test = kXMPNULL;

	if (fCurrent)
	{
		if (fTraversalType == kXMPTopDown)
		{
			if (fSiblingOrder == kXMPFrontToBack)
				next = fCurrent->GetNextSibling();
			else
				next = fCurrent->GetPreviousSibling();

			if (!next)
				next = fCurrent->GetNextAunt(fSiblingOrder);

			test = fCurrent;
			while ( test != next )
			{
				WASSERTM(test != kXMPNULL, "NodeTraverser::SkipChildren() ran off end of tree.");
				skipTo = test;
				test = this->NextTopDown(test, fSiblingOrder);
			}

			fCurrent = skipTo;
		}
	}
}

//------------------------------------------------------------------------------
// NodeTraverser::IsNotComplete
//
// Description
//------------------------------------------------------------------------------

XMPBoolean	NodeTraverser::IsNotComplete()
{
	return (fCurrent != kXMPNULL);
}

//------------------------------------------------------------------------------
// NodeTraverser::FirstTopDown
//
// Description
//------------------------------------------------------------------------------

Node* NodeTraverser::FirstTopDown(Node* node)
{
	return node;
}

//------------------------------------------------------------------------------
// NodeTraverser::NextTopDown
//
// Description
//------------------------------------------------------------------------------

Node* NodeTraverser::NextTopDown(Node* node, XMPSiblingOrder siblingOrder)
{
	Node* child;
	if (siblingOrder == kXMPFrontToBack)
		child = node->GetFirstChild();
	else
		child = node->GetLastChild();
	
	if (child)
		return child;
	else if (node != fRoot)
	{
		Node* sibling;
		if (siblingOrder == kXMPFrontToBack)
			sibling = node->GetNextSibling();
		else 
			sibling = node->GetPreviousSibling();

		if (sibling) 
			return  sibling;
		else 
			return this->GetNextAunt(node, siblingOrder);
	}
	return kXMPNULL;
}

//------------------------------------------------------------------------------
// NodeTraverser::GetNextAunt
//
// Description
//------------------------------------------------------------------------------

Node* NodeTraverser::GetNextAunt(Node* node, XMPSiblingOrder siblingOrder)
{
	Node* parent = node->GetParent();
	
	if (parent && (parent != fRoot))
	{
		Node* sibling;
		if (siblingOrder == kXMPFrontToBack)
			sibling = parent->GetNextSibling();
		else
			sibling = parent->GetPreviousSibling();
		
		if (sibling) 
			return sibling;
		else 
			return this->GetNextAunt(parent, siblingOrder);
	}
	else
		return kXMPNULL;
}

//------------------------------------------------------------------------------
// NodeTraverser::FirstBottomUp
//
// Description
//------------------------------------------------------------------------------

Node* NodeTraverser::FirstBottomUp(Node* node, XMPSiblingOrder siblingOrder)
{
	Node* child;
	if (siblingOrder == kXMPFrontToBack)
		child = node->GetFirstChild();
	else
		child = node->GetLastChild();

	if (child)
		return this->FirstBottomUp(child, siblingOrder);
	else
		return node;
}

//------------------------------------------------------------------------------
// Node::Node
//
// Description
//------------------------------------------------------------------------------

Node* NodeTraverser::NextBottomUp(Node* node, XMPSiblingOrder siblingOrder)
{
	if (node == fRoot)
		return kXMPNULL;
		
	Node* parent = node->GetParent();

	if (parent)
	{
		Node* sibling;
		if (siblingOrder == kXMPFrontToBack)
			sibling = parent->GetChildAfter(node);
		else
			sibling = parent->GetChildBefore(node);
		
		if (sibling)
			return this->FirstBottomUp(sibling, siblingOrder);
		else
			return parent;
	}
	else
		return kXMPNULL;	

	return kXMPNULL;
}

