/*
	File:		OrdColl.cpp

	Contains:	Implementation of OrderedCollection and OrderedCollectionIterator

	Written by:	Richard Rodseth

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <3>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <2>	 2/15/94	CG		#1143680 - OrdColls now allocated on
									specified heap.
		 <8>	  2/7/94	JA		Tiger Team Makeover!
		 <7>	  2/2/94	NP		Added support for allocating internal
									structures in given heap.
		 <6>	 1/19/94	NP		Fixed Remove to delete the link.
		 <5>	11/19/93	NP		Moved definition of ValueLink to OrdColl.h
		 <4>	10/28/93	RR		Removed ambiguous constructor
		 <3>	 8/13/93	CG		Added casting required by CFront.
		 <1>	 8/13/93	RCR		first checked in

	To Do:
*/

#ifndef _ORDCOLL_
#include "OrdColl.h"
#endif

#pragma segment XMPCollections

//======================================================================================
// Class ValueLink - Implementation
//======================================================================================

ASLM_SCDECLARATION(ValueLink)

ValueLink::ValueLink(ElementType value)
{
	fValue = value;
}

ValueLink::~ValueLink()
{
}

//======================================================================================
// Class OrderedCollection
//======================================================================================

ASLM_SCDECLARATION(OrderedCollection)

//------------------------------------------------------------------------------
// OrderedCollection::OrderedCollection
//------------------------------------------------------------------------------

OrderedCollection::OrderedCollection()
{
	fHeap = kXMPNULL;
}

//------------------------------------------------------------------------------
// OrderedCollection::OrderedCollection
//------------------------------------------------------------------------------

OrderedCollection::OrderedCollection(XMPHeap where)
{
	fHeap = where;
}

// OrderedCollection::~OrderedCollection
//------------------------------------------------------------------------------

OrderedCollection::~OrderedCollection()
{
	this->RemoveAll();
}

//------------------------------------------------------------------------------
// OrderedCollection::AddFirst
//------------------------------------------------------------------------------

void OrderedCollection::AddFirst(ElementType element)
{
	ValueLink* newLink = this->CreateNewLink(element);
	fImplementation.AddFirst(newLink);
}

//------------------------------------------------------------------------------
// OrderedCollection::AddLast
//------------------------------------------------------------------------------

void OrderedCollection::AddLast(ElementType element)
{
	ValueLink* newLink = CreateNewLink(element);
	fImplementation.AddLast(newLink);
}


//------------------------------------------------------------------------------
// OrderedCollection::AddBefore
//------------------------------------------------------------------------------

void OrderedCollection::AddBefore(ElementType existing, ElementType tobeadded)
{
	LinkedListIterator iter(&fImplementation);
	ValueLink* aLink = (ValueLink*) iter.First();
	while (aLink != kXMPNULL)
	{
		ElementType v = ((ValueLink*) aLink)->GetValue();

		if (this->ElementsMatch(v,existing))	
		{
			ValueLink* newLink = CreateNewLink(tobeadded);
			fImplementation.AddBefore(*aLink, newLink);
			aLink = kXMPNULL;
		}
		else
			aLink = (ValueLink*) iter.Next();
	}
}

//------------------------------------------------------------------------------
// OrderedCollection::AddAfter
//------------------------------------------------------------------------------

void OrderedCollection::AddAfter(ElementType existing, ElementType tobeadded)
{
	LinkedListIterator iter(&fImplementation);
	ValueLink* aLink = (ValueLink*) iter.First();
	while (aLink != kXMPNULL)
	{
		ElementType v = ((ValueLink*) aLink)->GetValue();

		if (this->ElementsMatch(v,existing))	
		{
			ValueLink* newLink = CreateNewLink(tobeadded);
			fImplementation.AddAfter(*aLink, newLink);
			aLink = kXMPNULL;
		}
		else
			aLink = (ValueLink*) iter.Next();
	}
}

//------------------------------------------------------------------------------
// OrderedCollection::After
//------------------------------------------------------------------------------

ElementType OrderedCollection::After(ElementType existing)
{
	ValueLink* linkAfter = kXMPNULL;

	LinkedListIterator iter(&fImplementation);

	for (ValueLink* link = (ValueLink*) iter.First(); iter.IsNotComplete(); link = (ValueLink*) iter.Next())
	{
		ElementType v = ((ValueLink*) link)->GetValue();

		if (this->ElementsMatch(v,existing))	
		{
			linkAfter = (ValueLink*) fImplementation.After(*link);
			break;
		}
	}
	
	return linkAfter ? linkAfter->GetValue() : (ElementType)kXMPNULL;
}

//------------------------------------------------------------------------------
// OrderedCollection::Before
//------------------------------------------------------------------------------

ElementType OrderedCollection::Before(ElementType existing)
{
	ValueLink* linkBefore = kXMPNULL;

	LinkedListIterator iter(&fImplementation);

	for (ValueLink* link = (ValueLink*) iter.First(); iter.IsNotComplete(); link = (ValueLink*) iter.Next())
	{
		ElementType v = ((ValueLink*) link)->GetValue();

		if (this->ElementsMatch(v,existing))	
		{
			linkBefore = (ValueLink*) fImplementation.Before(*link);
			break;
		}
	}
	
	return linkBefore ? linkBefore->GetValue() : (ElementType)kXMPNULL;
}

//------------------------------------------------------------------------------
// OrderedCollection::First
//------------------------------------------------------------------------------

ElementType OrderedCollection::First()
{
	ValueLink* firstLink = (ValueLink*) fImplementation.First();
	return firstLink ? firstLink->GetValue() : (ElementType)kXMPNULL;
}

//------------------------------------------------------------------------------
// OrderedCollection::Last
//------------------------------------------------------------------------------

ElementType OrderedCollection::Last()
{
	ValueLink* lastLink = (ValueLink*) fImplementation.Last();
	return lastLink ? lastLink->GetValue() : (ElementType)kXMPNULL;
}

//------------------------------------------------------------------------------
// OrderedCollection::RemoveFirst
//------------------------------------------------------------------------------

ElementType	OrderedCollection::RemoveFirst()
{
	ValueLink* aLink = (ValueLink*) fImplementation.RemoveFirst();
	ElementType value = aLink->GetValue();
	delete aLink;
	return value;
}

//------------------------------------------------------------------------------
// OrderedCollection::RemoveLast
//------------------------------------------------------------------------------

ElementType	OrderedCollection::RemoveLast()
{
	ValueLink* aLink = (ValueLink*) fImplementation.RemoveLast();
	ElementType value = aLink->GetValue();
	delete aLink;
	return value;
}

//------------------------------------------------------------------------------
// OrderedCollection::Remove
//------------------------------------------------------------------------------

void OrderedCollection::Remove(ElementType existing)
{
	LinkedListIterator iter(&fImplementation);
	ValueLink* aLink = (ValueLink*) iter.First();
	while (aLink != kXMPNULL)
	{
		ElementType v = ((ValueLink*) aLink)->GetValue();

		if (this->ElementsMatch(v,existing))
		{
			fImplementation.Remove(*aLink);
			delete aLink;
			aLink = kXMPNULL;	
		}
		else
			aLink = (ValueLink*) iter.Next();
	}	
}

//------------------------------------------------------------------------------
// OrderedCollection::RemoveAll
//------------------------------------------------------------------------------

void OrderedCollection::RemoveAll()
{
	Link* link = fImplementation.RemoveFirst();
	while (link != kXMPNULL)
	{
		delete link;
		link = fImplementation.RemoveFirst();
	}
}

//------------------------------------------------------------------------------
// OrderedCollection::DeleteAll
//------------------------------------------------------------------------------

void OrderedCollection::DeleteAll()
{
	Link* link = fImplementation.RemoveFirst();
	while (link != kXMPNULL)
	{
		ElementType value = ((ValueLink*) link)->GetValue();
		delete value;
		delete link;
		link = fImplementation.RemoveFirst();
	}
}

//------------------------------------------------------------------------------
// OrderedCollection::Contains
//------------------------------------------------------------------------------

XMPBoolean	OrderedCollection::Contains(ElementType existing)
{
	LinkedListIterator iter(&fImplementation);
	ValueLink* aLink = (ValueLink*) iter.First();
	while (aLink != kXMPNULL)
	{
		ElementType v = ((ValueLink*) aLink)->GetValue();

		if (this->ElementsMatch(v,existing))
		{
			return kXMPTrue;	
		}
		else
			aLink = (ValueLink*) iter.Next();
	}	
	return kXMPFalse;
}

//------------------------------------------------------------------------------
// OrderedCollection::CreateIterator
//------------------------------------------------------------------------------

OrderedCollectionIterator* OrderedCollection::CreateIterator()
{
	return new(fHeap) OrderedCollectionIterator(this);
}


//------------------------------------------------------------------------------
// OrderedCollection::CreateNewLink
//------------------------------------------------------------------------------

ValueLink*	OrderedCollection::CreateNewLink(ElementType value) const
{
	return new (fHeap) ValueLink(value);
}

//------------------------------------------------------------------------------
// OrderedCollection::ElementsMatch
//------------------------------------------------------------------------------

XMPBoolean	OrderedCollection::ElementsMatch(ElementType v1,ElementType v2) const
{
	return (v1 == v2);
}

//======================================================================================
// OrderedCollectionIterator
//======================================================================================

//------------------------------------------------------------------------------
// OrderedCollectionIterator::OrderedCollectionIterator
//------------------------------------------------------------------------------

OrderedCollectionIterator::OrderedCollectionIterator(OrderedCollection* collection)	
	: fImplementation(collection ? &(collection->fImplementation) : (LinkedList*)kXMPNULL)
{
	fCollection =  collection;
}

//------------------------------------------------------------------------------
// OrderedCollectionIterator::~OrderedCollectionIterator
//------------------------------------------------------------------------------

OrderedCollectionIterator::~OrderedCollectionIterator()						
{
}

//------------------------------------------------------------------------------
// OrderedCollectionIterator::First
//------------------------------------------------------------------------------

ElementType	OrderedCollectionIterator::First()
{
	ValueLink* link = (ValueLink*) fImplementation.First();
	
	return link ? link->GetValue() : (ElementType)kXMPNULL;
}

//------------------------------------------------------------------------------
// OrderedCollectionIterator::Next
//------------------------------------------------------------------------------

ElementType	OrderedCollectionIterator::Next()
{		
	ValueLink* link = (ValueLink*) fImplementation.Next();
	
	return link ? link->GetValue() : (ElementType)kXMPNULL;
}

//------------------------------------------------------------------------------
// OrderedCollectionIterator::Last
//------------------------------------------------------------------------------

ElementType	OrderedCollectionIterator::Last()
{
	ValueLink* link = (ValueLink*) fImplementation.Last();
	
	return link ? link->GetValue() : (ElementType)kXMPNULL;
}

//------------------------------------------------------------------------------
// OrderedCollectionIterator::Previous
//------------------------------------------------------------------------------

ElementType	OrderedCollectionIterator::Previous()
{
	ValueLink* link = (ValueLink*) fImplementation.Previous();
	
	return link ? link->GetValue() : (ElementType)kXMPNULL;
}

//------------------------------------------------------------------------------
// OrderedCollectionIterator::IsNotComplete
//------------------------------------------------------------------------------

XMPBoolean	OrderedCollectionIterator::IsNotComplete()
{
	return fImplementation.IsNotComplete();
}







