/*
	File:		OrdColl.h

	Contains:	Definition of class OrderedCollection

	Written by:	Richard Rodseth

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <9>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <8>	  2/9/94	T		add a couple of explicit "virtual" to the
									class declaration to pacify ASLM
		 <7>	  2/7/94	JA		Tiger Team Makeover!
		 <6>	  2/2/94	NP		Added support for allocating internal
									structures in given heap.
		 <5>	 12/7/93	NP		Added helpful comments, because PEOPLE
									AREN'T COMMENTING HEADER FILES!!!!!
		 <4>	11/19/93	PH		ASLM fix - add class id for ValueLink
		 <3>	11/19/93	NP		Add definition of ValueLink to here (to
									allow subclassing.)
		 <1>	 8/13/93	RCR		first checked in

	To Do:
*/

#ifndef _ORDCOLL_
#define _ORDCOLL_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _LIST_
#include "List.h"
#endif

//==============================================================================
// Theory of Operation
//==============================================================================

// OrdereCollection is an ordered collection of elements of type void* (since
// we can't use templates)
// Duplicates are allowed.

//==============================================================================
// Constants
//==============================================================================

//==============================================================================
// Scalar Types
//==============================================================================

typedef void* ElementType;

//=====================================================================================
// Classes defined in this interface
//=====================================================================================

class OrderedCollection;	// An ordered (not sorted) collection of ElementTypes
class OrderedCollectionIterator;

//=====================================================================================
// Classes used by this interface
//=====================================================================================

class ValueLink; 			// A link plus a value of type ElementType.

//=====================================================================================
// Global Variables
//=====================================================================================

//=====================================================================================
// Class ValueLink - Definition
//=====================================================================================

#define kValueLinkID "appl:valuelink$class,1.0.0"

class ValueLink : public Link {
	ASLM_COMPATIBLE
public:
							ValueLink(ElementType value);		
	XMPVMethod 			   ~ CDECL ValueLink();
	XMPNVMethod	ElementType	GetValue()						{ return fValue;}
	XMPNVMethod void		SetValue(ElementType v)			{ fValue = v;}

private:
	ElementType 		fValue;
};

//=====================================================================================
// Class OrderedCollection
//=====================================================================================

#define kOrderedCollectionID "appl:orderedcollection$class,1.0.0"

class OrderedCollection
{
	ASLM_COMPATIBLE
public:

	OrderedCollection();
	OrderedCollection(XMPHeap where);
	virtual ~ CDECL OrderedCollection();

	XMPNVMethod XMPULong Count() const 			{ return fImplementation.Count(); };
	
	XMPVMethod void	AddFirst(ElementType element);
	XMPVMethod void	AddLast(ElementType element);
	XMPVMethod void	AddBefore(ElementType existing, ElementType tobeadded);
	XMPVMethod void	AddAfter(ElementType existing, ElementType tobeadded);

	XMPVMethod ElementType	After(ElementType existing);
	XMPVMethod ElementType	Before(ElementType existing);

	XMPVMethod ElementType	First();
		// Returns kXMPNULL if there is no first element.
	XMPVMethod ElementType	Last();

	XMPVMethod ElementType	RemoveFirst();
		// Don't call if there are no elements. Crash will result.
	XMPVMethod ElementType	RemoveLast();
	XMPVMethod void	RemoveAll();
	
		// Called from the destructor. Removes all elements, deleting the links
		// Does not delete the elements themselves
		
	XMPVMethod void	DeleteAll();
	
		// Removes and deletes all elements
		
	XMPVMethod void	Remove(ElementType existing);
	XMPVMethod XMPBoolean	Contains(ElementType existing);
	
	XMPVMethod OrderedCollectionIterator* CreateIterator();

protected:
	 XMPVMethod ValueLink* 	CreateNewLink(ElementType value) const;
	 XMPVMethod XMPBoolean	ElementsMatch(ElementType v1,ElementType v2) const;
	 	// Does a pointer comparison by default 

private:
	LinkedList		fImplementation;
	XMPHeap			fHeap; // if kXMPNULL, use default heap.

	friend class OrderedCollectionIterator;
	friend class ListIterator;
};

//=====================================================================================
// Class OrderedCollectionIterator
//=====================================================================================

#define kOrderedCollectionIteratorID "appl:orderedcollectioniterator$class,1.0.0"

class OrderedCollectionIterator {
public:
	OrderedCollectionIterator(OrderedCollection* collection);
	~ CDECL OrderedCollectionIterator();
	ElementType	First();
	ElementType	Next();
	ElementType	Last();
	ElementType	Previous();
	XMPBoolean	IsNotComplete();
	
private:
  	OrderedCollection*	fCollection;
	LinkedListIterator	fImplementation;
};

#endif // _ORDCOLL_