/*
	File:		StrHshTb.cpp

	Contains:	Implementation for XMPStringHashTable class.

	Written by:	Nick Pilch

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <2>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<17>	  2/7/94	JA		Capitalized THROW.
		<16>	  2/7/94	JA		Tiger Team Makeover!
		<15>	  2/3/94	CG		Added fHeap to XMPStringHashTable.
		<14>	 12/3/93	T		Stop including XMPError.h, it is included
									as ErrorDef.h inside Except.h
		<13>	 11/5/93	NP		Added error checking and GetNumEntries
									methods.
		<12>	 11/2/93	NP		Bug fixes.
		<11>	10/22/93	NP		Changed for new protocol-only throw for out
									of memory.
		<10>	10/14/93	JA		Fixed a typo bug in
									XMPStringHashTableIterator::Next; it wasn't
									setting the length of the returned string
									if it wasn't found (was len=0, fixed to
									*len=0)
		 <9>	 9/21/93	CG		Find returns valuelength.
		 <8>	  7/2/93	NP		Mods to prevent shared memory problems.
		 <7>	 6/22/93	NP		Changed iterator semantics. Fixed bugs.
		 <6>	 6/16/93	NP		Fixed bug that compared keys incorrectly.
		 <5>	 6/15/93	NP		Mods to allow compilation with
									LibraryManager.h
		 <4>	  6/4/93	NP		Added iterator class.
		 <3>	 4/29/93	NP		File name changes.
		 <2>	 4/26/93	NP		It's working now.
		 <1>	 4/16/93	NP		first checked in

	To Do:
		Some factoring of code would be good.
*/

/*
	A simple hash table with chaining. Each hash table entry
	is a HashEntryRec. Each HashEntryRec contains a pointer to start of a
	linked list of EntryNodes. Each EntryNode contains the key, the length of
	key, the value, the length of the value and at least
	one link to another item in the list.
	
	The keys are stored as NULL terminated C-strings for debugging purposes
	even though they may have imbedded NULLs;
*/

#ifndef _STRHSHTB_
#include "StrHshTb.h"
#endif

#ifndef __CTYPE__
#include <ctype.h>
#endif

#ifndef __LIMITS__
#include <limits.h>
#endif

#ifndef _EXCEPT_
#include "Except.h"
#endif

#ifndef __STRING__
#include <string.h>
#endif

#ifndef _XMPMEM_
#include "XMPMem.h"
#endif

#pragma segment XMPStringHashTable

//==============================================================================
// Local Classes
//==============================================================================

struct EntryNode
{
	XMPUByte*	key;
	XMPULong	keySize;
	XMPPtr		value;
	XMPULong	valueLen;
	EntryNode*	nextEntry; // == 0 means end of list.
};

struct HashEntryRec
{
	EntryNode*	node;		//	pointer to first node
};

#if 0
	//OBSOLETE
	struct LinkedNodesIterator
	{
		LinkedNodesIterator(XMPStringHashTable* table, XMPULong tableIndex)
							{fTableIndex = tableIndex; fTable = table;
								fCurNode = (table->fTable + tableIndex)->node;}
		EntryNode*	First() {return fCurNode;}
		EntryNode*	Next() {return (fCurNode = fCurNode->nextEntry);}
		XMPBoolean	IsNotComplete() {return (fCurNode->nextEntry != 0);}
	  private:
		XMPStringHashTable*	fTable;
		XMPULong			fTableIndex;
		EntryNode*			fCurNode;
	};
#endif // 0

struct LinkedNodesIterator
{
  public:
	LinkedNodesIterator(XMPStringHashTable* table, XMPULong tableIndex);
	EntryNode*	First();
	EntryNode*	Next();
	XMPBoolean	IsNotComplete();
  private:
	XMPStringHashTable*	fTable;
	XMPULong			fTableIndex;
	EntryNode*			fCurNode;
	XMPBoolean			fDone;
};

LinkedNodesIterator::LinkedNodesIterator(XMPStringHashTable* table, XMPULong tableIndex)
{
	fTableIndex = tableIndex;
	fTable = table;
	fCurNode = (table->GetTable() + tableIndex)->node;
	fDone = kXMPFalse;
}

EntryNode*	LinkedNodesIterator::First()
{
	return fCurNode;
}

EntryNode*	LinkedNodesIterator::Next()
{
	if (fCurNode->nextEntry == kXMPNULL)
	{
		fDone = kXMPTrue;
		return 0;
	}
	else
		return (fCurNode = fCurNode->nextEntry);
}

XMPBoolean	LinkedNodesIterator::IsNotComplete()
{
	if (!fCurNode)
		return kXMPFalse;
	else
		return !fDone;
}


//==============================================================================
// XMPStringHashTable
//==============================================================================

//------------------------------------------------------------------------------
// XMPStringHashTable::XMPStringHashTable
//------------------------------------------------------------------------------

ASLM_SCDECLARATION(XMPStringHashTable)

XMPStringHashTable::XMPStringHashTable(XMPULong numEntries)
{
	if (numEntries == 0)
		fNumSlots = 1;
	else
		fNumSlots = numEntries;
	fHashFunc = XMPStringHashTable::StdHash;
	fNumEntries = 0;
	fHeap = 0;
}

//------------------------------------------------------------------------------
// XMPStringHashTable::Initialize
//------------------------------------------------------------------------------

void XMPStringHashTable::Initialize(XMPHeap heap)
{
	XMPULong	tableSize = fNumSlots * sizeof(HashEntryRec);

	fHeap = heap;
	fTable = (HashEntryRec*)XMPNewPtrClear(tableSize, fHeap);
	if (!fTable)
		THROW(kXMPErrOutOfMemory);
}

//------------------------------------------------------------------------------
// XMPStringHashTable::~XMPStringHashTable
//------------------------------------------------------------------------------

XMPStringHashTable::~XMPStringHashTable()
{
	// gotta be careful to delete a node only after we're done with it.
	for (XMPULong i = 0; i < fNumSlots; i++)
	{
		LinkedNodesIterator	iter(this, i);
		EntryNode*			someNode;
		EntryNode*			lastNode = 0; // first time through

		for (someNode = iter.First();
				iter.IsNotComplete();
				someNode = iter.Next())
		{
			if (lastNode)
			{
				XMPDisposePtr((Ptr)lastNode->key);
				XMPDisposePtr((Ptr)lastNode);
			}
			lastNode = someNode;
		}
		if (lastNode)
		{
			XMPDisposePtr((Ptr)lastNode->key);
			XMPDisposePtr((Ptr)lastNode);
		}
	}
	XMPDisposePtr((Ptr)fTable);
}

//------------------------------------------------------------------------------
// XMPStringHashTable::Insert
//
//	Hash the key, Map the hash to the range of table index values, insert the
//	hash into the linked list of EntryNodes at that table slot.
//------------------------------------------------------------------------------

void XMPStringHashTable::Insert(XMPUByte* string, XMPULong stringLength,
										XMPPtr value, XMPULong valueLength)
{
	if (stringLength == 0)
		THROW(kXMPErrInvalidKey);
	if (value == (XMPPtr)kXMPNULL)
		THROW(kXMPErrInvalidValue);
	XMPULong tableIndex = this->HashAndMap(string, stringLength);
	this->InsertAtIndex(tableIndex, string, stringLength, value, valueLength);
	++fNumEntries;
}

//------------------------------------------------------------------------------
// XMPStringHashTable::InsertAtIndex
//
//	Insert a hash table node at the the specified index in the table.
//------------------------------------------------------------------------------

void XMPStringHashTable::InsertAtIndex(XMPULong index, XMPUByte* string,
												XMPULong stringLength,
												XMPPtr value,
												XMPULong valueLength)
{
	HashEntryRec*	entry = fTable + index;
	
	// No nodes at this table entry? Stick in the first one.
	if (entry->node == kXMPNULL)
		entry->node = this->MakeNewNode(string, stringLength, value,
										valueLength);
	// Else, need to follow chain to end checking for duplicate keys
	else
	{
		LinkedNodesIterator	iter(this, index);
		EntryNode*			someNode;
		EntryNode*			lastNode;
		XMPBoolean			replaceOne = kXMPFalse;
		XMPBoolean			alreadyHadKey = kXMPFalse;

		// Iterate to the end of the list or until we find the key
		for (someNode = iter.First();
				iter.IsNotComplete();
				lastNode = someNode, someNode = iter.Next())
		{
			if ((someNode->keySize == stringLength)
				&& (!memcmp((const char*)someNode->key, (const char*)string,
							(size_t)stringLength)))
			{
				someNode->value = value;
				someNode->valueLen = valueLength;
				alreadyHadKey = kXMPTrue;
				break;
			}
		}
		if (!alreadyHadKey)
			// If we got here, we have to create a new node.
			lastNode->nextEntry = MakeNewNode(string, stringLength, value,
												valueLength);
	}
}

//------------------------------------------------------------------------------
// XMPStringHashTable::MakeStringFromBytes
//
//	Create a new NULL-terminated C-string from a stream of bytes.
//------------------------------------------------------------------------------

XMPUByte* XMPStringHashTable::MakeStringFromBytes(XMPUByte* string,
													XMPULong stringLength)
{
	XMPUByte* newString = (XMPUByte*)XMPNewPtr(stringLength + 1, fHeap);
	if (!newString)
		THROW(kXMPErrOutOfMemory);
	XMPBlockMove(string, newString, stringLength);
	newString[stringLength] = 0;
	return newString;
}

//------------------------------------------------------------------------------
// XMPStringHashTable::MakeNewNode
//
//	Allocate and intialize a new entry node.
//------------------------------------------------------------------------------

EntryNode* XMPStringHashTable::MakeNewNode(XMPUByte* string,
											XMPULong stringLength, XMPPtr value,
											XMPULong valueLength)
{
	EntryNode* newNode = (EntryNode*)XMPNewPtr(sizeof(EntryNode), fHeap);

	// Convert strings to C-strings and copy
	XMPUByte* newString = this->MakeStringFromBytes(string, stringLength);
	
	newNode->key = newString;
	newNode->keySize = stringLength;
	newNode->nextEntry = kXMPNULL;
	newNode->value = value;
	newNode->valueLen = valueLength;
	
	return newNode;
}

//------------------------------------------------------------------------------
// XMPStringHashTable::Remove
//
//	Hash the key, Map the hash to the range of table index values, search the
//	the linked list of EntryNodes for the key, remove the EntryNode in which
//	the key was found.
//------------------------------------------------------------------------------

void XMPStringHashTable::Remove(XMPUByte* string, XMPULong stringLength)
{
	if (stringLength == 0)
		THROW(kXMPErrInvalidKey);
	XMPULong tableIndex = this->HashAndMap(string, stringLength);
	this->RemoveAtIndex(tableIndex, string, stringLength);
	--fNumEntries;
}

//------------------------------------------------------------------------------
// XMPStringHashTable::RemoveAtIndex
//
//	Remove a hash table node at the the specified index in the table.
//------------------------------------------------------------------------------

void XMPStringHashTable::RemoveAtIndex(XMPULong index, XMPUByte* string,
												XMPULong stringLength)
{
	HashEntryRec*	entry = fTable + index;
	
	if (entry->node != kXMPNULL)
	{
		LinkedNodesIterator	iter(this, index);
		EntryNode*			someNode;
		EntryNode*			prevNode = 0; // start of chain

		for (someNode = iter.First();
				iter.IsNotComplete();
				someNode = iter.Next())
		{
			// look for a match
			if ((someNode->keySize == stringLength)
				&& (!memcmp((const char*)someNode->key, (const char*)string,
							(size_t)stringLength)))
			{
				// Special case first node
				if (prevNode == 0)
					entry->node = someNode->nextEntry;
				else
					prevNode->nextEntry = someNode->nextEntry;

				// Must dispose someNode->key first or else we'll lose its
				//	reference after disposing the someNode.
				XMPDisposePtr((Ptr)someNode->key);
				XMPDisposePtr((Ptr)someNode);
				break;
			}
			prevNode = someNode;
		}
	}
}

//------------------------------------------------------------------------------
// XMPStringHashTable::Find
//
//	Hash the key, Map the hash to the range of table index values, search the
//	the linked list of EntryNodes for the key. Signal error if not found.
//------------------------------------------------------------------------------

XMPBoolean XMPStringHashTable::Find(XMPUByte* string, XMPULong stringLength,
									XMPPtr* value, XMPULong* valueLength)
{
	XMPULong		tableIndex = this->HashAndMap(string, stringLength);
	HashEntryRec*	entry = fTable + tableIndex;
	XMPBoolean		result = kXMPFalse;
	
	if (stringLength == 0)
		THROW(kXMPErrInvalidKey);

	if (entry->node != kXMPNULL)
	{
		LinkedNodesIterator	iter(this, tableIndex);
		EntryNode*			someNode;

		for (someNode = iter.First();
				iter.IsNotComplete();
				someNode = iter.Next())
		{
			// match
			if ((someNode->keySize == stringLength)
				&& (!memcmp((const char*)someNode->key, (const char*)string,
							(size_t)stringLength)))
			{
				*value = someNode->value;
				*valueLength = someNode->valueLen;
				result = kXMPTrue;
				break;
			}
		}
	}

	return result;
}

//------------------------------------------------------------------------------
// XMPStringHashTable::GetNumEntries
//------------------------------------------------------------------------------

XMPULong XMPStringHashTable::GetNumEntries()
{
	return fNumEntries;
}

//------------------------------------------------------------------------------
// XMPStringHashTable::StdHash
//
//	Adds all the characters in the string and returns the smallest and largest
//	numbers that they could add up to in hashValueLowerBounds and
//	hashValueUpperBounds.
//------------------------------------------------------------------------------

XMPULong XMPStringHashTable::StdHash(XMPUByte* string,
											XMPULong stringLength,
											XMPULong& hashValueLowerBounds,
											XMPULong& hashValueUpperBounds)
{
	XMPULong	result = 0;

	for (XMPULong i = 0; i < stringLength; i++)
	{
		result += *(string + i);
	}
	hashValueLowerBounds = 0;
	hashValueUpperBounds = 255 * stringLength;
	return result;
}

//------------------------------------------------------------------------------
// XMPStringHashTable::MapToTableIndex
//
//	Map hash value in its range to the range 0..(fNumSlots - 1).
//------------------------------------------------------------------------------

XMPULong XMPStringHashTable::MapToTableIndex(XMPULong hash,
												XMPULong hashValueLowerBounds,
												XMPULong hashValueUpperBounds)
{
	if (hashValueLowerBounds >= hashValueUpperBounds)
		return 0;
	else
	{
		double index = (double)(hash - hashValueLowerBounds)
			/ (double)(hashValueUpperBounds - hashValueLowerBounds)
			* (double)(fNumSlots - 1);
		double indexToRound = index + .5;
		return (XMPULong)indexToRound;
	}
}

//------------------------------------------------------------------------------
// XMPStringHashTable::HashAndMap
//
//	Utility function.
//------------------------------------------------------------------------------

XMPULong XMPStringHashTable::HashAndMap(XMPUByte* string, XMPULong stringLength)
{
	XMPULong	hashUpperBounds, hashLowerBounds;

	XMPULong hash = fHashFunc(string, stringLength, hashLowerBounds,
								hashUpperBounds);
	return this->MapToTableIndex(hash, hashLowerBounds, hashUpperBounds);
}

//------------------------------------------------------------------------------
// XMPStringHashTable::PrintTable
//
//	Testing / Debugging function.
//------------------------------------------------------------------------------

#include <stdio.h>

void XMPStringHashTable::PrintTable(FILE* outfile)
{
	XMPStringHashTableIterator	iter(this);
	XMPUByte*					string;
	XMPULong					len;
	XMPPtr						value;
	XMPULong					valueLen;

	fprintf(outfile, "The table contains this:\n");

	for (iter.First(&string, &len, &value, &valueLen);
			iter.IsNotComplete();
			iter.Next(&string, &len, &value, &valueLen))
	{	
		fprintf(outfile, "%s, %d%d\n", string, value, valueLen);
	}

	fprintf(outfile, "\n");
}

//------------------------------------------------------------------------------
// XMPStringHashTable::PrintDistribution
//
//	Testing / Debugging function.
//------------------------------------------------------------------------------

void XMPStringHashTable::PrintDistribution(FILE* outfile)
{
	fprintf(outfile, "The table looks like this:\n");

	for (XMPULong i = 0; i < fNumSlots; i++)
	{
		fprintf(outfile, "Slot %d:\n", i);

		LinkedNodesIterator	iter(this, i);
		EntryNode*			someNode;

		for (someNode = iter.First();
				iter.IsNotComplete();
				someNode = iter.Next())
		{
			fprintf(outfile, "\t%s\n", someNode->key);
		}
	}

	fprintf(outfile, "\n");
}

//------------------------------------------------------------------------------
// XMPStringHashTable::GetHashFunc
//------------------------------------------------------------------------------

XMPStringHashTableFunc XMPStringHashTable::GetHashFunc()
{
	return fHashFunc;
}

//------------------------------------------------------------------------------
// XMPStringHashTable::SetHashFunc
//------------------------------------------------------------------------------

void XMPStringHashTable::SetHashFunc(XMPStringHashTableFunc func)
{
	fHashFunc = func;
}

//------------------------------------------------------------------------------
// XMPStringHashTable::GetNumSlots
//
//	For friends.
//------------------------------------------------------------------------------

XMPULong XMPStringHashTable::GetNumSlots()
{
	return fNumSlots;
}

//------------------------------------------------------------------------------
// XMPStringHashTable::GetTable
//
//	For friends.
//------------------------------------------------------------------------------

HashEntryRec* XMPStringHashTable::GetTable()
{
	return fTable;
}

//==============================================================================
// XMPStringHashTableIterator
//==============================================================================

//------------------------------------------------------------------------------
// XMPStringHashTableIterator::XMPStringHashTableIterator
//------------------------------------------------------------------------------

XMPStringHashTableIterator::XMPStringHashTableIterator(XMPStringHashTable* tb)
{
	fTable = tb;
	fTableIndex = 0;
	fCurNode = kXMPNULL;
	fDone = kXMPFalse;
}

//------------------------------------------------------------------------------
// XMPStringHashTableIterator::~XMPStringHashTableIterator
//------------------------------------------------------------------------------

XMPStringHashTableIterator::~XMPStringHashTableIterator()
{
}

//------------------------------------------------------------------------------
// XMPStringHashTableIterator::First
//
//	Use GetNext to find first. Set flag if we don't find an entry.
//------------------------------------------------------------------------------

void XMPStringHashTableIterator::First(XMPUByte** string, XMPULong* len,
											XMPPtr* value,
											XMPULong* valueLength)
{
	if (!this->GetNext(string, len, value, valueLength))
	{
//		*len  = 0;
//		*string = (XMPUByte*)kXMPNULL;
		fDone = kXMPTrue;
	}
}

//------------------------------------------------------------------------------
// XMPStringHashTableIterator::Next
//
//	Use GetNext. Set flag if we don't find an entry.
//------------------------------------------------------------------------------

void XMPStringHashTableIterator::Next(XMPUByte** string, XMPULong* len,
											XMPPtr* value,
											XMPULong* valueLength)
{
	if (!this->GetNext(string, len, value, valueLength))
	{
//		*len  = 0;
//		*string = (XMPUByte*)kXMPNULL;
		fDone = kXMPTrue;
	}
}

//------------------------------------------------------------------------------
// XMPStringHashTableIterator::IsNotComplete
//
//	Return value of previously set flag.
//------------------------------------------------------------------------------

XMPBoolean XMPStringHashTableIterator::IsNotComplete()
{
	return !fDone;
}

//------------------------------------------------------------------------------
// XMPStringHashTableIterator::GetNext
//
// kXMPTrue is returned if an entry was found, kXMPFalse otherwise. The
//	values pointed to by the parameters are undefined if kXMPFalse is
//	returned.
//	We can record the index of the last hash bucket we were looking in, but we
//	can't record the index into the linked list at that bucket. So we keep a
//	pointer to the last node found and look for it the next time we iterate
//	through that list.
//------------------------------------------------------------------------------

XMPBoolean XMPStringHashTableIterator::GetNext(XMPUByte** string, XMPULong* len,
												XMPPtr* value,
												XMPULong* valueLength)
{
	for (XMPULong i = fTableIndex; i < fTable->GetNumSlots(); i++)
	{
		LinkedNodesIterator	iter(fTable, i);
		EntryNode*			someNode;
		XMPBoolean			pastLastNode = fCurNode ? kXMPFalse : kXMPTrue;

		for (someNode = iter.First();
				iter.IsNotComplete();
				someNode = iter.Next())
		{
			if ((i == fTableIndex) && (someNode == fCurNode))
			{
				pastLastNode = kXMPTrue;
				continue;
			}
			if (pastLastNode || (i != fTableIndex))
			{
				fTableIndex = i;
				fCurNode = someNode;
				*string = someNode->key;
				*len = someNode->keySize;
				*value = someNode->value;
				*valueLength = someNode->valueLen;
				return kXMPTrue;
			}
		}
	}
	return kXMPFalse;
}

