/*
	File:		XMPMem.cpp

	Contains:	Implementation of XMPMem.h

	Written by:	Jens Alfke, Vincent Lo, Nick Pilch

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <9>	 4/13/94	T		jpa: Fix a #if so we can compile with
									XMPDebug off (1157156)
		 <8>	  3/2/94	VL		#1147830: BasicNewPtr was allocating memory
									in the wrong heap in non-TRACKSIZE mode.
		 <7>	  3/1/94	JA		Fill newly-created blocks with 0xBBBB.
		 <6>	 2/25/94	T		comment out the mem validation stuff
		 <5>	 2/24/94	JA		Turned off TRACK_SIZES; using ASLM calls to
									get block sizes.
		 <4>	 2/22/94	VL		Added stubs for new memory calls.
		 <3>	 2/22/94	VL		JA: Added lots of validation and sanity
									checking code to catch memory errors.
		 <2>	 2/16/94	JA		Always use ASLM instead of malloc.
		<16>	  2/7/94	JA		Capitalized THROW.
		<15>	  2/7/94	JA		Tiger Team Makeover!
		<14>	  2/3/94	CG		Added XMPNewPtrClear that takes a heap.
		<13>	  2/2/94	NP		Added XMPGetSystemHeap, XMPNewPtrSys and
									XMPGetPool -> XMPGetHeap.
		<12>	 1/12/94	CG		Added NewPtrClear.
		<11>	 1/10/94	NP		Removed bogus error constant.
		<10>	 12/9/93	VL		Added XMPSetHandleSize.
		 <9>	 12/8/93	VL		Set the size to 1 if the requested size is
									0 in XMPNewPtr (for ASLM).
		 <8>	11/24/93	VL		Added XMPUnlockHandle(void*) and made
									Disposing NULL ok.
		 <7>	11/23/93	VL		Added overloaded ptr calls and new handle
									calls.
		 <6>	11/23/93	JA		Made XMPBlockMove signature match
									declaration.
		 <5>	10/15/93	NP		Cast so CFront compile would work.
		 <4>	10/14/93	JA		Added XMPCopyHandle. Added assertions for
									input sanity checking. XMPNewPtr now fails
									when out of memory, even in non-ASLM
									builds. Worked around malloc(0) problem.
		 <3>	 8/20/93	VL		Throw if memory cannot be allocated.
		 <2>	 8/16/93	NP		Casts to satify compiler. Fixed typo.
		 <1>	 8/16/93	NP		first checked in

	To Do:
*/

#ifndef _XMPMEM_
#include "XMPMem.h"
#endif

#ifndef _EXCEPT_
#include "Except.h"
#endif

#ifndef __LIBRARYMANAGER__
#include "LibraryManager.h"
#endif

#ifndef __LIBRARYMANAGERUTILITIES__
#include "LibraryManagerUtilities.h"
#endif

#ifndef __LIBRARYMANAGERCLASSES__
#include "LibraryManagerClasses.h"
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#pragma segment XMPMem

//==============================================================================
// Constants
//==============================================================================


#if XMPDebug
	// When debugging, whe need to know the size of a block (so we can zap it
	// when it's deleted. If our current memory manager does not give us access
	// to block sizes, then turn on TRACK_SIZES. Else leave it off. In particular,
	// it causes problems with ASLM.
	#define TRACK_SIZES 0
#endif

const XMPULong	kMaxBlockSize	= 0x00FFFFFF;	// Max size is 24MB

#define OFFSET(PTR,AMT)		((XMPPtr)( (XMPULong)(PTR) + (AMT) ))

//==============================================================================
// Scalar Types
//==============================================================================

//==============================================================================
// Local Classes
//==============================================================================

//==============================================================================
// Global Variables
//==============================================================================


#if XMPDebug
static XMPSShort gValidate = 0;
#endif


//==============================================================================
// Function Prototype
//==============================================================================

//==============================================================================
// VALIDATION
//==============================================================================

#if XMPDebug

#ifndef _XMPOBJ_
#include "XMPObj.h"
#endif

void
XMPBeginMemValidation( )
{
	XMPValidateAllHeaps();
	gValidate++;
}

void
XMPEndMemValidation( )
{
	if( gValidate>0 )
		gValidate--;
	else
		WARN("Improper nesting of memory-validation calls");
	XMPValidateAllHeaps();
}


XMPBoolean
XMPValidateHeap( XMPHeap heap )
{
	TMemoryPool *pool;
	if( heap )
		pool = (TMemoryPool*)heap;
	else {
		// There seems to be no way to get the default memory pool,
		// so we allocate a pointer in it and then recover the pool:
		XMPPtr p = SLMNewOperator(1,NULL);
		if( p==kXMPNULL )
			THROW(kXMPErrOutOfMemory);
		pool = TMemoryPool::RecoverPool(p);
		SLMDeleteOperator(p);
	}
	
	XMPBoolean valid = pool->CheckPool();
	if( !valid )
		WARN("Heap at %p is bogus, dude!",pool);
		
	return valid;
}


XMPBoolean
XMPValidateAllHeaps( )
{
	return XMPValidateHeap(kXMPNULL) && XMPValidateHeap(XMPGetSystemHeap());
}


static inline XMPBoolean
PtrInHeap( void *p, THz heap )
{
	return (p >= &heap->heapData) && (p < heap->bkLim);
}


static const char*
BasicValidatePtr( void *p, XMPBoolean mustBeInHeap = kXMPTrue )
{
	if( p == kXMPNULL )
		return "is NULL";
	else if( (XMPULong)p & 1 )
		return "is odd";
	else if( mustBeInHeap && !PtrInHeap(p,ApplicationZone()) && !PtrInHeap(p,SystemZone()) )
		return "is not in app or sys heap";
	else if( !mustBeInHeap && (p<&SystemZone()->heapData || p>LMGetBufPtr()) )
		return "is outside the known universe";
	else
		return kXMPNULL;
}


XMPBoolean
XMPValidatePtr( XMPPtr p )
{
	// If we could be sure that p were in the app zone, we could be more
	// stringent. But it might be in the System heap or in temp-mem.
	
	const char *err = BasicValidatePtr(p);
	if( err )
		;
#if TRACK_SIZES
	else if( ((XMPULong*)p)[-1] > kMaxBlockSize )
		err = "points to block of bogus size";
	else if( !XMPValidateHeap(
				(XMPHeap)TMemoryPool::RecoverPool(OFFSET(p,-sizeof(XMPULong)))) )
		return kXMPFalse;	// XMPValidateHeap already warns the user
#else
	else if( (XMPULong)TMemoryPool::GetMemorySize(p) > kMaxBlockSize )
		err = "points to block of bogus size";
	else if( !XMPValidateHeap( (XMPHeap)TMemoryPool::RecoverPool(p) ) )
		return kXMPFalse;	// XMPValidateHeap already warns the user
#endif
else
		return kXMPTrue;
	WARN("Pointer %p %s!",p,err);
	return kXMPFalse;
}


XMPBoolean
XMPValidateObject( XMPObject *o )
{
	if( !XMPValidatePtr(o) )
		return kXMPFalse;
	
	// $$$$$ $$$$$ $$$$$ $$$$$
	// MEGA-SPOODGICIOUS HACK: We'd like to see if the object's vtable ptr (vptr)
	// is alive and well. The only way (?) to access it is by groping into the
	// object. You'd expect it to be the first field, but oddly enough it comes
	// after the XMPObject fields ... at least with cfront!           --Jens
	XMPPtr vptr = *(XMPPtr*) OFFSET(o,sizeof(XMPObject)-sizeof(void*));
	
	const char *err = BasicValidatePtr(vptr,kXMPFalse);	// Vptr is not in a heap...
	if( err )
		WARN("Vptr %p of 'object' %p %s",vptr,o,err);

	if( !o->ValidateObject() )
		WARN("Object %p says it's invalid",o);
	else
		return kXMPTrue;
	return kXMPFalse;
}

XMPBoolean
XMPValidateAllObjects( )
{
	XMPBoolean	valid = kXMPTrue;
	
#if 0
	XMPObject*	obj = XMPObject::GetFirstObject();
	while (obj != kXMPNULL) {
		if (XMPValidateObject(obj) == kXMPFalse)
			valid = kXMPFalse;
		obj = obj->GetNextObject();
	}
#endif
	return valid;
}


XMPBoolean
XMPValidateHandle( XMPHandle h )
{
	// Check handle itself as a pointer:
	const char *err = BasicValidatePtr(h);
	if( err == kXMPNULL && *h == kXMPNULL )
		err = "is purged";
	
	if( err ) {
		WARN("Handle %p %s!",h,err);
		return kXMPFalse;
	}
	
	// Check master pointer for validity:
	err = BasicValidatePtr(*h);
	if( err ) {
		WARN("Handle %p's master %p %s!",h,*h,err);
		return kXMPFalse;
	}
	
	// Check handle's size:
	XMPULong size = GetHandleSize(h);
	if( size > kMaxBlockSize )
		WARN("Handle %p has bogus size %08lx!",h,size);
	else if( MemError() != noErr )
		WARN("Handle %p caused err %d on GetHandleSize",h,MemError());
	else
		return kXMPTrue;
	return kXMPFalse;
}


void
XMPZapMem( XMPPtr p, XMPULong size )
{
	memset(p,0xDD,(size_t)size);
}

XMPBoolean
XMPZapBlock( XMPPtr p )
{
	if( !XMPValidatePtr(p) )
		return kXMPFalse;
	else {
		XMPZapMem(p, XMPGetPtrSize(p));
		return kXMPTrue;
	}
}

#endif //XMPDebug


//==============================================================================
// XMPMem
//==============================================================================


//------------------------------------------------------------------------------
// XMPNewPtr
//------------------------------------------------------------------------------

static XMPPtr BasicNewPtr(XMPULong howBig, XMPHeap heap)
{
	void*		ptr;

#if XMPDebug
	if( howBig > kMaxBlockSize ) {
		WARN("XMPNewPtr(%08lx)?!?!",howBig);
		THROW(kXMPErrOutOfMemory);
	}
	if( gValidate>0 ) {
		XMPValidateHeap(heap);
	}
#endif

#if TRACK_SIZES
	ptr = SLMNewOperator((size_t)howBig+sizeof(XMPULong), heap);
#else
	if( howBig==0 )		// malloc(0) returns NULL, which is not good.
		howBig = 1;		// So fake it out by actually asking for 1 byte. --jpa
	ptr = SLMNewOperator((size_t)howBig, heap);
#endif

	if (ptr == kXMPNULL)
		THROW(kXMPErrOutOfMemory);

#if TRACK_SIZES
	*(XMPULong*)ptr = howBig;				// Store block size in 1st 4 bytes
	ptr = OFFSET(ptr,sizeof(XMPULong));
#endif

#if XMPDebug
	if( gValidate>0 ) {
		XMPValidateHeap(heap);
	}
#endif

	return ptr;
}


XMPPtr XMPNewPtr( XMPULong howBig, XMPHeap heap )
{
	XMPPtr p = BasicNewPtr(howBig,heap);
#if XMPDebug
	if( gValidate>0 )
		memset(p,0xBB,(size_t)howBig);			// Fill new blocks with BB
#endif
	return p;
}


XMPPtr XMPNewPtr( XMPULong howBig )
{
	XMPPtr p = BasicNewPtr(howBig,kXMPNULL);
#if XMPDebug
	if( gValidate>0 )
		memset(p,0xBB,(size_t)howBig);			// Fill new blocks with BB
#endif
	return p;
}

//------------------------------------------------------------------------------
// XMPNewPtrClear
//------------------------------------------------------------------------------

XMPPtr XMPNewPtrClear(XMPULong howBig, XMPHeap heap)
{
	XMPPtr ptr = BasicNewPtr(howBig,heap);
	ZeroMem( ptr, (size_t)howBig );
	return ptr;
}


XMPPtr XMPNewPtrClear(XMPULong howBig)
{
	return XMPNewPtrClear(howBig, kXMPNULL);
}

//------------------------------------------------------------------------------
// XMPNewPtrSys
//------------------------------------------------------------------------------

XMPPtr XMPNewPtrSys(XMPULong howBig)
{
	XMPPtr ptr = BasicNewPtr(howBig, (TMemoryPool*)GetSystemPool());
#if XMPDebug
	if( gValidate>0 )
		memset(ptr,0xBB,(size_t)howBig);			// Fill new blocks with BB
#endif
	return ptr;
}

//------------------------------------------------------------------------------
// XMPGetPtrSize
//------------------------------------------------------------------------------

#if XMPDebug

XMPULong
XMPGetPtrSize( XMPPtr ptr )
{
	XMPHeap heap = XMPGetHeap(ptr);				// Validates the ptr, too
	return ((TMemoryPool*)heap) ->GetSize(ptr);
}

#endif

//------------------------------------------------------------------------------
// XMPDisposePtr
//------------------------------------------------------------------------------

void XMPDisposePtr(XMPPtr pointer)
{
	if( pointer==kXMPNULL )
		return;
		
#if TRACK_SIZES
	if( gValidate>0 && !XMPValidatePtr(pointer) )
		return;
	// Actual block begins at leading size field:
	pointer = OFFSET(pointer, -sizeof(XMPULong));
	if( gValidate>0 )
		XMPZapMem(pointer,*(XMPULong*)pointer + sizeof(XMPULong));
#elif XMPDebug
	if( gValidate>0 )
		if( !XMPZapBlock(pointer) )
			return;
#endif

#if XMPDebug
	XMPHeap heap = XMPGetHeap(pointer);
#endif

	SLMDeleteOperator(pointer);

#if XMPDebug
	if( gValidate>0 ) {
		XMPValidateHeap(heap);				// Make sure heap is still ok
	}
#endif
}

//------------------------------------------------------------------------------
// XMPGetHeap
//------------------------------------------------------------------------------

XMPHeap XMPGetHeap(XMPPtr pointer)
{
	if( pointer==kXMPNULL ) {
		WARN("Can't get heap for NULL ptr");
		THROW(kXMPErrMemoryProblem);
	}
	
#if XMPDebug
	if( gValidate>0 )
		if( !XMPValidatePtr(pointer) )
			THROW(kXMPErrMemoryProblem);
#endif

#if TRACK_SIZES
	pointer = OFFSET(pointer,-sizeof(XMPULong));
#endif
	return TMemoryPool::RecoverPool(pointer);
}

//------------------------------------------------------------------------------
// XMPGetSystemHeap
//------------------------------------------------------------------------------

XMPHeap XMPGetSystemHeap()
{
	return (TMemoryPool*)GetSystemPool();
}

//------------------------------------------------------------------------------
// XMPBlockMove
//------------------------------------------------------------------------------

void XMPBlockMove(ConstXMPPtr from, XMPPtr to, XMPULong size)
{
	ASSERT(from!=NULL,kXMPErrInvalidValue);
	ASSERT(to!=NULL,kXMPErrInvalidValue);
	ASSERT(size<=kMaxBlockSize,kXMPErrInvalidValue);
	
	SLMmemmove(to, from, (size_t)size);
}


//==============================================================================
// HANDLES (RELOCATABLE BLOCKS)
//==============================================================================


//------------------------------------------------------------------------------
// XMPNewHandle
//------------------------------------------------------------------------------

XMPHandle XMPNewHandle(XMPULong howBig)
{
	Handle	h = NewHandle(howBig);
	OSErr err = MemError();
	if( err )
		if( err==memFullErr )
			THROW(kXMPErrOutOfMemory);
		else {
			WARN("NewHandle got err %d",err);
			THROW(kXMPErrMemoryProblem);
		}
	THROW_IF_NULL(h);

#if XMPDebug
	if( gValidate>0 )
		memset(*h,0xBB,(size_t)howBig);			// Fill new blocks with BB
#endif

	return h;
}

//------------------------------------------------------------------------------
// XMPDisposeHandle
//------------------------------------------------------------------------------

void XMPDisposeHandle(XMPHandle handle)
{
	if( handle != kXMPNULL ) {
#if XMPDebug
		if( gValidate>0 ) {
			if( !XMPValidateHandle(handle) )
				return;
			XMPZapMem(*handle,GetHandleSize(handle));
		}
		
		if( HGetState(handle) & 0x20 ) {
			WARN("Tried to dispose rsrc %p",handle);
			return;
		}
#endif
		
		DisposeHandle((Handle)handle);
#if XMPDebug
		OSErr err = MemError();
		if( err!=noErr )
			WARN("DisposeHandle returned err %d",err);
#endif
	}
}

//------------------------------------------------------------------------------
// XMPCopyHandle
//------------------------------------------------------------------------------

XMPHandle XMPCopyHandle(XMPHandle handle)
{
	ASSERTM(handle!=NULL,kXMPErrInvalidValue,"Can't copy NULL handle");
	
#if XMPDebug
	if( gValidate>0 )
		if( !XMPValidateHandle(handle) )
			THROW(kXMPErrMemoryProblem);
	if( (HGetState(handle)&0x20) && !(HGetState(handle)&0x80) )
		WARN("Copying purgeable handle %p",handle);
#endif
		
	OSErr err = HandToHand((Handle*) &handle);
	if( err )
		if( err!=memFullErr )
			THROW(kXMPErrOutOfMemory);
		else {
			WARN("HandToHand returned %d",err);
			THROW(kXMPErrMemoryProblem);
		}
	return handle;
}

//------------------------------------------------------------------------------
// XMPGetHandleSize(XMPHandle handle)
//------------------------------------------------------------------------------

XMPULong XMPGetHandleSize(XMPHandle handle)
{
	XMPULong	size;
	
#if XMPDebug
	if( gValidate>0 )
		if( !XMPValidateHandle(handle) )
			THROW(kXMPErrMemoryProblem);
#endif
		
	size = GetHandleSize(handle);
	if( MemError() ) {
		WARN("GetHandleSize got err %d",MemError());
		THROW(kXMPErrMemoryProblem);
	}
	return size;
}

//------------------------------------------------------------------------------
// XMPSetHandleSize(XMPHandle handle, XMPULong size)
//------------------------------------------------------------------------------

void XMPSetHandleSize(XMPHandle handle, XMPULong size)
{
#if XMPDebug
	if( size>kMaxBlockSize ) {
		WARN("Bogus size %08lx for XMPSetHandleSize",size);
		THROW(kXMPErrOutOfMemory);
	}
	XMPULong oldSize;
	if( gValidate>0 ) {
		if( !XMPValidateHandle(handle) )
			THROW(kXMPErrMemoryProblem);
		// If block will shrink, zap area that will be lost:
		oldSize = GetHandleSize(handle);
		if( size < oldSize )
			XMPZapMem(OFFSET(*handle,size),oldSize-size);
	}
#endif
	
	SetHandleSize(handle, size);
	OSErr err = MemError();
	if( err ) {
		if( err==memFullErr )
			THROW(kXMPErrOutOfMemory);
		else {
			WARN("SetHandleSize got err %d",err);
			THROW(kXMPErrMemoryProblem);
		}
	}

#if XMPDebug
	// If block grew, zap new space at end:
	if( gValidate>0 )
		if( size > oldSize )
			memset(OFFSET(*handle,oldSize),0xBB,(size_t)(size-oldSize));	// Fill new space with BB
#endif
}

//------------------------------------------------------------------------------
// XMPLockHandle(XMPHandle handle)
//------------------------------------------------------------------------------

void* XMPLockHandle(XMPHandle handle)
{
#if XMPDebug
	if( gValidate>0 )
		if( !XMPValidateHandle(handle) )
			THROW(kXMPErrMemoryProblem);
#endif
		
	HLock(handle);
	
	return *handle;
}

//------------------------------------------------------------------------------
// XMPUnlockHandle(void* ptr)
//------------------------------------------------------------------------------

void XMPUnlockHandle(void* ptr)
{	
	ASSERTM(ptr!=NULL,kXMPErrInvalidValue,"Can't unlock a NULL ptr");
	
	if (ptr != kXMPNULL) {
		XMPHandle handle = RecoverHandle((Ptr) ptr);
		if( MemError() )
			WARN("RecoverHandle(%p) got err %d",ptr,MemError());
		else if (handle != kXMPNULL) {
#if XMPDebug
			if( gValidate>0 && !(HGetState(handle) & 0x80) )
				WARN("XMPUnlockHandle: Hdl %p already unlocked",h);
#endif
			HUnlock(handle);
		}
	}
}

//------------------------------------------------------------------------------
// XMPUnlockHandle(XMPHandle handle)
//------------------------------------------------------------------------------

void XMPUnlockHandle(XMPHandle handle)
{
#if XMPDebug
	if( gValidate>0 )
		if( !XMPValidateHandle(handle) )
			THROW(kXMPErrMemoryProblem);
#endif
		
	HUnlock(handle);
	if( MemError() )
		WARN("HUnlock(%p) got err %d",handle,MemError());
}



