/*
	File:		DrawSelection.h

	Contains:	Definition for class CDrawSelection

	Written by:	Henri Lamiraux

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<1+>	 3/17/94	DS		Make CSelectionIterator's constructor take
									constant parms
*/

#ifndef _DRAWSELECTION_
#define _DRAWSELECTION_

#ifndef _SHAPES_
#include "Shapes.h"
#endif

#ifndef _STYLE_
#include "Style.h"
#endif

// ----- FrameWork Includes -----

#ifndef _FWSELECT_
#include <FWSelect.h>
#endif

// ----- OpenDoc Includes -----

#ifndef _ORDCOLL_
#include <OrdColl.h>
#endif

//==============================================================================
//	 Forward Declarations
//==============================================================================

class CDrawPart;
class CDrawFrame;
class CFacet;
class CSelectionIterator;
class CDrawSelection;

//==============================================================================
//	 class CDrawPublishLink
//==============================================================================

class CDrawPublishLink : public CPublishLink
{
public:
	CDrawPublishLink(XMPChangeID changeID, CDrawSelection *drawSelection);
	virtual ~CDrawPublishLink();
	
	virtual void	Publish();
	virtual void	ExternalizeLinkContent(XMPStorageUnit* linkSU);
	virtual XMPValueType GetPublishFormat();
	
private:
	CDrawSelection*			fDrawSelection;
	OrderedCollection*		fCollection;
};

//==============================================================================
//	 class CDrawSubscribLink
//==============================================================================

class CDrawSubscribLink : public CSubscribLink
{
public:
	CDrawSubscribLink(XMPLink *xmpLink, CDrawSelection *drawSelection);
	virtual ~CDrawSubscribLink();

	virtual void	Subscrib();
	
private:
	CDrawSelection*			fDrawSelection;
	OrderedCollection*		fCollection;
};

//==============================================================================
//	 class CDrawSelection
//==============================================================================

class CDrawSelection : public FW_CSelection
{
	friend	CSelectionIterator;
	
//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
public:
	CDrawSelection();
	void			InitDrawSelection(CDrawPart* drawPart);
	virtual 		~CDrawSelection();
	
//------------------------------------------------------------------------------
//	 Inherited API
//
public:
	virtual void		CloseSelection();
	virtual void		SelectAll();
	virtual XMPBoolean	IsEmpty() const;
	virtual CProxyRun*	IsSelectionOnlyOneProxyRun() const;
	virtual XMPShape* 	GetSelectionShape();
	
	// ----- Cut/Copy/Paste/Clear -----
	virtual XMPBoolean	DoClear();
	virtual XMPBoolean	DoPaste();
	virtual XMPBoolean	DoPasteWithLink();
	
	virtual void		ExternalizeSelection(XMPStorageUnit* storageUnit, CFrame* commandFrame, XMPCloneKind kind);
	virtual XMPBoolean	InternalizeSelection(XMPStorageUnit* storageUnit, XMPCloneKind kind);
	virtual XMPBoolean	EmbedSelection(XMPPart* xmpPart, XMPShape* frameShape);

	// ----- Drag & Drop -----
	virtual XMPRgnHandle CalcDragRgn(CFacet* facet);
	virtual XMPBoolean	DoDroppedInSameFrame(XMPStorageUnit* dropSU, CFacet* facet, XMPPoint originPoint, XMPPoint dropPoint);
	virtual XMPBoolean	DoDrop(XMPStorageUnit* dropSU, CFacet* facet, XMPPoint originPoint, XMPPoint dropPoint);

 	// ----- Linking -----
	virtual CPublishLink* NewPublishLink(XMPChangeID changeID);
	virtual CSubscribLink* NewSubscribLink(XMPLink *xmpLink);
	virtual XMPBoolean		IsSelectionPublishable();
	
//------------------------------------------------------------------------------
//	 New API
//
public:
	CBaseShape* 	WhichHandle(CFacet* facet, Point mouse, short& whichHandle) const;
	
	void			DrawSelectionHandles(CFacet* facet, XMPBoolean turOn);
	void			DrawHandles(CBaseShape* shape, XMPBoolean turOn);
	void			DrawAllHandles(CFrame* frame, XMPBoolean turOn);
	
	void			AddToSelection(CBaseShape* theShape, XMPBoolean drawHandles);
	void			RemoveFromSelection(CBaseShape* theShape, XMPBoolean drawHandles);

	void			MapSelection(const Rect& srcRect, const Rect& dstRect);
	void			OffsetSelection(XMPCoordinate xDelta, XMPCoordinate yDelta);

	void			Resize(short whichHandle, CFacet* facet, Point firstPoint, XMPEventData event);	

	void			SelectWithRectangle(CFacet* facet, Point anchorPoint, XMPEventData event);

	void			SetSelectedShapesPenSize(XMPUShort newPenSize);
	void			SetSelectedShapesStyle(XMPUShort newStyle);

	XMPBoolean		HasProxySelected() const;
	
	XMPULong		Count() const;
	
	// ----- Frozen state -----
	XMPBoolean		HasFrozen() const;
	XMPBoolean		HasNotFrozen() const;
	void			SetFrozen(XMPBoolean state);
	
	XMPShape*		GetUpdateShape() const;

	// ----- Anchor Shape -----
	CBaseShape*		GetAnchorShape() const;
	void			SetAnchorShape(CBaseShape* anchorShape);

	// ----- Publish -----
	void			SelectionChanged();

private:
	void			DragFeedback(RgnHandle dragRgn, short xDelta, short yDelta);
	
	void			CalcDragRect();
	void			CalcUpdateShape();
	void			CalcMapRect(XMPSShort whichHandle, Point handleCenter, Point penSize, Rect& mapRect);
	
	void			RedrawShape(CBaseShape *shape);
	
	void			DoAdd(CBaseShape *shape);
	void			DoRemove(CBaseShape *shape);
	
	void			AdjustSelectionAfterPaste();
	
//------------------------------------------------------------------------------
//	 Data Members
//
private:
	CDrawPart*			fDrawPart;
	Rect				fDragRect;
	XMPShape*			fSelectionShape;
	XMPShape*			fUpdateShape;
	XMPUShort			fProxyShapeCount;
	XMPUShort			fFrozenCount;
	XMPULong			fCount;
	CBaseShape*			fAnchorShape;
	
	OrderedCollection*	fCollection;
};

//==============================================================================
//	 class CSelectionIterator
//==============================================================================

class CSelectionIterator
{
public:
	CSelectionIterator(const CDrawSelection* selection);
	virtual ~CSelectionIterator();

	CBaseShape*		First();
	CBaseShape*		Next();
	CBaseShape*		Last();
	CBaseShape*		Previous();
	XMPBoolean		IsNotComplete();
	
private:
	OrderedCollectionIterator fIterator;
};

//==============================================================================
//	 CDrawSelection Inlines
//==============================================================================

//------------------------------------------------------------------------------
//	 CDrawSelection::HasProxySelected
//------------------------------------------------------------------------------
inline XMPBoolean CDrawSelection::HasProxySelected() const
{		
	return fProxyShapeCount != 0;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::GetUpdateShape
//------------------------------------------------------------------------------
inline XMPShape* CDrawSelection::GetUpdateShape() const
{		
	return fUpdateShape;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::HasFrozen
//------------------------------------------------------------------------------
inline XMPBoolean CDrawSelection::HasFrozen() const
{
	return fFrozenCount != 0;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::HasNotFrozen
//------------------------------------------------------------------------------
inline XMPBoolean CDrawSelection::HasNotFrozen() const
{
	return fCount != 0 && fFrozenCount != fCount;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::Count
//------------------------------------------------------------------------------
inline XMPULong CDrawSelection::Count() const
{
	return fCount;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::GetAnchorShape
//------------------------------------------------------------------------------
inline CBaseShape* CDrawSelection::GetAnchorShape() const
{
	return fAnchorShape;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::SetAnchorShape
//------------------------------------------------------------------------------
inline void CDrawSelection::SetAnchorShape(CBaseShape* anchorShape)
{
	fAnchorShape = anchorShape;
}

//==============================================================================
//	 CSelectionIterator Inlines
//==============================================================================

//------------------------------------------------------------------------------
//	 CSelectionIterator::First
//------------------------------------------------------------------------------
inline CBaseShape* CSelectionIterator::First()
{
	return (CBaseShape*)fIterator.First();
}
	
//------------------------------------------------------------------------------
//	 CSelectionIterator::Next
//------------------------------------------------------------------------------
inline CBaseShape* CSelectionIterator::Next()
{
	return (CBaseShape*)fIterator.Next();
}

//------------------------------------------------------------------------------
//	 CSelectionIterator::Last
//------------------------------------------------------------------------------
inline CBaseShape* CSelectionIterator::Last()
{
	return (CBaseShape*)fIterator.Last();
}
	
//------------------------------------------------------------------------------
//	 CSelectionIterator::Next
//------------------------------------------------------------------------------
inline CBaseShape* CSelectionIterator::Previous()
{
	return (CBaseShape*)fIterator.Previous();
}

//------------------------------------------------------------------------------
//	 CSelectionIterator::IsNotComplete
//------------------------------------------------------------------------------
inline XMPBoolean CSelectionIterator::IsNotComplete()
{
	return fIterator.IsNotComplete();
}


#endif