/*
	File:		Shapes.cpp

	Contains:	Implementation of classes:
						CBaseShape
						CLineShape

	Written by:	Henri Lamiraux

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

*/

#ifndef _SHAPES_
#include "Shapes.h"
#endif

#ifndef _UTILITIES_
#include "Utilities.h"
#endif

#ifndef _QDRAWPART_
#include "QDrawPart.h"
#endif

#ifndef _DRAWSELECTION_
#include "DrawSelection.h"
#endif

// ----- Framework Includes -----
#ifndef _FWPROXY_
#include "FWProxy.h"
#endif

#ifndef _FWBORDER_
#include "FWBorder.h"
#endif

#ifndef _FWFRMING_
#include "FWFrming.h"
#endif

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

#ifndef _FWPART_
#include "FWPart.h"
#endif

#ifndef _FWFACET_
#include "FWFacet.h"
#endif

#ifndef _FWSELECT_
#include "FWSelect.h"
#endif

// ----- OpenDoc Includes

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _FRAME_
#include <Frame.h>
#endif

#ifndef _FACET_
#include <Facet.h>
#endif

#ifndef _DRAFT_
#include <Draft.h>
#endif

// ----- Macintosh Includes -----

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#ifndef __DRAG__
#include <Drag.h>
#endif

#ifndef mathRoutinesIncludes
#include <math routines.h>				// for ff()
#endif

#pragma segment qdrawpart

//==============================================================================
// class CBaseShape
//==============================================================================

//-------------------------------------------------------------------------
// CBaseShape::CBaseShape
//-------------------------------------------------------------------------

CBaseShape::CBaseShape(XMPSShort numberOfHandles, XMPUShort shapeType) :
	fSelected(kXMPFalse),
	fClipShape(kXMPNULL),
	fNumberOfHandles(numberOfHandles),
	fShapeType(shapeType),
	fPublishLink(kXMPNULL),
	fSubscribLink(kXMPNULL)
{
}

//-------------------------------------------------------------------------
// CBaseShape::~CBaseShape
//-------------------------------------------------------------------------

CBaseShape::~CBaseShape()
{
	delete fPublishLink;
	delete fSubscribLink;
	
	delete fClipShape;
}

//-------------------------------------------------------------------------
// CBaseShape::Removed
//-------------------------------------------------------------------------

void CBaseShape::Removed()
{
}

//-------------------------------------------------------------------------
// CBaseShape::Track
//-------------------------------------------------------------------------

XMPBoolean CBaseShape::Track(const CShapeStyle& trackStyle, Point anchorPoint, XMPEventData event)
{
	if (!::WaitMouseMoved(event->where))
		return kXMPFalse;

	::PenNormal();
	trackStyle.SetGrafPortFrameStyle();
	::PenMode(patXor);
	
	Point currentLoc;
	Point prevLoc = anchorPoint;
	
	XMPBoolean stillDown = kXMPTrue;
	XMPBoolean erase = kXMPFalse;
	
	while (stillDown)
	{
		::GetMouse(&currentLoc);
		
		// ----- Adjust for the size of the cursor
		if (anchorPoint.h < currentLoc.h)
			currentLoc.h++;
		if (anchorPoint.v < currentLoc.v)
			currentLoc.v++;
		
		// ----- Test if moved	
		if ((prevLoc.h != currentLoc.h) || (prevLoc.v != currentLoc.v))
		{
			if (erase)
				TrackFeedback(anchorPoint, prevLoc, kXMPTrue);		// erase
			
			TrackFeedback(anchorPoint, currentLoc, kXMPFalse);		// draw
			
			WaitForTick();
			
			prevLoc = currentLoc;
			erase = kXMPTrue;
		}
		
		stillDown = ::StillDown();
	}
	
	if (erase)
		TrackFeedback(anchorPoint, currentLoc, kXMPTrue);	// Erase
		
	::PenNormal();
	
	return (currentLoc.h != anchorPoint.h) || (currentLoc.v != anchorPoint.v);
}

//-------------------------------------------------------------------------
// CBaseShape::SelectShape
//-------------------------------------------------------------------------

void CBaseShape::SelectShape(XMPBoolean state)
{
	fSelected = state;
}

//-------------------------------------------------------------------------
// CBaseShape::ClearCache
//-------------------------------------------------------------------------

void CBaseShape::ClearCache()
{
	delete fClipShape;
	fClipShape = kXMPNULL;
}

//-------------------------------------------------------------------------
// CBaseShape::DrawShapeHandles
//-------------------------------------------------------------------------

void CBaseShape::DrawShapeHandles(CFacet* facet, XMPBoolean /*turOn*/)
{	
XMPUnused(facet);

	for (XMPSShort i=1; i<=fNumberOfHandles; i++)
	{
		DrawHandle(i);
	}
}

//-------------------------------------------------------------------------
// CBaseShape::WhichHandle
//-------------------------------------------------------------------------

XMPSShort CBaseShape::WhichHandle(CFacet* facet, Point mouse) const
{
XMPUnused(facet);

	Rect handleRect;
	
	for (XMPSShort i=1; i<=fNumberOfHandles; i++)
	{
		CalcHandleRect(i, handleRect);
		if (::PtInRect(mouse, &handleRect))
			return i;
	}
		
	return 0;
}

//-------------------------------------------------------------------------
// CBaseShape::CalcHandleRect
//-------------------------------------------------------------------------

void CBaseShape::CalcHandleRect(XMPSShort whichHandle, Rect& handleRect) const
{
	Point pt;
	GetHandleCenter(whichHandle, pt);
	::SetRect(&handleRect, 
			  pt.h-2, pt.v-2,
			  pt.h+3, pt.v+3);
}

//-------------------------------------------------------------------------
// CBaseShape::DrawHandle
//-------------------------------------------------------------------------

void CBaseShape::DrawHandle(XMPSShort whichHandle)
{
	Rect handleRect;
	CalcHandleRect(whichHandle, handleRect);
	::InvertRect(&handleRect);
}

//-------------------------------------------------------------------------
// CBaseShape::SetShapeStyle
//-------------------------------------------------------------------------

void CBaseShape::SetShapeStyle(const CShapeStyle& shapeStyle)
{
	ClearCache();
	fShapeStyle = shapeStyle;
}

//-------------------------------------------------------------------------
// CBaseShape::GetPenSize
//-------------------------------------------------------------------------

void CBaseShape::GetPenSize(Point& penSize) const
{
	fShapeStyle.GetPenSize(penSize);
}

//-------------------------------------------------------------------------
// CBaseShape::InSelectionRect
//-------------------------------------------------------------------------

XMPBoolean CBaseShape::InSelectionRect(const Rect& selectRect) const
{
	Rect bounds,result;
	GetBoundingBox(bounds);
	if (::SectRect(&bounds, &selectRect, &result))
	{
		return ::EqualRect(&bounds, &result);
	}
	return kXMPFalse;
}

//-------------------------------------------------------------------------
// CBaseShape::Externalize
//-------------------------------------------------------------------------

void CBaseShape::Externalize(XMPStorageUnit* storage)
{	
	storage->SetValue(sizeof(fShapeType), (XMPValue)&fShapeType);
	storage->SetValue(sizeof(fSelected), (XMPValue)&fSelected);
	storage->SetValue(sizeof(fNumberOfHandles), (XMPValue)&fNumberOfHandles);
	fShapeStyle.Externalize(storage);
}

//-------------------------------------------------------------------------
// CBaseShape::Internalize
//-------------------------------------------------------------------------

void CBaseShape::Internalize(CDrawPart* drawPart, XMPStorageUnit* storage)
{	
XMPUnused(drawPart);

//	We don't internalize the shape type because it was already read
//	storage->GetValue(sizeof(fShapeType), (XMPValue)&fShapeType);
	storage->GetValue(sizeof(fSelected), (XMPValue)&fSelected);
	storage->GetValue(sizeof(fNumberOfHandles), (XMPValue)&fNumberOfHandles);
	fShapeStyle.Internalize(storage);
}

//-------------------------------------------------------------------------
// CBaseShape::CloneTo
//-------------------------------------------------------------------------

void CBaseShape::CloneTo(XMPStorageUnit* storageUnit, CFrame* commandFrame, XMPCloneKind cloneKind)
{	
XMPUnused(commandFrame);
XMPUnused(cloneKind);

	Externalize(storageUnit);
}

//-------------------------------------------------------------------------
// CBaseShape::CloneFrom
//-------------------------------------------------------------------------

void CBaseShape::CloneFrom(CDrawPart* drawPart, XMPStorageUnit* storageUnit, XMPCloneKind cloneKind)
{	
XMPUnused(cloneKind);

	Internalize(drawPart, storageUnit);
}

//-------------------------------------------------------------------------
// CBaseShape::MapPoint
//
//	srcRect must be sorted
//-------------------------------------------------------------------------

void CBaseShape::MapPoint(Point &pt, const Rect& srcRect, const Rect& destRect)
{
	Rect tempRect = destRect;
	::SortRect(tempRect);
	
	::MapPt(&pt, &srcRect, &tempRect);
	
	if (destRect.right < destRect.left)
	{
		pt.h = destRect.right + destRect.left - pt.h;
	}
	
	if (destRect.bottom < destRect.top)
	{
		pt.v = destRect.bottom + destRect.top - pt.v;
	}
}

//-------------------------------------------------------------------------
// CBaseShape::MovedAfter
//-------------------------------------------------------------------------

void CBaseShape::MovedAfter(CBaseShape* shape)
{
XMPUnused(shape);
}

//-------------------------------------------------------------------------
// CBaseShape::MovedBefore
//-------------------------------------------------------------------------

void CBaseShape::MovedBefore(CBaseShape* shape)
{
XMPUnused(shape);
}

//-------------------------------------------------------------------------
// CBaseShape::MovedFirst
//-------------------------------------------------------------------------

void CBaseShape::MovedFirst()
{
}

//-------------------------------------------------------------------------
// CBaseShape::MovedLast
//-------------------------------------------------------------------------

void CBaseShape::MovedLast()
{
}

//-------------------------------------------------------------------------
// CBaseShape::SetFrozen
//-------------------------------------------------------------------------

XMPBoolean CBaseShape::SetFrozen(XMPBoolean state)
{
XMPUnused(state);
	return kXMPFalse;	// Means I don't care
}

//-------------------------------------------------------------------------
// CBaseShape::IsFrozen
//-------------------------------------------------------------------------

XMPBoolean CBaseShape::IsFrozen() const
{
	return kXMPFalse;
}

//=========================================================================
// class CLineShape
//=========================================================================

//-------------------------------------------------------------------------
// CLineShape::CLineShape
//-------------------------------------------------------------------------

CLineShape::CLineShape() :
	CBaseShape(2, kLineShape)
{
}

//-------------------------------------------------------------------------
// CLineShape::~CLineShape
//-------------------------------------------------------------------------

CLineShape::~CLineShape()
{
}

//-------------------------------------------------------------------------
// CLineShape::DrawShape
//-------------------------------------------------------------------------

void CLineShape::DrawShape(CFacet* /*facet*/, XMPShape* invalidShape)
{
	fShapeStyle.SetGrafPortFrameStyle();
	
	::MoveTo(fAnchorPoint.h, fAnchorPoint.v);
	::LineTo(fEndPoint.h, fEndPoint.v);

	// ----- Restore the grafport -----
	::PenNormal();
	::RGBForeColor(&kRGBBlack);
	
	// ----- Remove what I have drawn
	if (invalidShape)
		invalidShape->Subtract(GetClipShape());
}

//-------------------------------------------------------------------------
// CLineShape::OutlineShape
//-------------------------------------------------------------------------

void CLineShape::OutlineShape(Point pt1, Point pt2)
{
	::MoveTo(pt1.h, pt1.v);
	::LineTo(pt2.h, pt2.v);
}

//-------------------------------------------------------------------------
// CLineShape::HitTest
//-------------------------------------------------------------------------

XMPBoolean CLineShape::HitTest(CFacet* facet, Point mouse, XMPEventData event) const
{
XMPUnused(event);
XMPUnused(facet);

	Rect rect;
	GetBoundingBox(rect);
	::InsetRect(&rect, -2, -2);
	
	XMPBoolean result = kXMPFalse;
	
	if (::PtInRect(mouse, &rect))
	{
		long dx = (long)(fEndPoint.h - fAnchorPoint.h);
		long dy = (long)(fEndPoint.v - fAnchorPoint.v);
		
		if (dx == 0)		// vertical line
		{
			if ((fEndPoint.v <= mouse.v && mouse.v <= fAnchorPoint.v) || 
				(fAnchorPoint.v <= mouse.v && mouse.v <= fEndPoint.v))
			{
				result = Abs(fEndPoint.h - mouse.h) < 3;
			}
		}
		else if (dy == 0)	// horizontal line
		{
			if ((fEndPoint.h <= mouse.h && mouse.h <= fAnchorPoint.h) || 
				(fAnchorPoint.h <= mouse.h && mouse.h <= fEndPoint.h))
			{
				result = Abs(fEndPoint.v - mouse.v) < 3;
			}
		}
		else
		{
			long a2 = dx * (mouse.v - fAnchorPoint.v) - dy * (mouse.h - fAnchorPoint.h);
			if (a2<0) a2 *= -1;
			
			if (dx<0) dx *= -1;
			if (dy<0) dy *= -1;
			
			long v;
			if (dx < dy) v = dx / 2;
			else v = dy / 2;
			
			result = (a2 / (v + dx + dy)) < 3;
		}
	}
	
	return result;
}

//-------------------------------------------------------------------------
// CLineShape::GetUpdateShape
//-------------------------------------------------------------------------

void CLineShape::GetUpdateShape(XMPShape* updateShape) const
{
	Rect rect;
	GetBoundingBox(rect);
	
	if (IsSelected())
		::InsetRect(&rect, -2, -2);
	
	XMPRect xmpRect(rect);
	updateShape->SetRectangle(&xmpRect);
}

//-------------------------------------------------------------------------
// CLineShape::GetBoundingBox
//-------------------------------------------------------------------------

void CLineShape::GetBoundingBox(Rect& bounds) const
{
	Point penSize;
	GetPenSize(penSize);
	
	::SetRect(&bounds, fAnchorPoint.h, fAnchorPoint.v, fEndPoint.h, fEndPoint.v);
	::SortRect(bounds);
	bounds.right += penSize.h;
	bounds.bottom += penSize.v;
}

//-------------------------------------------------------------------------
// CLineShape::GetClipShape
//-------------------------------------------------------------------------

XMPShape* CLineShape::GetClipShape()
{
	if (fClipShape == kXMPNULL)
	{
		RgnHandle rgn = ::NewRgn();
		GetDragRgn(rgn);
		fClipShape = ::NewXMPShape(rgn);
	}
	
	return fClipShape;
}

//-------------------------------------------------------------------------
// MakeLineRgn
//-------------------------------------------------------------------------

void MakeLineRgn(Point from, Point to, Point penSize)
{
	::MoveTo(from.h, from.v);
	::LineTo(to.h, to.v);
	::Line(penSize.h, 0);
	::Line(0, penSize.v);
	::LineTo(from.h + penSize.h, from.v + penSize.v);
	::Line(-penSize.h, 0);
	::Line(0, -penSize.v);
}

//-------------------------------------------------------------------------
// CLineShape::GetDragRgn
//-------------------------------------------------------------------------

void CLineShape::GetDragRgn(RgnHandle dragRgn) const
{
	Point penSize;
	GetPenSize(penSize);
	
	::OpenRgn();
	
	if (fEndPoint.h < fAnchorPoint.h)
	{
		if (fEndPoint.v < fAnchorPoint.v)
		{
			::MoveTo(fAnchorPoint.h + penSize.h, fAnchorPoint.v);
			::LineTo(fEndPoint.h + penSize.h, fEndPoint.v);
			::Line(-penSize.h, 0);			
			::Line(0, penSize.v);
			::LineTo(fAnchorPoint.h, fAnchorPoint.v + penSize.v);			
			::Line(-penSize.h, 0);
			::Line(0, -penSize.v);
		}
		else if (fEndPoint.v > fAnchorPoint.v)
		{
			::MakeLineRgn(fAnchorPoint, fEndPoint, penSize);		
		}
		else
		{
			::MakeLineRgn(fEndPoint, fAnchorPoint, penSize);		
		}
	}
	else if (fEndPoint.h > fAnchorPoint.h)
	{
		if (fEndPoint.v < fAnchorPoint.v)
		{
			::MakeLineRgn(fAnchorPoint, fEndPoint, penSize);		
		}
		else if (fEndPoint.v > fAnchorPoint.v)
		{
			::MoveTo(fAnchorPoint.h, fAnchorPoint.v);
			::Line(penSize.h, 0);
			::LineTo(fEndPoint.h + penSize.h, fEndPoint.v);
			::Line(0, penSize.v);
			::Line(-penSize.h, 0);
			::LineTo(fAnchorPoint.h, fAnchorPoint.v + penSize.v);
			::Line(0, -penSize.v);
		}
		else
		{
			::MakeLineRgn(fAnchorPoint, fEndPoint, penSize);		
		}
	}
	else
	{
		if (fEndPoint.v < fAnchorPoint.v)
		{
			::MakeLineRgn(fAnchorPoint, fEndPoint, penSize);		
		}
		else if (fEndPoint.v > fAnchorPoint.v)
		{
			::MakeLineRgn(fEndPoint, fAnchorPoint, penSize);		
		}
		else
		{
			::MoveTo(fAnchorPoint.h, fAnchorPoint.v);
			::Line(penSize.h, 0);
			::Line(0, penSize.v);
			::Line(-penSize.h, 0);
			::Line(0, -penSize.v);			
		}
	}
	
	::CloseRgn(dragRgn);
}

//-------------------------------------------------------------------------
// CLineShape::GetHandleCenter
//-------------------------------------------------------------------------

void CLineShape::GetHandleCenter(XMPSShort whichHandle, Point& center) const
{
	if (whichHandle == 1)
		center = fAnchorPoint;
	else
		center = fEndPoint;
}

//-------------------------------------------------------------------------
// CLineShape::ResizeFeedback
//-------------------------------------------------------------------------

void CLineShape::ResizeFeedback(const Rect& srcRect, const Rect& dstRect)
{
	Point pt1, pt2;
	pt1 = fAnchorPoint;
	pt2 = fEndPoint;

	::MapPoint(pt1, srcRect, dstRect);
	::MapPoint(pt1, srcRect, dstRect);

	OutlineShape(pt1, pt2);
}

//-------------------------------------------------------------------------
// CLineShape::MapShape
//-------------------------------------------------------------------------

void CLineShape::MapShape(const Rect& srcRect, const Rect& dstRect)
{
	ClearCache();
	::MapPoint(fAnchorPoint, srcRect, dstRect);
	::MapPoint(fEndPoint, srcRect, dstRect);	
}

//-------------------------------------------------------------------------
// CLineShape::OffsetShape
//-------------------------------------------------------------------------

void CLineShape::OffsetShape(XMPCoordinate xDelta, XMPCoordinate yDelta)
{
	ClearCache();
	
	short x =  FixedToInt(xDelta);
	short y =  FixedToInt(yDelta);
	
	fAnchorPoint.h += x;
	fAnchorPoint.v += y;
	fEndPoint.h += x;
	fEndPoint.v += y;
}

//-------------------------------------------------------------------------
// CLineShape::TrackFeedback
//-------------------------------------------------------------------------

void CLineShape::TrackFeedback(Point anchorPoint, Point currentPoint, XMPBoolean erase)
{
	if (!erase)
	{	
		fAnchorPoint = anchorPoint;
		fEndPoint = currentPoint;
	}
		
	OutlineShape(fAnchorPoint, fEndPoint);	
}

//-------------------------------------------------------------------------
// CLineShape::Externalize
//-------------------------------------------------------------------------

void CLineShape::Externalize(XMPStorageUnit* storage)
{	
	CBaseShape::Externalize(storage);
	storage->SetValue(sizeof(fAnchorPoint), (XMPValue)&fAnchorPoint);
	storage->SetValue(sizeof(fEndPoint), (XMPValue)&fEndPoint);
}

//-------------------------------------------------------------------------
// CLineShape::Internalize
//-------------------------------------------------------------------------

void CLineShape::Internalize(CDrawPart* drawPart, XMPStorageUnit* storage)
{	
	CBaseShape::Internalize(drawPart, storage);
	storage->GetValue(sizeof(fAnchorPoint), (XMPValue)&fAnchorPoint);
	storage->GetValue(sizeof(fEndPoint), (XMPValue)&fEndPoint);
}

//==============================================================================
// class CRectShape
//==============================================================================

//-------------------------------------------------------------------------
// CRectShape::CRectShape
//-------------------------------------------------------------------------

CRectShape::CRectShape() :
	CBaseShape(4, kRectShape)
{
}

//-------------------------------------------------------------------------
// CRectShape::CRectShape
//-------------------------------------------------------------------------

CRectShape::CRectShape(XMPUShort shapeType) :
	CBaseShape(4, shapeType)
{
}

//-------------------------------------------------------------------------
// CRectShape::~CRectShape
//-------------------------------------------------------------------------

CRectShape::~CRectShape()
{
}

//-------------------------------------------------------------------------
// CRectShape::DrawShape
//-------------------------------------------------------------------------

void CRectShape::DrawShape(CFacet* /*facet*/, XMPShape* invalidShape)
{
	if (fShapeStyle.HasFillStyle())
	{
		fShapeStyle.SetGrafPortFillStyle();
		::PaintRect(&fRect);
	}
	
	if (fShapeStyle.HasFrameStyle())
	{
		fShapeStyle.SetGrafPortFrameStyle();
		::FrameRect(&fRect);
	}
	
	// ----- Restore the grafport -----
	::PenNormal();
	::RGBForeColor(&kRGBBlack);
	
	// ----- Remove what I have drawn
	if (invalidShape)
		invalidShape->Subtract(GetClipShape());
}

//-------------------------------------------------------------------------
// CRectShape::OutlineShape
//-------------------------------------------------------------------------

void CRectShape::OutlineShape(const Rect& rect)
{
	::FrameRect(&rect);
}

//-------------------------------------------------------------------------
// CRectShape::HitTest
//-------------------------------------------------------------------------

XMPBoolean CRectShape::HitTest(CFacet* facet, Point mouse, XMPEventData event) const
{
XMPUnused(event);
XMPUnused(facet);

	XMPBoolean result = kXMPFalse;
	
	if (fShapeStyle.HasFillStyle())
	{
		result = ::PtInRect(mouse, &((CRectShape*)this)->fRect);
	}
	else
	{
		Rect rect = fRect;
		::InsetRect(&rect, -2, -2);
		if (::PtInRect(mouse, &rect))
		{
			::InsetRect(&rect, 2 + fShapeStyle.GetPenWidth() + 2, 2 + fShapeStyle.GetPenHeight() + 2);
			result = !::PtInRect(mouse, &rect);
		}
	}
	
	return result;
}

//-------------------------------------------------------------------------
// CRectShape::GetUpdateShape
//-------------------------------------------------------------------------

void CRectShape::GetUpdateShape(XMPShape* updateShape) const
{
	Rect rect = fRect;
	if (IsSelected())
		::InsetRect(&rect, -2, -2);
	XMPRect xmpRect(rect);
	updateShape->SetRectangle(&xmpRect);
}

//-------------------------------------------------------------------------
// CRectShape::GetBoundingBox
//-------------------------------------------------------------------------

void CRectShape::GetBoundingBox(Rect& bounds) const
{
	bounds = fRect;
}

//-------------------------------------------------------------------------
// CRectShape::GetClipShape
//-------------------------------------------------------------------------

XMPShape* CRectShape::GetClipShape()
{
	if (fClipShape == kXMPNULL)
	{
		RgnHandle rgn = ::NewRgn();
		XMPUShort frameStyle = fShapeStyle.GetFrameFillStyle();
		if (frameStyle == kFrameOnly)
		{
			GetDragRgn(rgn);
		}
		else
		{
			::OpenRgn();
			((CRectShape*)this)->OutlineShape(fRect);
			::CloseRgn(rgn);
		}
		fClipShape = ::NewXMPShape(rgn);
	}
	
	return fClipShape;
}

//-------------------------------------------------------------------------
// CRectShape::GetDragRgn
//-------------------------------------------------------------------------

void CRectShape::GetDragRgn(RgnHandle dragRgn) const
{
	Point penSize;
	GetPenSize(penSize);
	
	::OpenRgn();
	((CRectShape*)this)->OutlineShape(fRect);
	::CloseRgn(dragRgn);
	
	RgnHandle tempRgn = ::NewRgn();
	::CopyRgn(dragRgn, tempRgn);
	::InsetRgn(tempRgn, penSize.h, penSize.v);
	::DiffRgn(dragRgn, tempRgn, dragRgn);
	::DisposeRgn(tempRgn);
}

//-------------------------------------------------------------------------
// CRectShape::GetHandleCenter
//-------------------------------------------------------------------------

void CRectShape::GetHandleCenter(XMPSShort whichHandle, Point& center) const
{
	switch (whichHandle)
	{
		case kInTopLeftCorner:
			center.h = fRect.left;
			center.v = fRect.top;
			break;
		case kInTopRightCorner:
			center.h = fRect.right-1;
			center.v = fRect.top;
			break;
		case kInBottomLeftCorner:
			center.h = fRect.left;
			center.v = fRect.bottom-1;
			break;
		case kInBottomRightCorner:
			center.h = fRect.right-1;
			center.v = fRect.bottom-1;
			break;
	}
}

//-------------------------------------------------------------------------
// CRectShape::ResizeFeedback
//-------------------------------------------------------------------------

void CRectShape::ResizeFeedback(const Rect& originalRect, const Rect& mapRect)
{
XMPUnused(originalRect);

	Rect rect = mapRect;
	::SortRect(rect);
	OutlineShape(rect);
}

//-------------------------------------------------------------------------
// CRectShape::MapShape
//-------------------------------------------------------------------------

void CRectShape::MapShape(const Rect& srcRect, const Rect& dstRect)
{
	ClearCache();
	
	XMPSLong deno = srcRect.right - srcRect.left;
	XMPSLong num = fRect.right - fRect.left;
	fRect.left += (XMPSShort)((num * (XMPSLong)(dstRect.left - srcRect.left)) / deno);
	fRect.right += (XMPSShort)((num * (XMPSLong)(dstRect.right - srcRect.right)) / deno);
	
	deno = srcRect.bottom - srcRect.top;
	num = fRect.bottom - fRect.top;
	fRect.top += (XMPSShort)((num * (XMPSLong)(dstRect.top - srcRect.top)) / deno);
	fRect.bottom += (XMPSShort)((num * (XMPSLong)(dstRect.bottom - srcRect.bottom)) / deno);

	::SortRect(fRect);
	
	Point penSize;
	GetPenSize(penSize);
	if (fRect.right == fRect.left)
		fRect.right = fRect.left + penSize.h;
	if (fRect.bottom == fRect.top)
		fRect.bottom = fRect.top + penSize.h;
}

//-------------------------------------------------------------------------
// CRectShape::OffsetShape
//-------------------------------------------------------------------------

void CRectShape::OffsetShape(XMPCoordinate xDelta, XMPCoordinate yDelta)
{
	ClearCache();

	::OffsetRect(&fRect, FixedToInt(xDelta), FixedToInt(yDelta));
}

//-------------------------------------------------------------------------
// CRectShape::TrackFeedback
//-------------------------------------------------------------------------

void CRectShape::TrackFeedback(Point anchorPoint, Point currentPoint, XMPBoolean erase)
{
	if (!erase)
		CalcRect(anchorPoint, currentPoint, fRect);
	OutlineShape(fRect);
}

//-------------------------------------------------------------------------
// CRectShape::CalcRect
//-------------------------------------------------------------------------

void CRectShape::CalcRect(Point anchorPoint, Point endPoint, Rect& rect)
{
	if (anchorPoint.h < endPoint.h)
	{
		rect.left = anchorPoint.h;
		rect.right = endPoint.h;
	}
	else
	{
		rect.left = endPoint.h;
		rect.right = anchorPoint.h + XMPASLMQDGlobals.thePort->pnSize.h;
	}
	
	if (anchorPoint.v < endPoint.v)
	{
		rect.top = anchorPoint.v;
		rect.bottom = endPoint.v;
	}
	else
	{
		rect.top = endPoint.v;
		rect.bottom = anchorPoint.v + XMPASLMQDGlobals.thePort->pnSize.v;
	}
}

//-------------------------------------------------------------------------
// CRectShape::Externalize
//-------------------------------------------------------------------------

void CRectShape::Externalize(XMPStorageUnit* storage)
{	
	CBaseShape::Externalize(storage);
	storage->SetValue(sizeof(fRect), (XMPValue)&fRect);
}

//-------------------------------------------------------------------------
// CRectShape::Internalize
//-------------------------------------------------------------------------

void CRectShape::Internalize(CDrawPart* drawPart, XMPStorageUnit* storage)
{	
	CBaseShape::Internalize(drawPart, storage);
	storage->GetValue(sizeof(fRect), (XMPValue)&fRect);
}

//==============================================================================
// class COvalShape
//==============================================================================

//-------------------------------------------------------------------------
// COvalShape::COvalShape
//-------------------------------------------------------------------------

COvalShape::COvalShape():
	CRectShape(kOvalShape)
{
}

//-------------------------------------------------------------------------
// COvalShape::~COvalShape
//-------------------------------------------------------------------------

COvalShape::~COvalShape()
{
}

//-------------------------------------------------------------------------
// COvalShape::DrawShape
//-------------------------------------------------------------------------

void COvalShape::DrawShape(CFacet* /*facet*/, XMPShape* invalidShape)
{
	if (fShapeStyle.HasFillStyle())
	{
		fShapeStyle.SetGrafPortFillStyle();
		::PaintOval(&fRect);
	}
	
	if (fShapeStyle.HasFrameStyle())
	{
		fShapeStyle.SetGrafPortFrameStyle();
		::FrameOval(&fRect);
	}

	// ----- Restore the grafport -----
	::PenNormal();
	::RGBForeColor(&kRGBBlack);
	
	// ----- Remove what I have drawn
	if (invalidShape)
		invalidShape->Subtract(GetClipShape());
}

//-------------------------------------------------------------------------
// COvalShape::OutlineShape
//-------------------------------------------------------------------------

void COvalShape::OutlineShape(const Rect& rect)
{
	::FrameOval(&rect);
}

//-------------------------------------------------------------------------
// COvalShape::HitTest
//-------------------------------------------------------------------------

XMPBoolean COvalShape::HitTest(CFacet* facet, Point mouse, XMPEventData event) const
{	
XMPUnused(event);
XMPUnused(facet);

	// For now use a region
	XMPBoolean result = kXMPFalse;
	
	Rect rect = fRect;
	::InsetRect(&rect, -2, -2);

	RgnHandle rgn = ::NewRgn();
	::OpenRgn();
	::FrameOval(&rect);
	::CloseRgn(rgn);
	
	if (::PtInRgn(mouse, rgn))
	{
		if (!fShapeStyle.HasFillStyle())
		{
			Rect rect2 = rect;
			::InsetRect(&rect2, 2 + fShapeStyle.GetPenWidth() + 2, 2 + fShapeStyle.GetPenHeight() + 2);			
			::MapRgn(rgn, &rect, &rect2);
			result = !::PtInRgn(mouse, rgn);
		}
		result = kXMPTrue; 
	}
		
	DisposeRgn(rgn);
	
	return result;
}

//=========================================================================
// class CRoundRectShape
//=========================================================================

//-------------------------------------------------------------------------
// CRoundRectShape::CRoundRectShape
//-------------------------------------------------------------------------

CRoundRectShape::CRoundRectShape():
	CRectShape(kRRectShape)
{
}

//-------------------------------------------------------------------------
// CRoundRectShape::~CRoundRectShape
//-------------------------------------------------------------------------

CRoundRectShape::~CRoundRectShape()
{
}

//-------------------------------------------------------------------------
// CRoundRectShape::DrawShape
//-------------------------------------------------------------------------

void CRoundRectShape::DrawShape(CFacet* /*facet*/, XMPShape* invalidShape)
{
	if (fShapeStyle.HasFillStyle())
	{
		fShapeStyle.SetGrafPortFillStyle();
		::PaintRoundRect(&fRect, 32, 32);
	}
	
	if (fShapeStyle.HasFrameStyle())
	{
		fShapeStyle.SetGrafPortFrameStyle();
		::FrameRoundRect(&fRect, 32, 32);
	}

	// ----- Restore the grafport -----
	::PenNormal();
	::RGBForeColor(&kRGBBlack);
	
	// ----- Remove what I have drawn
	if (invalidShape)
		invalidShape->Subtract(GetClipShape());
}

//-------------------------------------------------------------------------
// CRoundRectShape::OutlineShape
//-------------------------------------------------------------------------

void CRoundRectShape::OutlineShape(const Rect& rect)
{
	::FrameRoundRect(&rect, 32, 32);
}

//=========================================================================
// class CProxyShape
//=========================================================================

//-------------------------------------------------------------------------
// CProxyShape::CProxyShape
//-------------------------------------------------------------------------

CProxyShape::CProxyShape(const XMPRect& xmpRect):
	CRectShape(kProxyShape)
{
	fFrozen = kXMPFalse;

	xmpRect.AsQDRect(fRect);
	
	fProxyRun = kXMPNULL;
	
	Point penSize;
	penSize.h = 1;
	penSize.v = 1;
	CShapeStyle proxyStyle(kFrameFill, penSize, 
						  XMPASLMQDGlobals.black, XMPASLMQDGlobals.black, kRGBBlack, kRGBWhite);
	fShapeStyle = proxyStyle;
}

//-------------------------------------------------------------------------
// CProxyShape::CProxyShape
//-------------------------------------------------------------------------

CProxyShape::CProxyShape():
	CRectShape(kProxyShape)
{
	fProxyRun = kXMPNULL;
	fFrozen = kXMPFalse;
}

//-------------------------------------------------------------------------
// CProxyShape::~CProxyShape
//-------------------------------------------------------------------------

CProxyShape::~CProxyShape()
{
}
	
//-------------------------------------------------------------------------
// CProxyShape::Removed
//-------------------------------------------------------------------------

void CProxyShape::Removed()
{
	CBaseShape::Removed();
	
	fProxyRun->GetEmbeddingPart()->RemoveEmbeddedFrames(fProxyRun);
}

//-------------------------------------------------------------------------
// CProxyShape::DrawShape
//
//	Because there can be multiple of CProxyShape pointing on an embedded
//	frame, I only draw those belonging to this frame.
//-------------------------------------------------------------------------

void CProxyShape::DrawShape(CFacet* facet, XMPShape* invalidShape)
{
	Point svOrigin;
	svOrigin.h = XMPASLMQDGlobals.thePort->portRect.left;
	svOrigin.v = XMPASLMQDGlobals.thePort->portRect.top;
	RgnHandle svClip = ::NewRgn();
	::GetClip(svClip);
	
	XMPShape* tempShape = ::NewXMPShape();
	
	CEmbeddedXMPFacetsIterator ite(fProxyRun, facet ,kXMPFrontToBack);
	for (XMPFacet *embeddedXMPFacet = ite.First(); ite.IsNotComplete(); embeddedXMPFacet = ite.Next())
	{	
		tempShape->CopyFrom(invalidShape);
		tempShape->Transform(facet->GetXMPFacet()->GetFrameTransform());	// Frame -> Window
		tempShape->InverseTransform(embeddedXMPFacet->GetFrameTransform()); 	// Window -> Frame

		embeddedXMPFacet->Draw(tempShape);
		embeddedXMPFacet->DrawChildren(tempShape);
		
		tempShape->CopyFrom(embeddedXMPFacet->GetClipShape());
		tempShape->Transform(embeddedXMPFacet->GetExternalTransform());
		invalidShape->Subtract(tempShape);
	}
	
	delete tempShape;

	::SetClip(svClip);
	::DisposeRgn(svClip);
	::SetOrigin(svOrigin.h, svOrigin.v);
}


//-------------------------------------------------------------------------
// CProxyShape::HitTest
//-------------------------------------------------------------------------

XMPBoolean CProxyShape::HitTest(CFacet* facet, Point mouse, XMPEventData event) const
{
XMPUnused(event);

	XMPPoint xmpMouse(ff(mouse.h), ff(mouse.v));
	xmpMouse = facet->GetWindowContentTransform()->TransformPoint(xmpMouse);
	return fProxyRun->WhichEmbeddedFacet(facet, xmpMouse) != kXMPNULL;
}

//-------------------------------------------------------------------------
// CProxyShape::MapShape
//-------------------------------------------------------------------------

void CProxyShape::MapShape(const Rect& srcRect, const Rect& dstRect)
{
	CRectShape::MapShape(srcRect, dstRect);

	XMPShape *shape = ::NewXMPShape(fRect);
	
	fProxyRun->ResizeProxyFrames(shape);		// shape is kept by the frame
}

//-------------------------------------------------------------------------
// CProxyShape::OffsetShape
//-------------------------------------------------------------------------

void CProxyShape::OffsetShape(XMPCoordinate xDelta, XMPCoordinate yDelta)
{	
	CRectShape::OffsetShape(xDelta, yDelta);

	XMPPoint offset(xDelta, yDelta);
	
	fProxyRun->OffsetProxyFrames(offset);
}
		
//-------------------------------------------------------------------------
// CProxyShape::SelectShape
//-------------------------------------------------------------------------

void CProxyShape::SelectShape(XMPBoolean state)
{
	CBaseShape::SelectShape(state);
	fProxyRun->SetSelectState(state);
}

//-------------------------------------------------------------------------
// CProxyShape::Externalize
//-------------------------------------------------------------------------

void CProxyShape::Externalize(XMPStorageUnit* storage)
{	
	CRectShape::Externalize(storage);

	fProxyRun->Externalize(storage);
}

//-------------------------------------------------------------------------
// CProxyShape::Internalize
//-------------------------------------------------------------------------

void CProxyShape::Internalize(CDrawPart* drawPart, XMPStorageUnit* storage)
{	
	CRectShape::Internalize(drawPart, storage);
		
	fProxyRun = (CDrawProxyRun*)drawPart->NewProxyRun();
	fProxyRun->Internalize(storage);
	fProxyRun->SetShape(this);
}

//-------------------------------------------------------------------------
// CProxyShape::CloneTo
//-------------------------------------------------------------------------

void CProxyShape::CloneTo(XMPStorageUnit* storageUnit, CFrame* commandFrame, XMPCloneKind cloneKind)
{
	// I can't call CRectShape::CloneTo because it will call CProxyShape::Externalize
	CRectShape::Externalize(storageUnit);
	fProxyRun->CloneTo(storageUnit, commandFrame, cloneKind);
}

//-------------------------------------------------------------------------
// CProxyShape::CloneFrom
//-------------------------------------------------------------------------

void CProxyShape::CloneFrom(CDrawPart* drawPart, XMPStorageUnit* storageUnit, XMPCloneKind cloneKind)
{
	// I can't call CRectShape::CloneFrom because it will call CProxyShape::Internalize
	CRectShape::Internalize(drawPart, storageUnit);
	
	CDrawProxyRun* proxyRun = (CDrawProxyRun*)drawPart->NewProxyRun();
	
	// ----- Set the proxyRun of the shape and the shape of the proxyRun -----
	SetProxyRun(proxyRun);
	proxyRun->SetShape(this);

	// ----- Clone the proxyRun -----
	XMPFrame* embeddedXMPFrame = proxyRun->CloneFrom(storageUnit, cloneKind);
	
	// ----- Embed the frame -----
	Rect box;
	GetBoundingBox(box);
	Point ptTemp;
	::SetPt(&ptTemp, box.left, box.top);
	XMPTransform* externalXForm = ::NewXMPTransform(ptTemp);

	drawPart->EmbedFrame(embeddedXMPFrame, 
						 proxyRun,
						 drawPart->GetMainPresentation(), 
			  			 externalXForm,
						 kXMPFalse);
}

//-------------------------------------------------------------------------
// CProxyShape::SetShapeStyle
//-------------------------------------------------------------------------

void CProxyShape::SetShapeStyle(const CShapeStyle& shapeStyle)
{
XMPUnused(shapeStyle);

	// Can't change the style of a CProxyShape
}

//-------------------------------------------------------------------------
// CProxyShape::ProxyShapeChanged
//-------------------------------------------------------------------------

void CProxyShape::ProxyShapeChanged(const XMPRect& rect)
{
	rect.AsQDRect(fRect);
}

//-------------------------------------------------------------------------
// CProxyShape::MovedAfter
//
//	shape has been moved after this
//-------------------------------------------------------------------------

void CProxyShape::MovedAfter(CBaseShape* shape)
{
	if (shape->GetShapeType() == kProxyShape)
	{
		((CProxyShape*)shape)->GetProxyRun()->MoveBehind(GetProxyRun());
	}
}

//-------------------------------------------------------------------------
// CProxyShape::MovedBefore
//
//	shape has been moved before this
//-------------------------------------------------------------------------

void CProxyShape::MovedBefore(CBaseShape* shape)
{
	if (shape->GetShapeType() == kProxyShape)
	{
		((CProxyShape*)shape)->GetProxyRun()->MoveBefore(GetProxyRun());
	}
}

//-------------------------------------------------------------------------
// CProxyShape::MovedFirst
//
//	this has been moved First
//-------------------------------------------------------------------------

void CProxyShape::MovedFirst()
{
	GetProxyRun()->MoveFirst();
}

//-------------------------------------------------------------------------
// CProxyShape::MovedLast
//
//	this has been moved Last
//-------------------------------------------------------------------------

void CProxyShape::MovedLast()
{
	GetProxyRun()->MoveLast();
}

//-------------------------------------------------------------------------
// CProxyShape::SetFrozen
//-------------------------------------------------------------------------

XMPBoolean CProxyShape::SetFrozen(XMPBoolean state)
{
	if (fFrozen != state)
	{
		fFrozen = state;	
		return kXMPTrue;
	}
	
	return kXMPFalse;
}

//-------------------------------------------------------------------------
// CProxyShape::IsFrozen
//-------------------------------------------------------------------------

XMPBoolean CProxyShape::IsFrozen() const
{
	return fFrozen;
}

