/*
	File:		Shapes.h

	Contains:	Definition for class CBaseShape

	Written by:	Henri Lamiraux

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.
*/

#ifndef _SHAPES_
#define _SHAPES_

#ifndef _ALTPOINT_			// Just to be sure to be included before PlfmType.h
#include "AltPoint.h"
#endif

#ifndef _STYLE_
#include "Style.h"
#endif

#ifndef _DRAWFRAME_
#include "DrawFrame.h"
#endif

#ifndef _DRAWPROXY_
#include "DrawProxy.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _PLFMDEF_
#include <PlfmDef.h>
#endif

// ----- Mac Includes -----

#ifndef __TYPES__
#include <Types.h>
#endif

//==============================================================================
// class CBaseShape
//==============================================================================

class CEmbeddingFrame;
class XMPFrame;
class XMPShape;
class CDrawPart;
class CDrawPublishLink;
class CDrawSubscribLink;

//==============================================================================
// Constants
//==============================================================================

enum ShapeType {
	kLineShape = 1,
	kRectShape = 2,
	kOvalShape = 3,
	kRRectShape = 4,
	kProxyShape = 5
};

//==============================================================================
// class CBaseShape
//==============================================================================

class CBaseShape
{
public:
	virtual ~CBaseShape();

	virtual void Removed();
	
	// ----- Drawing -----
	virtual void 		DrawShape(CFacet* facet, XMPShape* invalidShape) = 0;
	void				DrawShapeHandles(CFacet* facet, XMPBoolean turnOn);
	void				DrawHandle(XMPSShort whichHandle);
	
	// ----- Selection -----
	virtual XMPBoolean	HitTest(CFacet* facet, Point mouse, XMPEventData event) const = 0;
	XMPBoolean			InSelectionRect(const Rect& rect) const;
	
	XMPSShort			WhichHandle(CFacet* facet, Point mouse) const;
	
	void 				CalcHandleRect(XMPSShort whichHandle, Rect& handleRect) const;

	virtual void		SelectShape(XMPBoolean state);
	XMPBoolean			IsSelected() const
							{return fSelected;}

	// ----- Creation -----
	XMPBoolean 			Track(const CShapeStyle& trackStyle, Point anchorPoint, XMPEventData event);
	virtual void 		TrackFeedback(Point anchorPoint, Point currentPoint, XMPBoolean erase) = 0;

	// ----- Drag -----
	virtual void		OffsetShape(XMPCoordinate xDelta, XMPCoordinate yDelta) = 0;
	
	// ----- Resize -----
	virtual void		ResizeFeedback(const Rect& originalRect, const Rect& mapRect) = 0;
	virtual void		GetHandleCenter(XMPSShort whichHandle, Point& center) const = 0;
	virtual void		MapShape(const Rect& srcRect, const Rect& dstRect) = 0;
	
	virtual void		GetUpdateShape(XMPShape *updateShape) const = 0;
	virtual void		GetDragRgn(RgnHandle dragRgn) const = 0;
	virtual XMPShape*	GetClipShape() = 0;
	virtual void		GetBoundingBox(Rect& bounds) const = 0;
	
	// ----- z-Order -----
	virtual void 		MovedAfter(CBaseShape* shape);
	virtual void 		MovedBefore(CBaseShape* shape);
	virtual void 		MovedFirst();
	virtual void 		MovedLast();
	
	// ----- Persistence -----
	virtual void 		Externalize(XMPStorageUnit* storage);
	virtual void 		Internalize(CDrawPart* drawPart, XMPStorageUnit* storage);
	virtual void		CloneTo(XMPStorageUnit* storageUnit, CFrame* commandFrame, XMPCloneKind cloneKind);
	virtual void		CloneFrom(CDrawPart* drawPart, XMPStorageUnit* storage, XMPCloneKind cloneKind);
	
	// ----- Cache -----
	void				ClearCache();
	
	// ----- Linking -----
	void				SetPublishLink(CDrawPublishLink *publishLink)
							{fPublishLink = publishLink;}
	CDrawPublishLink*	GetPublishLink() const
							{return fPublishLink;}
	XMPBoolean			IsPublished() const
							{return fPublishLink != kXMPNULL;}
							
	void				SetSubscribLink(CDrawSubscribLink *subscribLink)
							{fSubscribLink = subscribLink;}
	CDrawSubscribLink*	GetSubscribLink() const
							{return fSubscribLink;}
	XMPBoolean			IsSubscribed() const
							{return fSubscribLink != kXMPNULL;}
							
	// ----- Getters/Setters -----
	virtual XMPBoolean	SetFrozen(XMPBoolean state);
	virtual XMPBoolean IsFrozen() const;

	void				SetShapeStyle(const CShapeStyle& shapeStyle);
	void				GetShapeStyle(CShapeStyle& shapeStyle) const
							{shapeStyle = fShapeStyle;}
	void				GetPenSize(Point& penSize) const;
	
	XMPSShort			GetNumberOfHandles() const
							{return fNumberOfHandles;}
	ShapeType			GetShapeType() const
							{return (ShapeType)fShapeType;}
protected:
	CBaseShape(XMPSShort numberOfHandles, XMPUShort shapeType);

	void				MapPoint(Point &pt, const Rect& srcRect, const Rect& destRect);
	
protected:
	CShapeStyle			fShapeStyle;
	XMPShape*			fClipShape;
	
private:
	XMPUShort			fShapeType;
	XMPBoolean			fSelected;
	XMPSShort			fNumberOfHandles;
	CDrawPublishLink*	fPublishLink;
	CDrawSubscribLink*	fSubscribLink;
};

//==============================================================================
// class CLineShape
//==============================================================================

class CLineShape : public CBaseShape
{
public:
	CLineShape();
	virtual ~CLineShape();

	// ----- Geometry
	virtual void		GetUpdateShape(XMPShape *updateShape) const;
	virtual void		GetDragRgn(RgnHandle dragRgn) const;
	virtual XMPShape*	GetClipShape();
	virtual void		GetBoundingBox(Rect& bounds) const;
		
	// ----- Creation
	virtual void 		TrackFeedback(Point anchorPoint, Point currentPoint, XMPBoolean erase);
	
	// ----- Selection
	virtual XMPBoolean	HitTest(CFacet* facet, Point mouse, XMPEventData event) const;
	
	// ----- Dragging
	virtual void		OffsetShape(XMPCoordinate xDelta, XMPCoordinate yDelta);

	// ----- Resize
	virtual void		ResizeFeedback(const Rect& originalRect, const Rect& mapRect);
	virtual void		GetHandleCenter(XMPSShort whichHandle, Point& center) const;
	virtual void		MapShape(const Rect& srcRect, const Rect& dstRect);

	// ----- Drawing
	virtual void 		DrawShape(CFacet* facet, XMPShape* invalidShape);

	// ----- Persistence
	virtual void 		Externalize(XMPStorageUnit* storage);
	virtual void 		Internalize(CDrawPart* drawPart, XMPStorageUnit* storage);

private:
	void 				OutlineShape(Point pt1, Point pt2);
	
private:
	Point				fAnchorPoint;
	Point				fEndPoint;
};

//==============================================================================
// class CRectShape
//==============================================================================

class CRectShape : public CBaseShape
{
public:
	CRectShape();
	virtual ~CRectShape();

	// ----- Geometry
	virtual void		GetUpdateShape(XMPShape *updateShape) const;
	virtual void		GetDragRgn(RgnHandle dragRgn) const;
	virtual XMPShape*	GetClipShape();
	virtual void		GetBoundingBox(Rect& bounds) const;

	// ----- Creation
	virtual void 		TrackFeedback(Point anchorPoint, Point currentPoint, XMPBoolean erase);
	
	// ----- Selection
	virtual XMPBoolean	HitTest(CFacet* facet, Point mouse, XMPEventData event) const;
	
	// ----- Dragging
	virtual void		OffsetShape(XMPCoordinate xDelta, XMPCoordinate yDelta);

	// ----- Resize
	virtual void		ResizeFeedback(const Rect& originalRect, const Rect& mapRect);
	virtual void		GetHandleCenter(XMPSShort whichHandle, Point& center) const;
	virtual void		MapShape(const Rect& srcRect, const Rect& dstRect);

	// ----- Drawing
	virtual void 		DrawShape(CFacet* facet, XMPShape* invalidShape);

	// ----- Persistence
	virtual void 		Externalize(XMPStorageUnit* storage);
	virtual void 		Internalize(CDrawPart* drawPart, XMPStorageUnit* storage);

protected:
	CRectShape(XMPUShort shapeType);

private:
	virtual void 		OutlineShape(const Rect& rect);
	void				CalcRect(Point anchorPoint, Point endPoint, Rect& rect);

protected:
	Rect				fRect;

private:
	Point				fAnchorPoint;
};

//==============================================================================
// class COvalShape
//==============================================================================

class COvalShape : public CRectShape
{
public:
	COvalShape();
	virtual ~COvalShape();
	
	virtual void 		DrawShape(CFacet* facet, XMPShape* invalidShape);
	
	// ----- Selection
	virtual XMPBoolean	HitTest(CFacet* facet, Point mouse, XMPEventData event) const;
	
private:
	virtual void		OutlineShape(const Rect& rect);
	
};

//==============================================================================
// class CRoundRectShape
//==============================================================================

class CRoundRectShape : public CRectShape
{
public:
	CRoundRectShape();
	virtual ~CRoundRectShape();
	
	virtual void 		DrawShape(CFacet* facet, XMPShape* invalidShape);

private:
	virtual void		OutlineShape(const Rect& rect);
	
};

//==============================================================================
// class CProxyShape
//==============================================================================

class CProxyShape : public CRectShape
{
public:
	CProxyShape(const XMPRect& xmpRect);
	CProxyShape();
	virtual ~CProxyShape();
	virtual void Removed();
		
	virtual void 		DrawShape(CFacet* facet, XMPShape* invalidShape);

	virtual XMPBoolean	HitTest(CFacet* facet, Point mouse, XMPEventData event) const;
	
	virtual void		OffsetShape(XMPCoordinate xDelta, XMPCoordinate yDelta);
	virtual void		MapShape(const Rect& srcRect, const Rect& dstRect);
	
	virtual void 		SelectShape(XMPBoolean state);

	void				ProxyShapeChanged(const XMPRect& rect);
	
	CDrawProxyRun*		GetProxyRun() const
							{return fProxyRun;}
	void				SetProxyRun(CDrawProxyRun* proxyRun)
							{fProxyRun = proxyRun;}
							
	virtual void		SetShapeStyle(const CShapeStyle& shapeStyle);
	
	// ----- Persistence
	virtual void 		Externalize(XMPStorageUnit* storage);
	virtual void 		Internalize(CDrawPart* drawPart, XMPStorageUnit* storage);
	virtual void		CloneTo(XMPStorageUnit* storageUnit, 
							      CFrame* commandFrame, 
								  XMPCloneKind cloneKind);
	virtual void		CloneFrom(CDrawPart* drawPart, 
								  XMPStorageUnit* storage, 
								  XMPCloneKind cloneKind);

	// ----- z-Order -----
	virtual void 		MovedAfter(CBaseShape* shape);
	virtual void 		MovedBefore(CBaseShape* shape);
	virtual void 		MovedFirst();
	virtual void 		MovedLast();
	
	// ----- Frozen -----
	virtual XMPBoolean	SetFrozen(XMPBoolean state);
	virtual XMPBoolean IsFrozen() const;

private:
	CDrawProxyRun		*fProxyRun;
	XMPBoolean			fFrozen;

};
#endif