/*
	File:		RulerFacet.cp

	Contains:	implementation of class CRulerFacet

	Written by:	Mike Burbidge

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

*/

// ----- Macintish Includes -----

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

// ----- OpenDoc Includes -----

#ifndef _SHAPE_
#include "Shape.h"
#endif

// ----- Framework Includes -----

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

// ----- TextPart Includes -----

#ifndef _RulerObject_
#include "RulerObject.h"
#endif

#ifndef _RULERFACET_
#include "RulerFacet.h"
#endif

#ifndef _RULERFRAME_
#include "RulerFrame.h"
#endif

#ifndef _TEXTUTIL_
#include "TextUtil.h"
#endif

#ifndef _TEXTPART_
#include "TextPart.h"
#endif

#pragma segment TextPartSegment

//========================================================================================
// CLASS CRulerFacet
//========================================================================================

const RGBColor CRulerFacet::kGray =		{ 0xB000, 0xB000, 0xB000 };
const RGBColor CRulerFacet::kBlack =	{ 0x0000, 0x0000, 0x0000 };

//----------------------------------------------------------------------------------------
// CRulerFacet::CRulerFacet
//----------------------------------------------------------------------------------------

CRulerFacet::CRulerFacet() :
	CFacet(),
	fTextPart(kXMPNULL),
	fSelectedTool(0)
{
}

//----------------------------------------------------------------------------------------
// CRulerFacet::IRulerFacet
//----------------------------------------------------------------------------------------

void CRulerFacet::IRulerFacet(XMPFacet* facet, CTextPart* textPart)
{
	InitFacet(facet);
	fTextPart = textPart;
		
	// ----- Calculate the tool positions
	Rect toolArea;
	short center = CTextPart::kRulerWidth / 2;
	
	toolArea.top = CTextPart::kRulerRulerHeight + 4;
	toolArea.left = center - (CTextPart::kToolWidths + 4) * 2;
	toolArea.bottom = toolArea.top + CTextPart::kToolHeights;
	toolArea.right = toolArea.left + CTextPart::kToolWidths;
		
	// ----- Set the tool area rectangles
	for (short i = 0; i < CTextPart::kNumberOfTools; i++)
	{
		fToolAreas[i] = toolArea;
		toolArea.left += CTextPart::kToolWidths + 4;
		toolArea.right += CTextPart::kToolWidths + 4;
	}
}

//----------------------------------------------------------------------------------------
// CRulerFacet::~CRulerFacet
//----------------------------------------------------------------------------------------

CRulerFacet::~CRulerFacet()
{
}

//----------------------------------------------------------------------------------------
// CRulerFacet::Draw
//----------------------------------------------------------------------------------------

void CRulerFacet::Draw(XMPShape* invalidShape)
{
	XMPRgnHandle region;
	if (invalidShape)
		region = invalidShape->GetQDRegion();
	else
		region = GetClipShape()->GetQDRegion();

	::EraseRgn(region);
	
	CRulerFrame* frame = (CRulerFrame *) GetFrame();

	XMPRect frameBounds;
	frame->GetFrameShapeBounds(&frameBounds);

	Rect qdFrameBounds;
	frameBounds.AsQDRect(qdFrameBounds);

	DrawBackground(region, qdFrameBounds);
	DrawTools(region, qdFrameBounds);
	DrawRuler(region, qdFrameBounds);
}

//----------------------------------------------------------------------------------------
// CRulerFacet::DoMouseDown
//----------------------------------------------------------------------------------------

XMPBoolean CRulerFacet::DoMouseDown(const XMPPoint& where, XMPEventData event)
{
XMPUnused(event);

	CDrawInitiator di(this);

	const CTextPart::Justification justifyTable[] = { kLeftJust, kCenterJust,
													  kRightJust, kFullJust };
	
	for (short i = 0; i < CTextPart::kNumberOfTools; i++)
	{
		XMPRect rect(fToolAreas[i]);
		if (rect.Contains(where))
		{
			{
				CTextDrawInitiator di((CTextFacet *) fTextPart->GetActiveFrame()->GetActiveFacet());
				fTextPart->SetSelectionJustification(justifyTable[i]);
			}
			
			if (i != fSelectedTool)
			{
				Rect invertRect = fToolAreas[fSelectedTool];
				::InsetRect(&invertRect, 1, 1);
				::InvertRect(&invertRect);
				
				invertRect = fToolAreas[i];
				::InsetRect(&invertRect, 1, 1);
				::InvertRect(&invertRect);
			}
			fSelectedTool = i;
			break;
		}
	}
	return true;
}

//----------------------------------------------------------------------------------------
// CRulerFacet::MouseEnter
//----------------------------------------------------------------------------------------

void CRulerFacet::MouseEnter(const XMPPoint& where)
{
XMPUnused(where);
	SetCursor(&XMPASLMQDGlobals.arrow); 
}

//----------------------------------------------------------------------------------------
// CRulerFacet::DrawBackground
//----------------------------------------------------------------------------------------

void CRulerFacet::DrawBackground(RgnHandle invalidRgn, const Rect& boundingBox)
{
XMPUnused(invalidRgn);

	Rect grayArea = boundingBox;
	grayArea.top += CTextPart::kRulerRulerHeight;
	
	::RGBForeColor(&kGray);
	::FillRect(&grayArea, &XMPASLMQDGlobals.black);
	::RGBForeColor(&kBlack);
	::PenNormal();

	::MoveTo(boundingBox.left, boundingBox.bottom - 1);
	::LineTo(boundingBox.right, boundingBox.bottom - 1);
}

//----------------------------------------------------------------------------------------
// CRulerFacet::DrawTools
//----------------------------------------------------------------------------------------

void CRulerFacet::DrawTools(RgnHandle invalidRgn, const Rect& boundingBox)
{
XMPUnused(invalidRgn);
XMPUnused(boundingBox);

	FW_CAcquireASLMResourceAccess aq;
	
	const CTextPart::Justification justifyTable[] = { kLeftJust, kCenterJust, kRightJust, kFullJust };
	const CTextPart::Justification justify = fTextPart->GetSelectionJustification();
	
	// ----- Draw the tools
	for (short resourceId = 2021; resourceId <= 2024; resourceId++)
	{
		PicHandle pictHandle = ::GetPicture(resourceId);
		::DrawPicture(pictHandle, &fToolAreas[resourceId - 2021]);
		if (justifyTable[resourceId - 2021] == justify)
		{
			Rect invertRect = fToolAreas[resourceId - 2021];
			::InsetRect(&invertRect, 1, 1);
			::InvertRect(&invertRect);
			fSelectedTool = resourceId - 2021;
		}
	}
}

//----------------------------------------------------------------------------------------
// CRulerFacet::DrawRuler
//----------------------------------------------------------------------------------------

void CRulerFacet::DrawRuler(RgnHandle invalidRgn, const Rect& boundingBox)
{
XMPUnused(invalidRgn);
XMPUnused(boundingBox);

	FW_CAcquireASLMResourceAccess aq;
	
	Rect rulerArea = { 0, 0, CTextPart::kRulerHeight, CTextPart::kRulerWidth };
	PicHandle rulerHandle = ::GetPicture(2001);
	::DrawPicture(rulerHandle, &rulerArea);
}
