/*
	File:		TextFrame.cp

	Contains:	implementation of class CTextFrame

	Written by:	Mike Burbidge

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <3>	 4/11/94	DS		Change CreateEmbeddedFrame call to
									RequestEmbeddedFrame
		 <2>	 3/31/94	DS		Fix include of XMPSession
		<14>	 1/15/94	RR		WinState.h->WinStat.h,
									Dispatch.h->Disptch.h

	To Do:
*/

// ----- Macintosh Includes -----

#ifndef __FIXMATH__
#include <FixMath.h>
#endif

#ifndef mathRoutinesIncludes
#include <Math Routines.h>
#endif

// ----- OpenDoc Includes -----

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _FWUTIL_
#include <FWUtil.h>
#endif

#ifndef _MENUBAR_
#include <MenuBar.h>
#endif

#ifndef _XMPSESSN_
#include <XMPSessM.h>
#endif

#ifndef _DISPTCH_
#include <Disptch.h>
#endif

#ifndef _INFO_
#include <Info.h>
#endif

// ----- Textension Includes -----

#ifndef _Textension_
#include "Textension.h"
#endif

#ifndef _TextensionCommand_
#include "TextensionCommand.h"
#endif

// ----- TextPart Includes -----

#ifndef _TEXTUTIL_
#include "TextUtil.h"
#endif

#ifndef _TEXTFACET_
#include "TextFacet.h"
#endif

#ifndef _TEXTFRAME_
#include "TextFrame.h"
#endif

#ifndef _TEXTPART_
#include "TextPart.h"
#endif

#ifndef _STYLEINFO_
#include "StyleInfo.h"
#endif

#ifndef _ODTEXT_
#include "ODText.h"
#endif

#pragma segment TextPartSegment


//========================================================================================
// CLASS CTextFrame
//========================================================================================

//----------------------------------------------------------------------------------------
// CTextFrame::CTextFrame
//----------------------------------------------------------------------------------------

CTextFrame::CTextFrame() :
	CEmbeddingFrame(),
	fTextPart(NULL),
	fRulerFrame(NULL)
{
}

//----------------------------------------------------------------------------------------
// CTextFrame::ITextFrame
//----------------------------------------------------------------------------------------

void CTextFrame::ITextFrame(XMPFrame* xmpFrame, CTextPart* textPart)
{
	IEmbeddingFrame(xmpFrame, textPart);

	// ----- By default we put all of them
	AddToFocusSet(textPart->GetKeyFocusToken());
	AddToFocusSet(textPart->GetMenuFocusToken());
	AddToFocusSet(textPart->GetSelectionFocusToken());

	// ----- Register for idle time
	textPart->GetSession()->GetDispatcher()->RegisterIdle(textPart, GetXMPFrame(), 15);

	this->SetDroppable(kXMPTrue);

	fTextPart = textPart;
}

//----------------------------------------------------------------------------------------
// CTextFrame::~CTextFrame
//----------------------------------------------------------------------------------------

CTextFrame::~CTextFrame()
{
	fTextPart->GetSession()->GetDispatcher()->UnregisterIdle(fTextPart, GetXMPFrame());
}

//----------------------------------------------------------------------------------------
// CTextFrame::HandleKeyDown
//----------------------------------------------------------------------------------------

XMPBoolean CTextFrame::HandleKeyDown(XMPEventData event)
{
	static CTextensionKeyCommand* lastCommand = NULL;
	
	CTextDrawInitiator di((CTextFacet *) GetActiveFacet());
	
	CTextension* text = fTextPart->GetEditText();
	
	fTextPart->SetEmbeddedRunFrame(this);
	fTextPart->SetEmbeddedRunFacet((CTextFacet *) GetActiveFacet());
	
	text->KeyDown(event);
	
	this->InvalidateAllFacets((CTextFacet *) GetActiveFacet(), kXMPNULL);
	
	GrowFrame();
	
	return true;
}

//----------------------------------------------------------------------------------------
// CTextFrame::DoIdle
//----------------------------------------------------------------------------------------

XMPBoolean CTextFrame::DoIdle()
{
	CTextFacet* facet = (CTextFacet *) GetActiveFacet();
	
	if (facet)
	{
		CTextDrawInitiator di(facet);
		
		fTextPart->SetEmbeddedRunFrame(this);
		fTextPart->SetEmbeddedRunFacet((CTextFacet *) GetActiveFacet());
	
		(fTextPart->GetEditText())->Idle();
	
		return true;
	}
	else
		return false;
}

//----------------------------------------------------------------------------------------
// CTextFrame::FrameShapeChanged
//----------------------------------------------------------------------------------------

void CTextFrame::FrameShapeChanged()
{
	COpenDocText* textEdit = fTextPart->GetEditText();
	XMPFrame* xmpFrame = GetXMPFrame();
	
	xmpFrame->Invalidate(xmpFrame->GetUsedShape());

	CEmbeddingFrame::FrameShapeChanged();
	
	XMPRect frameRect;
	GetFrameShapeBounds(&frameRect);

	// ----- Set the new view rect size
	
	Rect qdFrameRect;
	frameRect.AsQDRect(qdFrameRect);
	textEdit->SetViewRect(&qdFrameRect);
			
	// ----- Set the frame size. This is similar to the dest rect in TE.
	
	frameRect.Inset(kBorderInset, kBorderInset);	
	
	LongPoint frameSize(FixedToInt(frameRect.Width()), 0/*0 means unlimited hite*/);

	CFrames* frames = textEdit->GetFramesHandler();
	
	CDisplayChanges displayChanges;
	frames->SetTextFrameSize(&frameSize, &displayChanges);
	
	// ------ Reflow the text if necessary.
	
	short action = textEdit->DisplayChanged(&displayChanges);
	
	// ------ Notify embedded parts of position changes.
	
	textEdit->NotifyEmbeddedPartsOfPositionChange(0, textEdit->CountLines() - 1);
}

//----------------------------------------------------------------------------------------
// CTextFrame::FocusStateChanged
//----------------------------------------------------------------------------------------

void CTextFrame::FocusStateChanged(XMPTypeToken focus, XMPBoolean newState)
{
	CEmbeddingFrame::FocusStateChanged(focus, newState);
	
	CTextDrawInitiator di((CTextFacet *) GetActiveFacet());

	if (focus == GetPart()->GetKeyFocusToken())
	{
		if (newState)
		{
			fTextPart->SetEmbeddedRunFrame(this);
			fTextPart->SetEmbeddedRunFacet((CTextFacet *) GetActiveFacet());
			
			(fTextPart->GetEditText())->Activate(true, true);
			ShowRuler();
		}
		else
		{
			fTextPart->SetEmbeddedRunFrame(this);
			fTextPart->SetEmbeddedRunFacet((CTextFacet *) GetActiveFacet());
			
			(fTextPart->GetEditText())->Activate(false, false);
			HideRuler();
		}
	}
}

//----------------------------------------------------------------------------------------
// CTextFrame::NewFacet
//----------------------------------------------------------------------------------------

CFacet* CTextFrame::NewFacet(XMPFacet* xmpFacet)
{
	CTextFacet* facet = new CTextFacet;
	facet->InitTextFacet(xmpFacet, fTextPart);
	return facet;
}

//----------------------------------------------------------------------------------------
// CTextFrame::DoMenuEvent
//----------------------------------------------------------------------------------------

XMPBoolean CTextFrame::DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID command)
{
	XMPMenuID menuId;
	XMPMenuItemID itemId;
	
	CTextDrawInitiator di((CTextFacet *) GetActiveFacet());
	
	menuBar->GetMenuAndItem(command, menuId, itemId);
	MenuHandle menu = menuBar->GetMenu(menuId);

	switch (menuId)
	{
		case CTextPart::kSizeMenu:
			fTextPart->SetEmbeddedRunFrame(this);
			fTextPart->SetEmbeddedRunFacet((CTextFacet *) GetActiveFacet());
			fTextPart->SetFontSize(gFontSizeTable[itemId - 1].fFontSize);
			break;
		
		case CTextPart::kFontMenu:
			{
				Str255 fontName;
				
				::GetItem(menu, itemId, fontName);
				fTextPart->SetEmbeddedRunFrame(this);
				fTextPart->SetEmbeddedRunFacet((CTextFacet *) GetActiveFacet());
				fTextPart->SetFont(fontName);
			}
			break;
		
		default:
			return CEmbeddingFrame::DoMenuEvent(menuBar, command);
	}
		
	return true;
}

//----------------------------------------------------------------------------------------
// CTextFrame::FacetAdded
//----------------------------------------------------------------------------------------

void CTextFrame::FacetAdded(CFacet* facet)
{
	CEmbeddingFrame::FacetAdded(facet);
}

//----------------------------------------------------------------------------------------
// CTextFrame::FacetRemoved
//----------------------------------------------------------------------------------------

void CTextFrame::FacetRemoved(CFacet* facet)
{
	CEmbeddingFrame::FacetRemoved(facet);
}

//----------------------------------------------------------------------------------------
// CTextFrame::ShowRuler
//----------------------------------------------------------------------------------------

void CTextFrame::ShowRuler()
{
	if (this->GetXMPFrame()->IsRoot())
		return;
		
	XMPFrame* rootFrame = GetActiveFacet()->GetXMPWindow()->GetRootFrame();
	XMPFacet* rootFacet = GetActiveFacet()->GetXMPWindow()->GetRootFacet();
	XMPPart* rootPart = rootFrame->GetPart();

	Point pt = { 0, 0 };
	XMPTransform* externalTransform = ::NewXMPTransform(pt);
	
	XMPRect rect;
	rootFrame->GetFrameShape()->GetBoundingBox(&rect);
	rect.right = IntToFixed(FixedToInt(rect.right) - 15);
	rect.bottom = IntToFixed(FixedToInt(rect.top)
					+ CTextPart::kRulerRulerHeight
					+ CTextPart::kRulerToolsHeight);
	
	XMPShape* newFrameShape = ::NewXMPShape(&rect);

	fRulerFrame = rootPart->RequestEmbeddedFrame(rootFrame,
												this->GetXMPFrame(),
												newFrameShape,
												fTextPart,
												fTextPart->GetSession()->Tokenize(kXMPViewAsFrame),
												fTextPart->GetRulerPresentation(),
												true);
	
	XMPShape* facetClip = ::NewXMPShape(fRulerFrame->GetFrameShape());
	XMPTransform* facetTransform = ::NewXMPTransform(pt);
	XMPFacet* rulerFacet
		= rootFacet->CreateEmbeddedFacet(fRulerFrame,
										 facetClip,
										 facetTransform,
										 kXMPNULL, kXMPFrameInFront);
	fRulerFrame->Invalidate(kXMPNULL);
}

//----------------------------------------------------------------------------------------
// CTextFrame::HideRuler
//----------------------------------------------------------------------------------------

void CTextFrame::HideRuler()
{
	if (this->GetXMPFrame()->IsRoot())
		return;
		
	fRulerFrame->Invalidate(kXMPNULL);

	XMPFrame* rootFrame = GetActiveFacet()->GetXMPWindow()->GetRootFrame();
	XMPFacet* rootFacet = GetActiveFacet()->GetXMPWindow()->GetRootFacet();
	XMPPart* rootPart = rootFrame->GetPart();
	
	XMPFacet* facet;
	OrderedCollection facets;
	
	XMPFrameFacetIterator it1(fRulerFrame);
	for (facet = it1.First(); it1.IsNotComplete(); facet = it1.Next())
		facets.AddLast(facet);
	
	OrderedCollectionIterator it2(&facets);
	for (facet = (XMPFacet *) it2.First(); it2.IsNotComplete(); facet = (XMPFacet *) it2.Next())
	{
		rootFacet->RemoveFacet(facet);
		delete facet;
	}

//	fTextPart->RemoveDisplayFrame(fRulerFrame);
	
	rootPart->RemoveEmbeddedFrame(fRulerFrame);
	
	fRulerFrame = NULL;
}

//----------------------------------------------------------------------------------------
// CTextFrame::GrowFrame
//----------------------------------------------------------------------------------------

void CTextFrame::GrowFrame()
{
	CTextension* textRecord = fTextPart->GetEditText();
	long textHeight  = textRecord->GetTotalLinesHite();
	
	XMPRect frameRect;
	GetFrameShapeBounds(&frameRect);
	
	if (textHeight > FixedToInt(frameRect.Height()) - 2 * kBorderInset)
	{
		// Calculate the new frame size and request the change.
		
		XMPShape* shape = new XMPShape();
		
		frameRect.bottom 
			= ff(FixedToInt(frameRect.top) + kBorderInset + textHeight + kBorderInset);
		shape->SetRectangle(&frameRect);
		GetXMPFrame()->RequestFrameShape(shape);
		UpdateUsedAndActiveShapes();
		
		// Invalidate the entire frame. Should be smarter about this.
		
		Invalidate(shape);
		
		// ----- Set the new view rect size
		
		Rect qdFrameRect;
		frameRect.AsQDRect(qdFrameRect);
		textRecord->SetViewRect(&qdFrameRect);
				
		// ----- Set the frame size. This is similar to the dest rect in TE.
		
		frameRect.Inset(kBorderInset, kBorderInset);	
		
		LongPoint frameSize(FixedToInt(frameRect.Width()), 0/*0 means unlimited hite*/);
	
		CFrames* frames = textRecord->GetFramesHandler();
		
		CDisplayChanges displayChanges;
		frames->SetTextFrameSize(&frameSize, &displayChanges);
	}
}

