/*
	File:		AltPoint.cpp

	Contains:	Alternate XMPPoint, XMPRect (C++ savvy)

	Written by:	Jens Alfke

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <1>	 2/16/94	JA		first checked in
		----------------------------Moved to TestParts, renamed AltPoint.
		 <3>	  2/7/94	JA		Only compile Mac-specific methods on Mac
									platform.
		 <2>	  2/3/94	JA		Tiger Team Makeover!
		 <1>	11/29/93	JA		first checked in
		----------------------------Moved from Utilities to Imaging project
		 <1>	 10/6/93	JA		first checked in
	To Do:
*/

#ifndef __ALTPOINT__
#include "AltPoint.h"
#endif

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _EXCEPT_
#include "Except.h"
#endif


#if PLATFORM_MACINTOSH
	#ifndef mathRoutinesIncludes
	#include <math routines.h>
	#endif
#endif


#pragma segment XMPShape


//==============================================================================
// XMPPoint
//==============================================================================


#if PLATFORM_MACINTOSH
XMPPoint::XMPPoint( Point qdpt )
{
	x = IntToFixed(qdpt.h);
	y = IntToFixed(qdpt.v);
}
#endif


XMPPoint::XMPPoint( const XMPPoint &pt )
{
	x = pt.x;
	y = pt.y;
}


XMPPoint& XMPPoint::operator= ( const XMPPoint &pt )
{
	x = pt.x;
	y = pt.y;
	return *this;
}


#if PLATFORM_MACINTOSH
XMPPoint& XMPPoint::operator= ( const Point &pt )
{
	x = IntToFixed(pt.h);
	y = IntToFixed(pt.v);
	return *this;
}
#endif


void XMPPoint::Offset ( XMPCoordinate xx, XMPCoordinate yy )
{
	x += xx;
	y += yy;
}


void XMPPoint::operator+= ( const XMPPoint &pt )
{
	x += pt.x;
	y += pt.y;
}


void XMPPoint::operator-= ( const XMPPoint &pt )
{
	x -= pt.x;
	y -= pt.y;
}


#if PLATFORM_MACINTOSH
void XMPPoint::operator+= ( const Point &pt )
{
	x += IntToFixed(pt.h);
	y += IntToFixed(pt.v);
}


void XMPPoint::operator-= ( const Point &pt )
{
	x -= IntToFixed(pt.h);
	y -= IntToFixed(pt.v);
}

Point
XMPPoint::AsQDPoint( ) const
{
	Point pt;
	pt.h = FixedToInt(x);
	pt.v = FixedToInt(y);
	return pt;
}
#endif


XMPSShort
XMPPoint::IntX( ) const
{
	return FixedToInt(x);
}


XMPSShort
XMPPoint::IntY( ) const
{
	return FixedToInt(y);
}


Boolean
XMPPoint::operator==( const XMPPoint &pt ) const
{
	return x==pt.x && y==pt.y;
}


Boolean
XMPPoint::operator!=( const XMPPoint &pt ) const
{
	return x!=pt.x || y!=pt.y;
}


//==============================================================================
// XMPRect
//==============================================================================


#pragma segment XMPShape


#define ASSERTVALID()		WASSERT(right>=left && bottom>=top)


inline XMPCoordinate Min( XMPCoordinate a, XMPCoordinate b )	{return a<b ?a :b;}
inline XMPCoordinate Max( XMPCoordinate a, XMPCoordinate b )	{return a>b ?a :b;}


//------------------------------------------------------------------------------
// ::XMPRect( corner1, corner2 )
//
// Construct a rectangle given two opposite corners (not necessarily topLeft,botRight)
//------------------------------------------------------------------------------

XMPRect::XMPRect( const XMPPoint &a, const XMPPoint &b )
{
	if( a.x<b.x ) {
		left = a.x;
		right= b.x;
	} else {
		left = b.x;
		right= a.x;
	}
	if( a.y<b.y ) {
		top   = a.y;
		bottom= b.y;
	} else {
		top   = b.y;
		bottom= a.y;
	}
}


//------------------------------------------------------------------------------
// ::XMPRect( topLeft, width, height )
//
// Construct a rectangle given its origin, width and height.
//------------------------------------------------------------------------------

XMPRect::XMPRect( const XMPPoint &topLeft, XMPCoordinate width, XMPCoordinate height )
{
	WASSERT(width>=0);
	WASSERT(height>=0);
	
	left = topLeft.x;		right = topLeft.x + width;
	top  = topLeft.y;		bottom= topLeft.y + height;
}


#if PLATFORM_MACINTOSH
//------------------------------------------------------------------------------
// ::XMPRect( Rect )
//
// Construct an XMPRect from a QuickDraw Rect
//------------------------------------------------------------------------------

XMPRect::XMPRect( const Rect &r )
{
	left = ff(r.left);		right = ff(r.right);
	top  = ff(r.top);		bottom= ff(r.bottom);
}


//------------------------------------------------------------------------------
// ::= Rect
//
// Set an XMPRect from a QuickDraw Rect
//------------------------------------------------------------------------------

XMPRect&
XMPRect:: operator= ( const Rect &r )
{
	left = ff(r.left);		right = ff(r.right);
	top  = ff(r.top);		bottom= ff(r.bottom);
	ASSERTVALID();
	return *this;
}
#endif


//------------------------------------------------------------------------------
// ::Set
//
// Set all four coordinates of a rectangle. Warn if the resulting rect is bogus.
//------------------------------------------------------------------------------

void
XMPRect::Set( XMPCoordinate l, XMPCoordinate t, XMPCoordinate r, XMPCoordinate b )
{
	left = l;				right = r;
	top  = t;				bottom= b;
	ASSERTVALID();
}


//------------------------------------------------------------------------------
// ::Set
//
// Set all four coordinates of a rectangle. Warn if the resulting rect is bogus.
//------------------------------------------------------------------------------

void
XMPRect::Set( const XMPPoint &origin, XMPCoordinate width, XMPCoordinate height )
{
	WASSERT(width>=0);
	WASSERT(height>=0);
	left = origin.x;				right = left+width;
	top  = origin.y;				bottom= top +height;
}


//------------------------------------------------------------------------------
// ::SetInt
//
// Set coordinates of a rectangle from integers. Warn if the resulting rect is bogus.
//------------------------------------------------------------------------------

void
XMPRect::SetInt( short l, short t, short r, short b )
{
	left = ff(l);			right = ff(r);
	top  = ff(t);			bottom= ff(b);
	ASSERTVALID();
}


//------------------------------------------------------------------------------
// ::Offset
//
// Move a rectangle.
//------------------------------------------------------------------------------

void
XMPRect::Offset( XMPCoordinate x, XMPCoordinate y )
{
	left += x;				right += x;
	top  += y;				bottom+= y;
}

void
XMPRect::Offset( const XMPPoint &pt )
{
	left += pt.x;			right += pt.x;
	top  += pt.y;			bottom+= pt.y;
}


//------------------------------------------------------------------------------
// ::Inset
//
// Inset the edges of a rectangle, without collapsing to negative dimensions.
//------------------------------------------------------------------------------

void
XMPRect::Inset( XMPCoordinate x, XMPCoordinate y )
{
	left += x;
	right = Max(left,right-x);
	top += y;
	bottom = Max(top,bottom-y);
}


//------------------------------------------------------------------------------
// ::Clear
//
// Zero all coordinates
//------------------------------------------------------------------------------

void
XMPRect::Clear( )
{
	left = right = top = bottom = 0;
}


//------------------------------------------------------------------------------
// :: &=
//
// Intersect me with another rectangle (result stored in me)
//------------------------------------------------------------------------------

void
XMPRect:: operator&= ( const XMPRect &r )
{
	left = Max(left,r.left);	right = Min(right, r.right);
	top  = Max(top, r.top);		bottom= Min(bottom,r.bottom);
	if( this->IsEmpty() )
		this->Clear();
}


//------------------------------------------------------------------------------
// :: |=
//
// Union me with another rectangle (result stored in me)
//------------------------------------------------------------------------------

void
XMPRect:: operator|= ( const XMPRect &r )
{
	if( this->IsEmpty() )
		*this = r;
	else if( !r.IsEmpty() ) {
		left = Min(left,r.left);
		right= Max(right,r.right);
		top  = Min(top,r.top);
		bottom=Max(bottom,r.bottom);
	}
}


//------------------------------------------------------------------------------
// :: |= XMPPoint
//
// Union me with a point (expand to fit point)
//------------------------------------------------------------------------------

void
XMPRect:: operator|= ( const XMPPoint &pt )
{
	left = Min(left,pt.x);
	right= Max(right,pt.x);
	top  = Min(top,pt.y);
	bottom=Max(bottom,pt.y);
}


#if PLATFORM_MACINTOSH
//------------------------------------------------------------------------------
// :: AsQDRect
//
// Turn me into a QuickDraw rect
//------------------------------------------------------------------------------

void
XMPRect::AsQDRect( Rect &r ) const
{
	SetRect(&r, FixedToInt(left), FixedToInt(top), FixedToInt(right), FixedToInt(bottom));
}
#endif


//------------------------------------------------------------------------------
// :: IsEmpty
//
// Do I have no area?
//------------------------------------------------------------------------------

Boolean
XMPRect::IsEmpty( ) const
{
	return right<=left || bottom<=top;
}


//------------------------------------------------------------------------------
// :: Contains( Point )
//
// Do I contain a point? (Remember, I don't contain my right and bottom edges.)
//------------------------------------------------------------------------------

Boolean
XMPRect::Contains( const XMPPoint &pt ) const
{
	return left<=pt.x && pt.x<right
	    && top <=pt.y && pt.y<bottom;
}


#if PLATFORM_MACINTOSH
//------------------------------------------------------------------------------
// :: Contains( Rect )
//
// Do I contain an entire rect?
//------------------------------------------------------------------------------

Boolean
XMPRect::Contains( const XMPRect &r ) const
{
	return left<=r.left && r.right<=right
	    && top <=r.top  && r.bottom<=bottom;
}
#endif


//------------------------------------------------------------------------------
// :: ==
//
// Am I equal to another rect?
//------------------------------------------------------------------------------

Boolean
XMPRect::operator==( const XMPRect &r ) const
{
	return memcmp(this,&r,sizeof(XMPRect)) == 0;
}


//------------------------------------------------------------------------------
// :: Intersects
//
// Do I intersect another rectangle?
// (Remember, I don't contain my right and bottom edges.)
//------------------------------------------------------------------------------

Boolean
XMPRect::Intersects( const XMPRect &r ) const
{
	return Max(left,r.left) < Min(right,r.right)
		&& Max(top,r.top) < Min(bottom,r.bottom);
}
