/*
	File:		AltPoly.cpp

	Contains:	OpenDoc polygon data structure and operations

	Written by:	Jens Alfke

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <2>	 4/13/94	T		jpa: Added PolyEdgeIterator from
									now-obsolete PolyIter.cpp. (1145831)
		 <1>	 2/17/94	JA		first checked in
		 <2>	 2/16/94	JA		Include new AltPoint.h.
		 <7>	  2/7/94	JA		Capitalized THROW.
		 <6>	  2/7/94	JA		Only compile Mac-specific methods on Mac
									platform.
		 <5>	  2/3/94	JA		Tiger Team Makeover!
		 <4>	 1/31/94	JA		Added Contains and AsQDRegion. Many fixes
									to New fns. Other improvements.
		 <3>	12/21/93	VL		Changed XMPStorageUnit::GetValueSize to
									StorageUnit::GetSize.
		 <2>	 12/3/93	T		Stop including XMPError.h, it is included
									as ErrorDef.h inside Except.h
		 <1>	11/29/93	JA		first checked in
		----------------------------Moved from Utilities to Imaging project
		 <4>	11/24/93	JA		cfront fixes.
		 <3>	11/24/93	VL		Rolled back changes as ASLM build breaks
									the THINK build.
		 <2>	11/23/93	VL		Made this work with ASLM.
		 <1>	11/23/93	JA		first checked in

	To Do:
		$$$$$ Improve the equality tests for contours and polygons. See comments
		in the two "operator==" methods for details.
		
		$$$$$ Switch all polygon-related stuff to using winding-number fill instead of
		even-odd. Will require fixing AsQDRegion and Contains.
	
*/


#ifndef _ALTPOINT_
#include "AltPoint.h"			// Use C++ savvy XMPPoint and XMPRect
#endif

#ifndef _ALTPOLY_
#include "AltPoly.h"
#endif

#ifndef _TRNSFORM_
#include "Trnsform.h"
#endif

#ifndef _LINEOPS_
#include "LineOps.h"
#endif

#ifndef _XMPMEM_
#include "XMPMem.h"
#endif

#ifndef _EXCEPT_
#include "Except.h"
#endif

#ifndef _STORAGEU_
#include "StorageU.h"
#endif

#ifndef _STDTYPES_
#include "StdTypes.h"
#endif

#if PLATFORM_MACINTOSH
	#ifndef graphicsErrorsIncludes
	#include "graphics errors.h"
	#endif
	
	#ifndef graphicsRoutinesIncludes
	#include "graphics routines.h"
	#endif
#endif

#include <stddef.h>					// Defines offsetof() macro
#include <Values.h>


const XMPSLong kMaxLong	= 0x7FFFFFFF;


#pragma segment XMPShape


/******************************************************************************/
#pragma mark	GX UTILS
/******************************************************************************/


#pragma segment XMPGXShape


#if PLATFORM_MACINTOSH
static void
ClearGXError( )
{
	GXGetGraphicsError(kXMPNULL);
	// GX error status is cleared after asking for errors.
}


static void
ThrowIfGXError( )
{
	gxGraphicsError err = GXGetGraphicsError(kXMPNULL);		// Get latest graphics error
	if( err )
		THROW(err,"QuickDraw GX error");
}


static void
ThrowIfFirstGXError( )
{
	gxGraphicsError err;
	GXGetGraphicsError(&err);							// Get first error, not last
	if( err )
		THROW(err,"QuickDraw GX error");
}
#endif


/******************************************************************************/
#pragma mark	CREATORS
/******************************************************************************/


void*
XMPPolygon::operator new( size_t size, XMPSLong nVertices )
{
	if( nVertices==0 )
		size = offsetof(XMPPolygon,firstContour);
	else
		size = offsetof(XMPPolygon,firstContour.vertex[nVertices]);
	return ::operator new(size);
}


XMPPolygon*
XMPPolygon::New( XMPSLong nVertices )
{
	ASSERT(nVertices>=0,kXMPErrInvalidValue);
	
	XMPPolygon *poly = new (nVertices) XMPPolygon;
	if( nVertices==0 ) {
		poly->nContours = 0;
	} else {
		poly->nContours = 1;
		poly->firstContour.nVertices = nVertices;
	}
	return poly;
}


XMPPolygon*
XMPPolygon::New( XMPSLong nVertices, const XMPPoint *vertices )
{
	ASSERT(nVertices>=0,kXMPErrInvalidValue);
	ASSERT(vertices!=kXMPNULL,kXMPErrInvalidValue);
	
	XMPPolygon *poly = XMPPolygon::New(nVertices);
	if( nVertices>0 )
		XMPBlockMove( vertices, poly->firstContour.vertex, nVertices*sizeof(XMPPoint) );
	return poly;
}


XMPPolygon*
XMPPolygon::New( XMPSLong nContours, const XMPSLong *contourVertices )
{
	ASSERT(nContours>=0,kXMPErrInvalidValue);
	if( nContours==0 )
		return XMPPolygon::New(0);
	else {
		ASSERT(contourVertices!=kXMPNULL,kXMPErrInvalidValue);
		XMPSLong totalVertices = 0;
		for( XMPSLong i=nContours-1; i>=0; i-- )
			totalVertices += contourVertices[i];
		XMPPolygon *poly = (XMPPolygon*)
			::operator new( (size_t)(offsetof(XMPPolygon,firstContour)
									+ offsetof(XMPContour,vertex[0])*nContours
									+ sizeof(XMPPoint)*totalVertices) );
		poly->nContours = nContours;
		XMPContour *cont = poly->FirstContour();
		for( i=0; i<nContours; i++ ) {
			cont->nVertices = contourVertices[i];
			cont = cont->NextContour();
		}
		return poly;
	}
}


XMPPolygon*
XMPPolygon::New( const XMPRect &r )
{
	XMPPolygon *poly;
	if( r.IsEmpty() )
		poly = XMPPolygon::New(0);
	else {
		poly = XMPPolygon::New(4);
		poly->firstContour.vertex[0] = r.TopLeft();
		poly->firstContour.vertex[1].Set(r.right,r.top);
		poly->firstContour.vertex[2] = r.BotRight();
		poly->firstContour.vertex[3].Set(r.left,r.bottom);
	}
	return poly;
}


XMPPolygon*
XMPPolygon::NewFromPolygon( const XMPPolygon *oldPoly )
{
	XMPULong size = oldPoly->GetDataSize();
	XMPPolygon *poly = (XMPPolygon*) ::operator new( (size_t)size );
	XMPBlockMove(oldPoly,poly,size);
	return poly;
}



#if PLATFORM_MACINTOSH
#pragma segment XMPGXShape

XMPPolygon*
XMPPolygon::NewFromGXShape( gxShape shape )
{
	gxShape copiedShape = kXMPNULL;
	XMPPolygon *poly;
	
	TRY{
		ClearGXError();
		copiedShape = shape = GXCopyToShape(kXMPNULL,shape);
		GXPrimitiveShape(copiedShape);
		GXSimplifyShape(copiedShape);
		GXSetShapeType(copiedShape,gxPolygonType);
		ThrowIfFirstGXError();
		
		XMPSLong size = GXGetPolygonParts(shape, 1,gxSelectToEnd, kXMPNULL);
		ThrowIfGXError();
		poly = (XMPPolygon*) XMPNewPtr( size );
		GXGetPolygonParts(shape, 1,gxSelectToEnd, (gxPolygons*)poly);
		ThrowIfGXError();
		
		GXDisposeShape(copiedShape);
	
	}CATCH_ALL{
		XMPVolatile(copiedShape);
		GXDisposeShape(copiedShape);
		RERAISE;
	}ENDTRY
	
	return poly;
}
#pragma segment XMPShape
#endif


XMPPolygon*
XMPPolygon::NewFromSU( XMPStorageUnit *su )
{
	su->Focus(kXMPNULL,kXMPPosSame, kXMPPolygon,0, kXMPPosUndefined);
	XMPSLong size = su->GetSize();
	if( size==0 )
		return kXMPNULL;						// Inform caller if data is empty
		
	XMPPolygon *poly = (XMPPolygon*) XMPNewPtr(size);
	TRY{
		su->GetValue(size, (XMPValue)poly);
	}CATCH_ALL{
		delete poly;
		RERAISE;
	}ENDTRY
	return poly;
}


/******************************************************************************/
#pragma mark	POLYGON STUFF
/******************************************************************************/


XMPPolygon*
XMPPolygon::Copy( ) const
{
	XMPULong size = this->GetDataSize();
	XMPPolygon *poly = (XMPPolygon*) XMPNewPtr(size);
	XMPBlockMove(this,poly,size);
	return poly;
}


XMPULong
XMPPolygon::GetDataSize( ) const
{
	XMPULong totalPoints = 0;
	const XMPContour *c = &firstContour;
	for( XMPSLong i=nContours; i>0; i-- ) {
		totalPoints += c->nVertices;
		c = c->NextContour();
	}
	return sizeof(nContours) + nContours*sizeof(c->nVertices) + totalPoints*sizeof(XMPPoint);
}


void
XMPPolygon::ComputeBoundingBox( XMPRect *bbox ) const
{
	ASSERT(bbox!=kXMPNULL,kXMPErrInvalidValue);
	
	if( nContours <= 0 ) {
		bbox->Clear();
		return;
	}
	
	// Start bbox out as maximally empty:
	bbox->left	= bbox->top		=  kMaxLong;
	bbox->right = bbox->bottom	= -kMaxLong;
	
	const XMPContour *c = &firstContour;
	for( XMPSLong i=nContours; i>0; i-- ) {
		XMPPoint *pt = c->vertex;
		for( XMPSLong v=c->nVertices; v>0; v--,pt++ ) {
			if( pt->x < bbox->left )	bbox->left	= pt->x;
			if( pt->x > bbox->right )	bbox->right	= pt->x;
			if( pt->y < bbox->top )		bbox->top	= pt->y;
			if( pt->y > bbox->bottom )	bbox->bottom= pt->y;
		}
		if( i>1 )
			c = c->NextContour();
	}
}


Boolean
XMPContour::operator== ( const XMPContour &cont ) const
{
	// FIX: This is a pretty brain-dead approach. The two contours might have the same
	// points, but out of phase and/or in the opposite order. We might also need to
	// introduce an "epsilon" since vertices previously computed as intersections might
	// have a few bits of round-off error.
	
	if( nVertices != cont.nVertices )
		return kXMPFalse;
	
	const XMPPoint *p0 = &this->vertex[0];
	const XMPPoint *p1 = &cont.vertex[0];
	for( XMPSLong i=nVertices; i>0; i-- )
		if( *p0++ != *p1++ )
			return kXMPFalse;
	return kXMPTrue;
}


Boolean
XMPPolygon::operator== ( const XMPPolygon &poly ) const
{
	// FIX: This is a pretty brain-dead approach. The two polygons might have
	// identical contours, but not listed in the same order.
	
	if( nContours != poly.nContours )
		return kXMPFalse;
	
	const XMPContour * c = this->FirstContour();
	const XMPContour *pc = poly.FirstContour();
	for( XMPSLong i=nContours; i>0; i-- ) {
		if( *c != *pc )
			return kXMPFalse;
		if( i>1 ) {
			c = c->NextContour();
			pc=pc->NextContour();
		}
	}
	return kXMPTrue;
}


Boolean
XMPPolygon::IsEmpty( ) const
{
	// FIX: This is again not very smart. It will probably be necessary to compute the area
	// of each contour...
	
	return nContours==0;
}


Boolean
XMPPolygon::IsRectangular( ) const
{
	return nContours==0 || (nContours==1  && firstContour.IsRectangular());
}


Boolean
XMPPolygon::AsRectangle( XMPRect *r ) const
{
	if( nContours==0 ) {
		r->Clear();
		return kXMPTrue;
	} else if( nContours==1 )
		return firstContour.AsRectangle(r);
	else
		return kXMPFalse;
}


Boolean
XMPContour::IsRectangular( ) const
{
	if( nVertices != 4 )
		return kXMPFalse;
	else if( vertex[0].x == vertex[1].x )			// 1st edge is vertical
		return vertex[1].y==vertex[2].y
		    && vertex[2].x==vertex[3].x
		    && vertex[3].y==vertex[0].y;
	else if( vertex[0].y == vertex[1].y )			// 1st edge is horizontal
		return vertex[1].x==vertex[2].x
		    && vertex[2].y==vertex[3].y
		    && vertex[3].x==vertex[0].x;
	else
		return kXMPFalse;
}


Boolean
XMPContour::AsRectangle( XMPRect *r ) const
{
	ASSERT(r!=kXMPNULL,kXMPErrInvalidValue);
	
	if( this->IsRectangular() ) {
		XMPRect r2(vertex[0],vertex[2]);			// C'tor properly orders the coords
		*r = r2;
		return kXMPTrue;
	} else
		return kXMPFalse;
}


/******************************************************************************/
#pragma mark	REGION CONVERSION
/******************************************************************************/


XMPBoolean
XMPContour::HasExactRegion( ) const
{
	const XMPPoint *b = &vertex[0];
	for( XMPSLong i=nVertices; i>=0; i-- ) {
		const XMPPoint *a = &vertex[i];
		if( (a->x & 0xFFFF) || (a->y & 0xFFFF) )
			return kXMPFalse;						// Non-integer coordinates
		if( a->x!=b->x && a->y!=b->y )
			return kXMPFalse;						// Diagonal line
		b = a;
	}
	return kXMPTrue;
}


XMPBoolean
XMPPolygon::HasExactRegion( ) const
{
	const XMPContour *c = this->FirstContour();
	for( long i=nContours; i>=0; i-- ) {
		if( !c->HasExactRegion() )
			return kXMPFalse;
		if( i>0 )
			c = c->NextContour();
	}
	return kXMPTrue;
}


#if PLATFORM_MACINTOSH
PolyHandle
XMPContour::AsQDPolygon( ) const
{
	XMPSLong size = 10 + nVertices*sizeof(Point);
	if( size > 32767 )
		THROW(kXMPErrShapeTooComplex);
	PolyHandle p = (PolyHandle) XMPNewHandle(size);
	(**p).polySize = (short)size;
	Rect &bbox = (**p).polyBBox;
	SetRect(&bbox,32767,32767,-32767,-32767);
	
	const XMPPoint *src = &vertex[0];
	Point *dst = &(**p).polyPoints[0];
	for( XMPSLong i=nVertices; i>0; i-- ) {
		*dst = src->AsQDPoint();
		if( dst->h < bbox.left  )	bbox.left  = dst->h;
		if( dst->v < bbox.top   )	bbox.top   = dst->v;
		if( dst->h > bbox.right )	bbox.right = dst->h;
		if( dst->v > bbox.bottom)	bbox.bottom= dst->v;
		src++;
		dst++;
	}
	return p;
}
#endif


#if PLATFORM_MACINTOSH
RgnHandle
XMPPolygon::AsQDRegion( ) const
{
	GrafPtr port, myPort;

	// Make sure we have a real port to work with:
	GetPort(&port);
	if( FrontWindow() ) {
		SetPort(FrontWindow());
		myPort = kXMPNULL;
	} else {
		myPort = new GrafPort;
		OpenPort(myPort);
	}
	
	RgnHandle rgn = NewRgn();
	THROW_IF_NULL(rgn);
	
	OpenRgn();
	TRY{
		PolyHandle poly;
		const XMPContour *cont = this->FirstContour();
		for( XMPSLong i=nContours; i>0; i-- ) {
			poly = cont->AsQDPolygon();
			FramePoly(poly);
			KillPoly(poly);
			cont = cont->NextContour();
		}
	}CATCH_ALL{
		CloseRgn(rgn);
		DisposeRgn(rgn);
		SetPort(port);
		if( myPort ) {
			ClosePort(myPort);
			delete myPort;
		}
		RERAISE;
	}ENDTRY
	
	CloseRgn(rgn);
	
	SetPort(port);
	if( myPort ) {
		ClosePort(myPort);
		delete myPort;
	}
	return rgn;
}
#endif


void
XMPPolygon::Transform( XMPTransform *xform )
{
	XMPContour *c = &firstContour;
	for( XMPSLong i=nContours; i>0; i-- ) {
		XMPPoint *pt = c->vertex;
		for( XMPSLong v=c->nVertices; v>0; v--, pt++ )
			*pt = xform->TransformPoint(*pt);
		if( i>1 )
			c = c->NextContour();
	}
}


/******************************************************************************/
#pragma mark	CONTAINMENT TEST
/******************************************************************************/


//------------------------------------------------------------------------------
// XMPPolygon::Contains
//
// Does a polygon contain a point?
// We determine this by drawing a ray from the point to the right towards
// infinity, and finding the polygon edges that intersect this ray. For each
// such edge, count it as 1 if its y value is increasing, -1 if decreasing.
// The sum of these values is 0 if the point is outside the polygon.
//------------------------------------------------------------------------------

XMPSLong
XMPPolygon::Contains( XMPPoint point, Boolean *onEdge ) const
{
	XMPSLong count = 0;
	const XMPPoint *pp1, *pp2;
	XMPPoint p1, p2;
	XMPPoint ray = point;
	
	for( PolyEdgeIterator polyIter (this); polyIter.IsNotComplete(); polyIter.Next() ) {
		polyIter.CurrentEdge(pp1,pp2);
		p1 = *pp1;
		p2 = *pp2;
		
		if( p1.y==p2.y ) {											// Horizontal line: ignore
			if( p1.y==point.y && InRange(point.x, p1.x,p2.x) ) {	// unless point is on it
				*onEdge = kXMPTrue;
				return 0;
			}
		} else {
			ray.x = Max(p1.x,p2.x);
			XMPPoint sect;
			if( ray.x >= point.x )
				if( IntersectSegments(p1,p2, point,ray, &sect) ) {
					if( WithinEpsilon(point.x,sect.x) && WithinEpsilon(point.y,sect.y) ) {
						*onEdge = kXMPTrue;
						return 0;
					}
					if( p2.y > p1.y )
						count++;
					else
						count--;
				}
		}
	}
	
	*onEdge = kXMPFalse;
	return count;
}


/******************************************************************************/
#pragma mark	POLYGON EDGE ITERATOR
/******************************************************************************/


PolyEdgeIterator::PolyEdgeIterator( const XMPPolygon *poly )
	:fPoly (poly)
{
	fCurContour = poly->FirstContour();
	fCurContourIndex = 0;
	fCurVertex = 0;
}


void
PolyEdgeIterator::CurrentEdge( const XMPPoint* &v1, const XMPPoint* &v2 )
{
	v1 = &fCurContour->vertex[fCurVertex];
	if( fCurVertex+1 < fCurContour->nVertices )
		v2 = v1+1;
	else
		v2 = &fCurContour->vertex[0];
}


Boolean
PolyEdgeIterator::Next( )
{
	if( !fCurContour )									// Was already finished
		return false;
	if( ++fCurVertex >= fCurContour->nVertices )		// Next vertex; if past contour:
		if( ++fCurContourIndex >= fPoly->nContours ) {		// Next contour; if past end:
			fCurContour = kXMPNULL;
			return false;										//...we're done.
		} else {
			fCurContour = fCurContour->NextContour();			// Else go to start of contour
			fCurVertex = 0;
		}
	return kXMPTrue;
}


Boolean
PolyEdgeIterator::IsNotComplete( )
{
	return fCurContour!=kXMPNULL;
}