/*
	File:		AltPoly.h

	Contains:	OpenDoc polygon: optional C++ savvy classes

	Written by:	Jens Alfke

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <2>	 4/13/94	T		jpa: Added PolyEdgeIterator from
									now-obsolete PolyIter.h. (1145831)
		 <1>	 2/17/94	JA		first checked in
		----------------------------Rename AltPoly, move to TestParts since this
									is not a part of the public API.
		 <6>	 2/16/94	JA		Stop including Point.h.
		 <5>	  2/8/94	JA		Include <FixMath.h> before GX header to
									avoid conflicts.
		 <4>	  2/7/94	JA		Only declare Mac-specific methods if on Mac
									platform.
		 <3>	  2/3/94	JA		Tiger Team Makeover!
		 <2>	 1/31/94	JA		Added Contains and AsQDRegion and improved
									New methods.
		 <1>	11/29/93	JA		first checked in
		----------------------------Moved from Utilities to Imaging project
		 <4>	11/24/93	VL		Rolled back changes as ASLM build breaks
									the THINK build.
		 <3>	11/23/93	VL		Made this work with ASLM.
		 <2>	11/23/93	VL		Added ASLM string.
		 <1>	11/23/93	JA		first checked in

	To Do:
	
	
	Theory of Operation:
	
		This is an alternate definition of XMPPolygon and XMPContour. The data format is
		identical, but the structs defined here have a lot of useful methods including
		constructors, accessors and conversion operators.
		
		To use these instead of the regular structs defined in Polygon.h, just include
		this header file _before_ Polygon.h. An easy way to do this is to include it
		first.
	
		QuickDraw GX users take note:
		XMPContour is identical in data format to a gxPolygon.
		XMPPolygon is identical in data format to a gxPolygons. (See <graphics types.h>)
		
		You cannot directly instantiate XMPPolygons (by declaring an XMPPolygon variable,
		or via "new XMPPolygon") since they are variable-size objects. The compiler will
		complain if you try. Instead, use the static "New" methods.
*/


#ifndef _ALTPOLY_
#define _ALTPOLY_

#ifdef _POLYGON_
	#error "Must include AltPoly.h *before* Polygon.h!"
#else
	#define _POLYGON_		/* Make sure Polygon.h does NOT get included later! */
#endif

#ifndef _OBJECT_
#include "XMPObj.h"
#endif

#include <stddef.h>					// for size_t

#if PLATFORM_MACINTOSH
	#ifndef __QUICKDRAW__
	#include <QuickDraw.h>				// for Region and Polygon types
	#endif
	#ifndef __FIXMATH__
	#include <FixMath.h>				// Must include before GX headers...
	#endif
	#ifndef graphicsTypesIncludes
	#include "graphics types.h"			// for gxShape type
	#endif
#endif


//==============================================================================
// Classes used in this interface
//==============================================================================

struct XMPRect;
class XMPStorageUnit;
class XMPTransform;

//==============================================================================
// XMPContour
//==============================================================================

#define kXMPContourID "appl:xmpcontour$class,1.0.0"

struct XMPContour {
	public:
	
	XMPSLong	nVertices;
	XMPPoint	vertex[1];		// Array size is actually nVertices
	
	XMPContour* NextContour( )				const	{return (XMPContour*)&vertex[nVertices];}
	XMPBoolean	IsRectangular( )			const;
	XMPBoolean	AsRectangle( XMPRect* )		const;
#if PLATFORM_MACINTOSH
	PolyHandle	AsQDPolygon( )				const;
	XMPBoolean	HasExactRegion( )			const;
#endif
	
	XMPBoolean	operator== ( const XMPContour& )			const;
	XMPBoolean	operator!= ( const XMPContour &c )			const	{return !(*this==c);}
		// Equality tests not fully robust yet; may generate false negatives.

	private:
	
	XMPContour( ) { }			// Don't allow foreigners to create contours themselves
	friend struct XMPPolygon;
};

//==============================================================================
// XMPPolygon
//==============================================================================

#define kXMPPolygonID "appl:XMPPolygon$class,1.0.0"

struct XMPPolygon {
	public:
	
	// DATA MEMBERS:
	
	XMPSLong	nContours;
	XMPContour	firstContour;				// Rest of contours follow after first
								// firstContour field does not exist if nContours==0!
	
	// CREATION:
	
	static XMPPolygon*	New( XMPSLong nVertices );
	static XMPPolygon*	New( XMPSLong nVertices, const XMPPoint *vertices );
	static XMPPolygon*	New( XMPSLong nContours, const XMPSLong *contourVertices );
	static XMPPolygon*	New( const XMPRect& );
	static XMPPolygon*	NewFromPolygon( const XMPPolygon* );
	static XMPPolygon*	NewFromSU( XMPStorageUnit* );
#if PLATFORM_MACINTOSH
	static XMPPolygon*	NewFromGXShape( gxShape );		// Accepts rect, polygon(s), path(s)
#endif
	
	void		operator delete( void *obj )	{::delete obj;}		//Keeps compiler happy

	// ACCESSORS:  (Alternatives to directly accessing the data members.)
	
	XMPSLong	GetNContours( )								const	{return nContours;}
	XMPContour*	FirstContour( )										{return &firstContour;}
	const XMPContour*	FirstContour( )						const	{return &firstContour;}
	XMPSLong	GetNVertices( XMPSLong contour )			const;
	XMPPoint	GetVertex( XMPSLong contour, XMPSLong vertex ) const;
	XMPPoint	SetVertex( XMPSLong contour, XMPSLong vertex, const XMPPoint& );
	
	// GEOMETRY:
	
	void		ComputeBoundingBox( XMPRect* )				const;
	XMPBoolean	IsRectangular( )							const;
	void		Transform( XMPTransform* );
	
	XMPBoolean	operator== ( const XMPPolygon& )			const;
	XMPBoolean	operator!= ( const XMPPolygon &poly )		const	{return !(*this==poly);}
		// Equality tests not fully robust yet; may generate false negatives.
	
	// TESTS:
	
	XMPSLong	Contains( XMPPoint, XMPBoolean *onEdge )	const;
	XMPBoolean	IsEmpty( )									const;
	
	// CONVERSIONS:
	
	XMPBoolean	AsRectangle( XMPRect* )						const;	// Returns false if impossible
#if PLATFORM_MACINTOSH
	XMPBoolean	HasExactRegion( )							const;
	RgnHandle	AsQDRegion( )								const;
#endif
	
	// MEMORY:
	
	XMPULong	GetDataSize( )								const;
	XMPPolygon*	Copy( )										const;
	
	private:
	
	void*		operator new( size_t, long );
	XMPPolygon( ) { }		// Don't allow foreigners to create polygons themselves
};


//==============================================================================
// Polygon Edge Iterator
//==============================================================================


#define kPolyEdgeIteratorID "appl:PolyEdgeIterator$class,1.0.0"

class PolyEdgeIterator {
	public:

	PolyEdgeIterator( const XMPPolygon* );
	
	void		CurrentEdge( const XMPPoint* &v1, const XMPPoint* &v2 );
	const XMPContour* CurrentContour( )			{return fCurContour;}
	long		CurrentContourIndex( )			{return fCurContourIndex;}
	long		CurrentEdgeIndex( )				{return fCurVertex;}
	
	XMPBoolean	Next( );
	XMPBoolean	IsNotComplete( );
	
	private:
	const XMPPolygon*	const fPoly;
	const XMPContour*		  fCurContour;
	long					  fCurContourIndex;
	long					  fCurVertex;
};


#endif //_ALTPOLY_