/*
	File:		FocusLib.h

	Contains:	Library routines for focusing (setting up for drawing into a facet)

	Written by:	Jens Alfke

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):
	
		 <4>	 4/13/94	SS		changes for laserwriter printing support
		 <3>	  2/2/94	JA		Added CFocusWindow and PS calls.
		 <2>	11/23/93	VL		Made this work with ASLM.
		 <1>	11/23/93	JA		first checked in
		 
	THEORY OF OPERATION:
	
		FocusLib sets up the drawing environment for a QuickDraw-based part so that
		it can start drawing into a facet. It provides both a one-time change of
		focus (via the Focus call) and a way to set the focus but restore it later
		when the current scope exits (via the CFocus object.)
		
		Focus() and FocusWindow() are slightly different for offscreen facets. If the
		facet is in an offscreen canvas, Focus will set up to draw into that canvas;
		this is what one usually wants to do. On the other hand, FocusWindow will
		always focus onto an on-screen canvas, even for an off-screen facet. Use
		FocusWindow for things like rubber-banding where your drawing needs to show
		up immediately, even if the facet is offscreen.

		The CFocus object deserves an example:
		
		void DrawSomething( XMPFacet *facet )
		{
			CFocus f(facet);		// Creates a CFocus object on the stack, which
									// changes the focus but stores the old state
			....drawing code...
		}
		
		Declaring the CFocus object calls its constructor, which sets the focus and
		stores the old state in the CFocus object itself (on the stack.) When its
		scope exists, the CFocus object's destructor is called, which restores the
		old focus.
		
		This can cause trouble with exceptions, because our exception package currently
		does NOT destruct objects when their scope disappears as a result of a Throw.
		You must do this manually:
	
		void DrawSomething( XMPFacet *facet )
		{
			CFocus f(facet);
			TRY{
				...drawing code that might Throw an exception...
			}CATCH_ALL{
				f.Defocus();
			}ENDTRY
		}
		
		This example function catches any exception and manually restores the old focus
		before letting the exception continue.
*/


#ifndef _FOCUSLIB_
#define _FOCUSLIB_

#ifndef __QUICKDRAW__
#include <QuickDraw.h>
#endif

#ifndef __PRINTING__
#include <Printing.h>
#endif


class XMPMacFacet;						// Can't use "XMPFacet" for some reason
class XMPShape;


void Focus( XMPMacFacet *facet );
void FocusWindow( XMPMacFacet *facet );

#define kCFocusID "appl:cfocus$class"

struct PrintingInfo {
	TPPrPort	printingPort;
	THPrint	printRecord;
};
typedef PrintingInfo *PrInfoPtr;

class CFocus {
	public:
	CFocus( XMPMacFacet* );
	~CFocus( )					{this->Defocus();}
	
	void Focus( XMPFacet *facet, Boolean toWindow, GrafPtr *oldPort,
				Point *oldOrigin, RgnHandle *oldClip );
	void Defocus( );
	
	protected:
	CFocus( )	{ }
	GrafPtr		fPort;
	Point		fOrigin;
	RgnHandle	fClip;
	Boolean		fPrintingPostScript;
};


class CFocusWindow :public CFocus {
	public:
	CFocusWindow( XMPMacFacet* );
};


// XMPBeginPostScriptClip sets the PostScript clipping path to the given shape.
// XMPEndPostScriptClip resets PostScript clipping to the full page.
// Neither has any effect except while printing to a PostScript printer.
// WARNING: UNDER CONSTRUCTION. These calls have not been used or tested yet.

void XMPBeginPostScriptClip( XMPShape* );
void XMPEndPostScriptClip( );


#endif //_FOCUSLIB_