/*
	File:		XMPPart.h

	Contains:	Class definition for XMPPart

	Written by:	Joshua Susser

	Protocols:	DragAndDrop, Embedding, Frames, Imaging, Linking,
				Memory Management, Part Activation, Storage, UI Events, Undo

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<39>	  4/6/94	JBS		1155477, 1155479, 1155480
		<38>	  4/5/94	CC		ExternalizeKinds() parameter changed from
									XMPTypeSet to XMPTypeList. (1153046)
		<37>	 3/25/94	JA		JBS: Changed API of HighlightChanged,
									RevealFrame, CreateEmbeddedFrameIterator,
									CanvasChanged. (1148310, 1153052, 1153056,
									1153059)
		<36>	 3/25/94	VL		#1147173: Moved constructor, destructor and
									init methods to private by convention.
		<35>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<34>	  2/2/94	JBS		removed sourceFrame from AddDisplayFrame();
									added AttachSourceFrame()
		<33>	  2/2/94	NP		Added methods for new linking protocol
									APIs.
		<32>	  2/1/94	CC		Added RevealLink method.
		<31>	 1/31/94	JBS		added GetPrintResolution()
		<30>	 1/27/94	VL		Changed CloneTo to CloneInto.
		<29>	 1/24/94	CC		Added XMPChangeID parameter to
									LinkUpdated().
		<28>	 1/20/94	SS		::Open returns a window id
		<27>	 1/11/94	T		Init... changes
		<26>	12/21/93	JBS		add viewType and presentation calls
		<25>	12/16/93	JBS		added RevealFrame(), EmbeddedFrameSpec(),
									CanvasUpdated()
		<24>	12/13/93	T		InitPart changes
		<23>	12/12/93	T		Added ExternalizeKinds, and ChangeKind
		<22>	11/30/93	LM		Added an XMPFrame parameter to ReadPartInfo
		<21>	11/16/93	RR		Added AdjustMenus (not yet called), and
									MouseWithin. Removed point from MouseLeave
		<20>	11/15/93	CC		Changed parms of CreateLink() to XMPPtr &
									XMPULong
		<19>	11/12/93	JBS		use Facets
		<18>	10/21/93	PH		Change FulfillPromise call
		<17>	10/18/93	CG		Put con/de-structor defs back the way they
									were.
		<16>	10/14/93	RR		Added sourceFrame parameter to
									AddDisplayFrame
		<15>	10/12/93	PH		Remove DragDrop.h
		<13>	 10/7/93	CG		ASLM wants constructor impl in .cp.
		<12>	 10/6/93	JBS		remove fSession field; inline
									con/de-structor
		<11>	 9/30/93	PH		Add support for multiple drag items
		<10>	 9/29/93	VL		Added InitializePersistent.
		 <9>	  8/9/93	NP		Added class id string.
		 <8>	  8/6/93	JBS		add HighlightChanged() method
		 <7>	 7/21/93	NP		Added a virtual keyword (temporary for
									ASLM).
		 <6>	  7/9/93	JBS		replaced with prototype version
		 <5>	 6/23/93	RCR		Use Tokenized strings
		 <4>	 4/29/93	T		change names of includefiles
		 <3>	 4/21/93	JBS		remove fSU; add friend class
									XMPEmbeddedFramesIterator
		 <2>	  4/13/93	RCR		Removed XMPInclude.h
		 <1>	  4/9/93	JBS		first checked in
*/

#ifndef _PART_
#define _PART_

#ifndef _PLFMDEF_
#include "PlfmDef.h"
#endif

#ifndef _PSTOBJ_
#include "PstObj.h"
#endif

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _DRAFT_
#include "Draft.h"	// For XMPDraftKey
#endif


//==============================================================================
// Constants
//==============================================================================

const XMPID kNewGroup = 0;

//==============================================================================
// Classes defined in this interface
//==============================================================================

class XMPPart;
class XMPEmbeddedFramesIterator;

//==============================================================================
// Classes used by this interface
//==============================================================================

class XMPCanvas;
class XMPFacet;
class XMPFrame;
class XMPLink;
class XMPPersistentObject;
class XMPShape;
class XMPStorageUnit;
class XMPStorageUnitView;
class XMPTransform;
class XMPDragItemIterator;
class XMPSession;
class XMPTypeList;

struct XMPPoint;

//==============================================================================
// XMPPart
//==============================================================================

#define kXMPPartID "appl:part$class,1.0.0"

class XMPPart : public XMPPersistentObject
{
	friend class XMPDraft;
	friend class XMPEmbeddedFramesIterator;

public:

	//-------------------------------------------------------------------------
	// From DragAndDrop protocol
	//-------------------------------------------------------------------------

	XMPVMethod void FulfillPromise(XMPStorageUnitView *promiseSUView)
		= 0;
	
	XMPVMethod void DropCompleted(XMPPart* destPart, XMPDropResult dropResult)
		= 0;
		
	XMPVMethod void DragEnter(XMPDragItemIterator* dragInfo,
							  XMPFacet* facet,
							  XMPPoint where)
		= 0;

	XMPVMethod void DragWithin(XMPDragItemIterator* dragInfo,
							   XMPFacet* facet,
							   XMPPoint where)
		= 0;

	XMPVMethod void DragLeave(XMPFacet* facet,
							  XMPPoint where)
		= 0;

	XMPVMethod XMPDropResult Drop(XMPDragItemIterator* dropInfo,
								  XMPFacet* facet,
								  XMPPoint where)
		= 0;

	//-------------------------------------------------------------------------
	// From Embedding protocol
	//-------------------------------------------------------------------------

	XMPVMethod void ContainingPartPropertiesChanged(XMPFrame* frame,
													XMPStorageUnit* propertyUnit)
		= 0;

	// ********* for containing parts *********

	XMPVMethod XMPStorageUnit* GetContainingPartProperties(XMPFrame* frame)
		= 0;

	XMPVMethod void RevealFrame(XMPFrame* embeddedFrame, XMPShape* revealShape)
		= 0;
		
	XMPVMethod void EmbeddedFrameSpec(XMPFrame* embeddedFrame, XMPObjectSpec* spec)
		= 0;
		
	XMPVMethod XMPEmbeddedFramesIterator* CreateEmbeddedFramesIterator(XMPFrame* frame)
		= 0;

	//-------------------------------------------------------------------------
	// From Frames protocol
	//-------------------------------------------------------------------------

	XMPVMethod void AddDisplayFrame(XMPFrame* frame)
		= 0;

	XMPVMethod void AttachSourceFrame(XMPFrame* frame, XMPFrame* sourceFrame)
		= 0;

	XMPVMethod void RemoveDisplayFrame(XMPFrame* frame)
		= 0;

	XMPVMethod void CloseDisplayFrame(XMPFrame* frame)
		= 0;

	XMPVMethod void FrameShapeChanged(XMPFrame* frame)
		= 0;

	XMPVMethod void ViewTypeChanged(XMPFrame* frame)
		= 0;

	XMPVMethod void PresentationChanged(XMPFrame* frame)
		= 0;

	XMPVMethod void SequenceChanged(XMPFrame* frame)
		= 0;

	XMPVMethod void WritePartInfo(XMPPtr partInfo,
								  XMPStorageUnitView* storageUnitView)
		= 0;

	XMPVMethod XMPPtr ReadPartInfo(XMPFrame* frame, XMPStorageUnitView* storageUnitView)
		= 0;

	XMPVMethod XMPID Open(XMPFrame* frame)
		= 0;

	//---------------------------------
	// for containing parts

	XMPVMethod XMPFrame* RequestEmbeddedFrame(XMPFrame* containingFrame,
											  XMPFrame* baseFrame,
											  XMPShape* frameShape,
											  XMPPart* embedPart,
											  XMPTypeToken viewType,
											  XMPTypeToken presentation,
											  XMPBoolean isOverlaid)
		= 0;
		
	XMPVMethod void RemoveEmbeddedFrame(XMPFrame* embeddedFrame)
		= 0;

	XMPVMethod XMPShape* RequestFrameShape(XMPFrame* embeddedFrame, XMPShape* frameShape)
		= 0;

	XMPVMethod void UsedShapeChanged(XMPFrame* embeddedFrame)
		= 0;

	XMPVMethod XMPShape* AdjustBorderShape(XMPFrame* embeddedFrame, XMPShape* shape)
		= 0;

	//-------------------------------------------------------------------------
	// From Facet protocol
	//-------------------------------------------------------------------------

	XMPVMethod void FacetAdded(XMPFacet* facet)
		= 0;

	XMPVMethod void FacetRemoved(XMPFacet* facet)
		= 0;

	XMPVMethod void CanvasChanged(XMPFacet* facet)
		= 0 ;

	XMPVMethod void GeometryChanged(XMPFacet* facet)
		= 0;

	//-------------------------------------------------------------------------
	// From Imaging protocol
	//-------------------------------------------------------------------------

	XMPVMethod void Draw(XMPFacet* facet, XMPShape* invalidShape)
		= 0;

	XMPVMethod void CanvasUpdated(XMPCanvas* canvas)
		= 0 ;

	XMPVMethod void HighlightChanged(XMPFacet* facet)
		= 0 ;

	XMPVMethod XMPULong GetPrintResolution(XMPFrame* frame)
		= 0 ;

	//-------------------------------------------------------------------------
	// From Linking protocol
	//-------------------------------------------------------------------------

	XMPVMethod XMPLink* CreateLink(XMPPtr data, XMPULong size)
		= 0;

	XMPVMethod void LinkUpdated(XMPLink* updatedLink, XMPChangeID id)
		= 0;

	XMPVMethod void RevealLink(XMPLinkSource* linkSource)
		= 0;

	XMPVMethod void	EmbeddedFrameChanged(XMPFrame* frame, XMPChangeID change)
		= 0;

	XMPVMethod void	LinkStatusChanged(XMPFrame* frame)
		= 0;

	//-------------------------------------------------------------------------
	// From Part Activation protocol
	//-------------------------------------------------------------------------

	XMPVMethod XMPBoolean BeginRelinquishFocus(XMPTypeToken focus,
											   XMPFrame* ownerFrame,
											   XMPFrame* proposedFrame)
		= 0;

	XMPVMethod void CommitRelinquishFocus(XMPTypeToken focus, 
										  XMPFrame* ownerFrame,
										  XMPFrame* proposedFrame)
		= 0;
							
	XMPVMethod void AbortRelinquishFocus(XMPTypeToken focus,
										 XMPFrame* ownerFrame,
										 XMPFrame* proposedFrame)
		= 0;
								  
	XMPVMethod void FocusAcquired(XMPTypeToken focus,
								  XMPFrame* ownerFrame)
		= 0;
	
	XMPVMethod void FocusLost(XMPTypeToken focus,
							  XMPFrame* ownerFrame)
		= 0;
									

	//-------------------------------------------------------------------------
	// From Storage protocol
	//-------------------------------------------------------------------------

	XMPVMethod void CloneInto(XMPDraftKey key, XMPStorageUnit* storageUnit, XMPStorageUnit* initiatingFrame)
		= 0;

	//-------------------------------------------------------------------------
	// From Binding protocol
	//-------------------------------------------------------------------------

	XMPVMethod void ExternalizeKinds(XMPTypeList* kindList)
		= 0;

	XMPVMethod void ChangeKind(XMPType kind)
		= 0;

	//-------------------------------------------------------------------------
	// From UI Events protocol
	//-------------------------------------------------------------------------

	XMPVMethod XMPBoolean HandleEvent(XMPEventData event,
									  XMPFrame* frame,
									  XMPFacet* facet)
		= 0;

	XMPVMethod XMPBoolean HandleEventInEmbedded(XMPEventData event,
												XMPFrame* frame,
												XMPFacet* facet,
												XMPFrame* embeddedFrame,
												XMPFacet* embeddedFacet)
		= 0;

	XMPVMethod void MouseEnter(XMPFacet* facet, XMPPoint where)
		= 0;
	
	XMPVMethod void MouseWithin(XMPFacet* facet, XMPPoint where)
		= 0;
	
	XMPVMethod void MouseLeave(XMPFacet* facet)
		= 0;
	
	XMPVMethod void AdjustMenus(XMPFrame* frame)
		= 0;
		
	//-------------------------------------------------------------------------
	// From Undo protocol
	//-------------------------------------------------------------------------

	XMPVMethod void UndoAction(XMPActionData actionState)
		= 0;

	XMPVMethod void RedoAction(XMPActionData actionState)
		= 0;

	XMPVMethod void DisposeActionState(XMPActionData actionState,
									   XMPDoneState doneState)
		= 0;

	XMPVMethod void WriteActionState(XMPPtr actionState,
									 XMPStorageUnitView* storageUnitView)
		= 0;

	XMPVMethod XMPPtr ReadActionState(XMPStorageUnitView* storageUnitView)
		= 0;

public:	// private by convention

	//-------------------------------------------------------------------------
	// constructors/destructors
	//-------------------------------------------------------------------------

	XMPPart();

	virtual ~ CDECL XMPPart();

	XMPVMethod void InitPart(XMPStorageUnit* storageUnit)
		= 0;
		
	XMPVMethod void InitPartFromStorage(XMPStorageUnit* storageUnit)
		= 0;
};


//==============================================================================
// XMPEmbeddedFramesIterator
//==============================================================================

class XMPEmbeddedFramesIterator
{

public:

	//-------------------------------------------------------------------------
	// from Embedding protocol:
	//-------------------------------------------------------------------------

	XMPVMethod XMPFrame*	First() = 0;
	XMPVMethod XMPFrame*	Next() = 0;
	XMPVMethod XMPBoolean	IsNotComplete() = 0;
};

#endif // _PART_
