/*
	File:		CALib.h

	Contains:	Container Application Library header

	Written by:	Tantek elik, Jens Alfke, Joshua Susser

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <6>	 4/20/94	jpa		Added error reporting calls.
		 <5>	 4/19/94	JA		Got it to compile!
		 <4>	 3/25/94	JBS		layout, menu & UI fixes
		 <3>	 3/21/94	T		use pascal calling conventions, reconcile
									CALength into CASize, length as out
									parameter in CASaveDocument
		 <2>	 3/18/94	JBS		imaging changes, remove XMP-types, install
									callbacks
		 <1>	 3/17/94	T		first checked in
	To Do:
		- Implement
*/

#ifndef _CALIB_
#define _CALIB_

#ifndef __DRAG__
#include <Drag.h>			// For DragTrackingHandler type, etc.
#endif

#ifndef __EDITIONS__
#include <Editions.h>		// For SectionHandle type, etc.
#endif


//=========================================================================
// Constants
//=========================================================================

#define kCAHasOpenDoc		// Gestalt selector

//=========================================================================
// Scalar Types
//=========================================================================

//-------------------------------------------------------------------------
// General
//-------------------------------------------------------------------------

typedef unsigned long		CASize;
typedef unsigned long		CAOffset;
typedef char*				CAISOStr;

//-------------------------------------------------------------------------
// Storage
//-------------------------------------------------------------------------

// Note: we used pointers to fake structs instead of pointers to void to
// permit better typechecking by the compiler.

typedef struct CADocumentStruct
{
	long fBogus;
} CADocumentStruct;
typedef CADocumentStruct*	CADocumentRef;

typedef struct CAFrameStruct
{
	long fBogus;
} CAFrameStruct;
typedef CAFrameStruct*	CAFrameRef;
// Note: a CAFrameRef is only valid within the context of a CADocumentRef


//-------------------------------------------------------------------------
// Imaging
//-------------------------------------------------------------------------

typedef struct CATransformStruct
{
	long fBogus;
} CATransformStruct;
typedef CATransformStruct* CATransform;

struct CAPoint {
	Fixed x, y;
};

struct CAContour {
	long		nVertices;					// Variable-size struct:
	CAPoint		vertex[1];					// Array size is actually nVertices
};

struct CAPolygon {
	long		nContours;					// Variable-size struct:
	CAContour	firstContour;				// Rest of contours follow after first
};

enum {
	kCAShapeAsRect,			// Rect*
	kCAShapeAsRegion,		// RgnHandle
	kCAShapeAsPolygon		// CAPolygon*
};
typedef short CAShapeFormat;

//-------------------------------------------------------------------------
// Layout
//-------------------------------------------------------------------------

typedef struct CAVisFrameStruct
{
	long fBogus;
} CAVisFrameStruct;
typedef CAVisFrameStruct* CAVisFrame;

enum {
	kCANoHighlight = 0,
	kCAFullHighlight,
	kCADimHighlight
};
typedef short CAHighlight;

//-------------------------------------------------------------------------
//	Arbitration / Focusing
//-------------------------------------------------------------------------

enum {
	kCAPrimaryFocus		// There is only one focus now (but room for expansion)
};
typedef short CAFocusType;

//-------------------------------------------------------------------------
//	Menus
//-------------------------------------------------------------------------

typedef short	CAMenuID;
typedef short	CAMenuItemID;
typedef long	CACommandID;

//-------------------------------------------------------------------------
//	Window Management
//-------------------------------------------------------------------------

enum {
	kCANoLayer,
	kCADocumentWindow,
	kCAFloatingWindow
};
typedef short CAWindowLayer;


//=========================================================================
// Functions
//=========================================================================


#ifdef __cplusplus
extern "C" {
#endif


//-------------------------------------------------------------------------
// Error reporting
//-------------------------------------------------------------------------

pascal	OSErr		CAError( void );

typedef void 	(*	CAErrorProc)( OSErr );

pascal	CAErrorProc	CASetErrorProc( CAErrorProc );

//-------------------------------------------------------------------------
// Session
//-------------------------------------------------------------------------

pascal	void	CAInit( void );
pascal	void	CAShutdown( void );

//-------------------------------------------------------------------------
// Document
//-------------------------------------------------------------------------

pascal	CADocumentRef	CACreateDocument( FSSpecPtr docSpec );
pascal	void			CASetDocumentKind( CADocumentRef document, CAISOStr kind );
pascal	CADocumentRef	CAOpenDocument( FSSpecPtr docSpec, 
									CAOffset offset, CASize length );
// if offset & length are both 0 then the entire document is used as
// an XMPContainer in place
// else the XMPContainer at the offset&length is copied out to a temp file
// and manipulated there.  When the document is saved or closed,
// the temp file is copied back (in the case of Save, to the indicated
// offset & length). Save may also indicate a new file to begin using.

pascal	Boolean			CAHasDocumentChanged( CADocumentRef );
pascal	void 			CASaveDocument( CADocumentRef, 
										CAOffset offset, CASize* length,
										FSSpecPtr newFile );
pascal	void 			CACloseDocument( CADocumentRef );

//-------------------------------------------------------------------------
// Stream
//-------------------------------------------------------------------------

pascal	void	CARead( CADocumentRef document,char* buffer,CASize* length );
pascal	void	CAWrite( CADocumentRef document,char* buffer,CASize* length );
pascal	CASize	CAGetSize( CADocumentRef document );
pascal	CASize	CAGetOffset( CADocumentRef document );
pascal	void	CASetOffset( CADocumentRef document, CASize offset );
pascal	void	CAInsert( CADocumentRef document,char* buffer,CASize* length );
pascal	void	CADelete( CADocumentRef document,char* buffer,CASize* length );

//-------------------------------------------------------------------------
// Data Interchange
//-------------------------------------------------------------------------

pascal	void	CABeginClone( CADocumentRef sourceDocument, CADocumentRef destDocument );
pascal	CAFrameRef	CAGetRootFrameRef( CADocumentRef document );
pascal	void	CAEndClone( CADocumentRef sourceDocument, CADocumentRef destDocument );

pascal	CAFrameRef CACloneFrameRef( 
	CAFrameRef		sourceFrameRef,		
			// can be NULL if only one frame in source document
	CADocumentRef	sourceDocument,
	CADocumentRef	destDocument,
	Boolean			isRootFrameRef );

pascal	void	CARemoveFrameRef( CADocumentRef document, CAFrameRef frame );

// when copying a bunch of data from one document to another,
// BeginClone, then Clone the FrameRefs, the stream out native content using
// the newly returned FrameRefs, then EndClone
// if none of the Cloned FrameRefs had isRootFrameRef set to true, then EndClone
// sets the native content as the root.

//-------------------------------------------------------------------------
// Clipboard
//-------------------------------------------------------------------------

pascal	CADocumentRef CAGetClipboardDocument( void );	// returns NULL if no part on clipboard

// to paste from the clipboard, 
//	CloneFrameRef(clipbdFrameRef,clipbdDocument,destDocument)
// clipbdFrameRef is cloned from the clipboard to destDocument, and its 
// CAFrameRef within the context of destDocument is returned

// to copy to the clipboard, 
//	CloneFrameRef(sourceFrameRef,sourceDocument,clipbdDocument)
// sourceFrameRef is cloned from sourceDocument to the clipboard, and its
// CAFrameRef within the context of the clipboard is returned

// Use Stream API to write native contents to the ClipDocument

//-------------------------------------------------------------------------
// Drag and Drop
//-------------------------------------------------------------------------

pascal	CADocumentRef CAGetDragDropDocument( void );
// same recipe as Clipboard

// for backward compatibilty with Apps which already support Drag Manager
pascal	OSErr CAInstallTrackingHandler( DragTrackingHandler trackingHandler,
								 WindowPtr theWindow,
								 void *handlerRefCon );
pascal	OSErr CAInstallReceiveHandler( DragReceiveHandler receiveHandler,
								 WindowPtr theWindow,
								 void *handlerRefCon );
								 
pascal	OSErr CARemoveTrackingHandler( DragTrackingHandler trackingHandler,
								 WindowPtr theWindow );

pascal	OSErr CARemoveReceiveHandler( DragReceiveHandler receiveHandler,
								 WindowPtr theWindow );
	
pascal	OSErr CASetDragSendProc	( DragReference theDragRef,
						 DragSendDataProc sendProc,
						 void *dragSendRefCon );


//-------------------------------------------------------------------------
// Linking
//-------------------------------------------------------------------------

pascal	CADocumentRef CAGetEditionDocument( SectionHandle* section ); //jpa: Is this param right?
// same recipe as Clipboard

//-------------------------------------------------------------------------
//	Window Management
//-------------------------------------------------------------------------

pascal	CAWindowLayer	CAIsPartWindow(  WindowPtr );

pascal	WindowPtr		CAGetFrontDocWindow( void );
pascal	WindowPtr		CAGetFrontFloatingWindow( void );
pascal	void			CASelectWindow( WindowPtr );


//-------------------------------------------------------------------------
//	Window Management Callbacks (Optional)
//-------------------------------------------------------------------------

// $$$$$ Need to define routine descriptors for these callbacks!
typedef WindowPtr	(*CAAllocWindowRecProc)		( CAWindowLayer );
typedef void		(*CADisposeWindowRecProc)	( WindowPtr );

pascal	void	CAInstallWindowAllocator( CAAllocWindowRecProc,
										  CADisposeWindowRecProc );


typedef WindowPtr	(*CAFrontWindowProc)	( CAWindowLayer );
typedef void		(*CASelectWindowProc)	( WindowPtr );
typedef Boolean		(*CAIsFloaterProc)		( WindowPtr );

pascal	void	CAInstallFloatingWindowHandlers( CAFrontWindowProc,
										 CASelectWindowProc,
										 CAIsFloaterProc );

//-------------------------------------------------------------------------
// Arbitration
//-------------------------------------------------------------------------


pascal	Boolean		CARequestStandardFocusSet( WindowPtr window );
pascal	Boolean		CARequestModalFocus( WindowPtr window );

pascal	void		CARelinquishStandardFocusSet( WindowPtr window );
pascal	void		CARelinquishModalFocus( WindowPtr window );

// callbacks

typedef 	void (* CAFocusAcquiredProc)	( CAFocusType );
typedef 	void (* CAFocusLostProc)		( CAFocusType );

pascal	void		CAInstallFocusNotification( CAAllocWindowRecProc,
												CADisposeWindowRecProc );

pascal	void		CAFocusAcquired( CAFocusType focus );
pascal	void		CAFocusLost( CAFocusType focus );

//-------------------------------------------------------------------------
// Layout
//-------------------------------------------------------------------------

pascal	RgnHandle	CAGetFrameRgn( CADocumentRef document, CAFrameRef frame );
pascal	void		CASetFrameRgn( CADocumentRef document, CAFrameRef frame,
						RgnHandle frameRgn );
pascal	RgnHandle	CAGetUsedRgn( CADocumentRef document, CAFrameRef frame );

pascal	Boolean		CAIsVisible( CADocumentRef document, CAFrameRef frame );
pascal	CAVisFrame	CAMakeVisible( CADocumentRef document, CAFrameRef frame,
						RgnHandle clipRgn, CATransform xform, GrafPtr port );
pascal	void		CAChangeVisible( CAVisFrame visFrame, RgnHandle clipRgn,
						CATransform xform );
pascal	void		CAMakeInvisible( CAVisFrame visFrame );

pascal	Boolean		CAIsFrozen( CADocumentRef document, CAFrameRef frame );
pascal	void		CASetFrozen( CADocumentRef document, CAFrameRef frame,
						Boolean isFrozen );

pascal	void		CASetSelected( CAVisFrame visFrame, Boolean isSelected );
pascal	void		CASetHighlight( CAVisFrame visFrame, CAHighlight highlight );

//-------------------------------------------------------------------------
// Imaging
//-------------------------------------------------------------------------

pascal	GrafPtr		CAGetCanvas( CAVisFrame visFrame );

pascal	void		CADrawFrame( CAVisFrame visFrame, RgnHandle invalidRgn );

// callbacks

typedef 	void (* CACanvasUpdatedProc) ( CAFocusType );

pascal	void		CAInstallCanvasNotification( CACanvasUpdatedProc );

pascal	void		CACanvasUpdated( GrafPtr port );

//-------------------------------------------------------------------------
// Menu
//-------------------------------------------------------------------------

pascal	void		CASetBaseMenuBar( void );
pascal	void		CAAdjustMenus( void );
pascal	void		CARegisterCommand( CACommandID command, CAMenuID menu,
						CAMenuItemID menuItem );
pascal	Boolean		CADispatchMenuEvent( EventRecord* event, long menuSelection );

//-------------------------------------------------------------------------
// UI Events
//-------------------------------------------------------------------------

pascal	Boolean		CADispatchEvent( EventRecord* event );
pascal	RgnHandle	CAGetMouseRegion( void );
pascal	long		CAGetSleepTime( void );

// callbacks

typedef 	void (* CAAdjustBorderProc) ( CADocumentRef, CAFrameRef, RgnHandle );

pascal	void		CAInstallBorderAdjuster( CAAdjustBorderProc );

pascal	void		CAAdjustActiveBorder( CADocumentRef document, CAFrameRef frame, RgnHandle border );

//-------------------------------------------------------------------------
// Undo
//-------------------------------------------------------------------------

pascal	void		CAClearUndo( void );

//-------------------------------------------------------------------------
// Proxy Root Part
//-------------------------------------------------------------------------

// Does this really belong here?
// I think the Proxy Root Part is invisible to the Container Application.
// -Tantek


#ifdef __cplusplus
}
#endif



#endif // _CALIB_