/* layout types.h -- Line Layout types

	by Dave Opstad and Eric Mader,
	with special thanks to the following folks:
		 Mark Davis, for starting it all off;
		 Russ Sonnenschein, for many good ideas;
		 Cary Clark, Dave Good, Rob Johnson, Keith McGreggor, and Chris Yerga for the graphics system;
		 Richard Becker, Lynn Bekkala and Mike Reed for fonts and the font system;
		 Robert Dierkes and the SQE team for testing and good ideas;
		 Parry Kejriwal and Richard Klein, for font tools;
		 Diane Patterson, for great patience in writing documentation;
		 Hugo Ayala, Tom Dowdy, Dan Lipton, Sean Parent and the rest of the PrintShop;
		 Mitch Allen for managing to manage us; and
		 many other good folk at Apple!
*/

/* Copyright 1988-1994 Apple Computer, Inc.  All rights reserved. */

#pragma once

#ifndef layoutTypesIncludes
#define layoutTypesIncludes

#ifndef mathTypesIncludes
#include "math types.h"
#endif

#ifndef graphicsTypesIncludes
#include "graphics types.h"
#endif

typedef unsigned short gxGlyphcode;	/* single glyph in a font */
typedef long gxByteOffset;					/* byte offset within backing store */

/* The actual constants for feature types and selectors have been moved to a library. */

typedef unsigned short gxRunFeatureType;
typedef unsigned short gxRunFeatureSelector;

/* If tracking is not desired, specify the following value in the track field in the
		gxRunControls record (note that a track of 0 does *not* mean to turn tracking off;
		rather, it means to use normal tracking). */

#define gxNoTracking gxNegativeInfinity

/* The special "gxNoStake" value is returned by the GXGetLayoutBreakOffset call to
		indicate the absence of a character offset that is stable with respect to
		metamorphosis and contextual kerning. */

#define gxNoStake -1

/* A glyph's behavior with respect to other glyphs on its line is defined in part by its
		gxBaselineType. These types allow correct alignment of the baselines of all glyphs on
		the line. */

enum gxBaselineTypes {
	gxRomanBaseline = 0,
	gxIdeographicCenterBaseline,
	gxIdeographicLowBaseline,
	gxHangingBaseline,
	gxMathBaseline,
	gxLastBaseline = 31,
	gxNumberOfBaselineTypes = gxLastBaseline + 1,
	gxNoOverrideBaseline = 255
};
typedef unsigned long gxBaselineType;

typedef Fixed gxBaselineDeltas[gxNumberOfBaselineTypes];

/* gxJustificationPriority defines the point during the justification process at which a
	glyph will begin to receive deltas before and after itself. */

enum gxJustificationPriorities {
	gxKashidaPriority							= 0,
	gxWhiteSpacePriority					= 1,
	gxInterCharPriority						= 2,
	gxNullJustificationPriority		= 3,
	gxNumberOfJustificationPriorities
};
typedef unsigned char gxJustificationPriority;

/* gxJustificationFlags are used to control which fields of a gxWidthDeltaRecord are to
		be overridden and which are not if a gxPriorityJustificationOverride or
		gxGlyphJustificationOverride (qq.v.) is specified. */

#define gxOverridePriority					0x8000	/* use priority value from override */
#define gxOverrideLimits						0x4000	/* use limits values from override */
#define gxOverrideUnlimited					0x2000	/* use unlimited flag from override */
#define gxUnlimitedGapAbsorption		0x1000 	/* glyph can take unlimited gap */
#define gxJustificationPriorityMask	0x000F 	/* justification priority */

#define gxAllJustificationFlags (gxOverridePriority | gxOverrideLimits | \
	gxOverrideUnlimited | gxUnlimitedGapAbsorption | gxJustificationPriorityMask)

typedef unsigned short gxJustificationFlags;

/* The directional behavior of a glyph can be overridden using a gxDirectionOverride. */

enum gxDirectionOverrides {
	gxNoDirectionOverride	= 0,
	gxImposeLeftToRight		= 1,
	gxImposeRightToLeft		= 2,
	gxImposeArabic				= 3
};
typedef unsigned short gxDirectionOverride;

/* gxRunControlFlags describe the nonparametric layout options contained in a gxStyle. */

#define gxNoLigatureSplits				0x80000000
#define gxNoCaretAngle						0x40000000
#define gxImposeWidth							0x20000000
#define gxNoCrossKerning					0x10000000
#define gxNoOpticalAlignment			0x08000000
#define gxForceHanging						0x04000000
#define gxNoSpecialJustification	0x02000000
#define gxDirectionOverrideMask		0x00000003

#define gxAllRunControlFlags (gxNoLigatureSplits|gxNoCaretAngle|gxImposeWidth|gxNoCrossKerning|gxNoOpticalAlignment|gxForceHanging|gxNoSpecialJustification|gxDirectionOverrideMask)

typedef unsigned long gxRunControlFlags;

/* gxHighlightType is used to distinguish various highlighting methods, both in terms of
		character offset based vs. visual based, and in terms of angled sides vs. non-angled
		sides. */

enum gxHighlightTypes {
	gxHighlightStraight			= 0,	/* straight-edged simple highlighting */
	gxHighlightAverageAngle	= 1		/* takes average of two edge angles */
};
typedef unsigned long gxHighlightType;

/* gxCaretType is used to control whether the caret that is returned from GXGetLayoutCaret
		is a split caret or a (keyboard-syncronized) single caret. */

enum gxCaretTypes {
	gxSplitCaretType					= 0,	/* returns Mac-style split caret (default) */
	gxLeftRightKeyboardCaret	= 1,	/* single caret in left-right position */
	gxRightLeftKeyboardCaret	= 2		/* single caret in right-left position */
};
typedef unsigned long gxCaretType;

/* gxLayoutOffsetState describes the characteristics of a given gxByteOffset in some
		layout. It is returned by the GXGetOffsetGlyphs call. Note that the
		gxOffsetInsideLigature value is returned in addition to the 8/16 (or invalid)
		indication. */

enum gxLayoutOffsetStates {
	gxOffset8_8			= 0,
	gxOffset8_16		= 1,
	gxOffset16_8		= 2,
	gxOffset16_16		= 3,
	gxOffsetInvalid	= 4
};

#define gxOffsetInsideLigature 0x8000

typedef unsigned short gxLayoutOffsetState;

/* gxLayoutOptionsFlags are single-bit flags contained in a gxLayoutOptions record. We
		also define here some utility constants that are useful in setting various fields in
		the gxLayoutOptions record. */

#define gxNoLayoutOptions 0

#define gxLineIsDisplayOnly 0x00000001

#define gxAllLayoutOptionsFlags (gxLineIsDisplayOnly)

#define gxMaxRunLevel 15
#define gxFlushLeft 0
#define gxFlushCenter (fract1/2)
#define gxFlushRight fract1

#define gxNoJustification 0
#define gxFullJustification fract1

typedef unsigned long gxLayoutOptionsFlags;

/* A gxRunFeature describes a feature and a level for that feature. */

struct gxRunFeature {
	gxRunFeatureType			featureType;
	gxRunFeatureSelector	featureSelector;
};

/* A gxWidthDeltaRecord contains all of the information needed to describe the behavior of one
		class of glyphs during the justification process. */

struct gxWidthDeltaRecord {
	Fixed									beforeGrowLimit;		/* ems AW can grow by at most on LT */
	Fixed									beforeShrinkLimit;	/* ems AW can shrink by at most on LT */
	Fixed									afterGrowLimit;			/* ems AW can grow by at most on RB */
	Fixed									afterShrinkLimit;		/* ems AW can shrink by at most on RB */
	gxJustificationFlags	growFlags;					/* flags controlling grow case */
	gxJustificationFlags	shrinkFlags;				/* flags controlling shrink case */
};

/* A gxPriorityJustificationOverride contains an array of WidthDeltaRecords, one for each
		gxJustificationPriority. */

struct gxPriorityJustificationOverride {
	struct gxWidthDeltaRecord deltas[gxNumberOfJustificationPriorities]; /* overrides for each of the priorities */
};

/* A gxGlyphJustificationOverride contains a gxWidthDeltaRecord that is to be used for a
		specific glyph in a specific run (this limitation is because glyphcodes vary by font). */

struct gxGlyphJustificationOverride {
	gxGlyphcode								glyph;
	struct gxWidthDeltaRecord	override;
};

/* gxRunControls contains flags, shifts, imposed widths and overrides for a run. */
/* NOTE: a value of "gxNoTracking" (see above) in track disables tracking */

struct gxRunControls {
	gxRunControlFlags	flags;
	Fixed							beforeWithStreamShift;
	Fixed							afterWithStreamShift;
	Fixed							crossStreamShift;
	Fixed							imposedWidth;
	Fixed							track;
	fract							hangingInhibitFactor;
	fract							kerningInhibitFactor;
	Fixed							decompositionAdjustmentFactor;
	gxBaselineType		baselineType;
};

/* A gxGlyphSubstitution describes one client-provided substitution that occurs after all
		other automatic glyph changes have happened. */

struct gxGlyphSubstitution {
	gxGlyphcode 	originalGlyph;		/* Whenever you see this glyph... */
	gxGlyphcode 	substituteGlyph;	/* ...change it to this one. */
};

/* gxKerningAdjustmentFactors specify an adjustment to automatic kerning. The adjustment
		is ax + b where x is the automatic kerning value, a is scaleFactor, and b is
		adjustmentPointSizeFactor times the run's point size. */
	
struct gxKerningAdjustmentFactors {
	fract 	scaleFactor;
	Fixed 	adjustmentPointSizeFactor;
};

/* A gxKerningAdjustment identifies with- and cross-stream kerning adjustments
		for specific glyph pairs. */

struct gxKerningAdjustment {
	gxGlyphcode												firstGlyph;
	gxGlyphcode												secondGlyph;
	struct gxKerningAdjustmentFactors	withStreamFactors;
	struct gxKerningAdjustmentFactors	crossStreamFactors;
};

/* A value of gxResetCrossStreamFactor in crossStreamFactors.adjustmentPointSizeFactor
		will reset the cross-stream kerning to the baseline. */
	
#define gxResetCrossStreamFactor gxNegativeInfinity

/* gxLayoutHitInfo contains the output from the GXHitTestLayout call. */

struct gxLayoutHitInfo {
	Fixed							firstPartialDist;
	Fixed							lastPartialDist;
	gxByteOffset 			hitSideOffset;
	gxByteOffset 			nonHitSideOffset;
	boolean						leadingEdge;
	boolean						inLoose;
};

/* A gxLineBaselineRecord contains the deltas from 0 to all the different baselines for
		the layout. It can be filled via a call to GetBaselineDeltas (q.v.). */

struct gxLineBaselineRecord {
	gxBaselineDeltas	deltas;
};

/* The gxLayoutOptions type contains information about the layout characteristics of the
		whole line. */

struct gxLayoutOptions {
	Fixed												width;
	fract												flush;
	fract												just;
	gxLayoutOptionsFlags				flags;
	struct gxLineBaselineRecord	*baselineRec;
};

#ifndef __cplusplus
typedef struct gxRunFeature gxRunFeature;
typedef struct gxWidthDeltaRecord gxWidthDeltaRecord;
typedef struct gxPriorityJustificationOverride gxPriorityJustificationOverride;
typedef struct gxGlyphJustificationOverride gxGlyphJustificationOverride;
typedef struct gxRunControls gxRunControls;
typedef struct gxGlyphSubstitution gxGlyphSubstitution;
typedef struct gxKerningAdjustmentFactors gxKerningAdjustmentFactors;
typedef struct gxKerningAdjustment gxKerningAdjustment;
typedef struct gxLayoutHitInfo gxLayoutHitInfo;
typedef struct gxLineBaselineRecord gxLineBaselineRecord;
typedef struct gxLayoutOptions gxLayoutOptions;
#endif	/* __cplusplus */

#endif	/* layoutTypesIncludes */
